wsAndroid = {
    service: 'brasil',  // This gets modified by the Grunt tasks
    version: '1.5.3',  // This gets modified by the Grunt tasks
    remoteConfigRoot:  'http://wsapps.bbc.co.uk/worldservice/apps/2',
    jsonExt:           '.app.esi',
    mobileRedirect:    '/mobile',
    debug:             true, // Dumps lots of messages to console
    enableOffline:     true,
    nativeVideoPlayer: true, // Use native video player to force landscape mode?
    models:            {},
    collections:       {},
    views:             {},
    latestNews: {
        numOfItems: 10
    },
    mirrors: {
        enabled:    false,
        interval:   90, // seconds
		list:       ["www.2000acresofsky.com",
                    "www.2pints.co.uk",
                    "www.changingrooms.biz",
                    "www.crinkleybottom.co.uk",
                    "www.digitalvideofootage.com",
                    "www.free2air.co.uk",
                    "www.free2view.org.uk",
                    "www.openviewlicensing.com",
                    "www.peopleandpower.com",
                    "www.stylechallenge.co.uk"]
    },
	currentGenre:	   null,
	currentHomeTab:	   null,
    env: {
        status:      "online",
        orientation: "portrait",
        network:     {},
        retina:      window.devicePixelRatio >= 2 ? true : false,
        width:       $(window).width(),
        height:      $(window).height(),
        allowRotate: true, // This is used to temporarily allow rotation while videos are playing (doesn't apply to native player)
        systemBar:	false // This is set on the first orientation change
    },
    dialectSwap: false,
    router: Backbone.Router.extend({
        routes: {
          "":                     "main",
          "story/:id":            "storyById",
          "story/url/*path":      "storyByUrl",
          "institutional/:page":  "institutional"
        },

        initialize: function() {
            ws.views._activeMainView   = new ws.views.MainView();
            ws.views._activeStoryView  = new ws.views.StoryView();
        },

        main: function() {
            $(ws.views._activeStoryView.el).find('video').remove(); // Remove any videos playing in the story view
            ws.views.switchTo(ws.views._activeMainView.el, true);
        },

        institutional: function(page) {
            this.storyByUrl(ws.config.pages[page], { disableSharing: true, disableNavigation: true });
        },

        storyByUrl: function(url, options) {;
            // This method loads any page by URL - provided that a JSON feed exists
            if (!options) { options = {}; }
            var storyModel = new ws.models.Story();

            ws.views._activeStoryView.options = options;
			ws.views._activeStoryView.options.referrer = 'byUrl';
            ws.views._activeStoryView.model   = storyModel;

            storyModel.bind('change', function() {
                ws.views._activeStoryView.render();

                try{
                	// Do metrics
                	var contentFlavor = "STORY";
                	_.each(model.get('media'), function(val,i) {
                		if (val.type == 'audio') {
                			contentFlavor = "AUDIO";
                		} else if (val.type == 'video') {
                			contentFlavor = "VIDEO";
                		}

                		ws.metrics.track("storypage", {
                			"pageName":         storyModel.get('title'),
                			"contentId":        storyModel.get('bbcws_id'),
                			"url":              storyModel.get('href'),
                			"contentFlavor":    "ws::"+contentFlavor
                		});
                	});
                } catch (err) {}

            });


            storyModel.getFullContent(url, {
                error: function() {
                    // If getting the feed failed (usually because it's a very old story,
                    // launch the Child Browser and go to the mobile version of the page

                    var mobileUrl = url.replace(ws.config.serviceRoot, ws.config.serviceRoot+ws.mobileRedirect);

                    // if simp or trad chinese service, build the mobile url using regex instead
                    if (url.match('/simp/') || url.match('/trad/')) {
                        //test for /simp/, /trad/
                        var r = new RegExp(/\/simp\/|\/trad\//);
                        var s = r.exec(url);
                        mobileUrl = url.replace(s, s+ws.mobileRedirect + '/');
                    }

                    //ws.log(mobileUrl);
                    window.plugins.childBrowser.showWebPage(mobileUrl);

                    // Take the main window back by one
                    // The delay is only to hide the 'jump back' behind the Child Browser (once it has loaded)
                    _.delay(function() { history.back(); }, 1000);
                }
            });

            ws.views._activeStoryView.render();
            ws.views.switchTo(ws.views._activeStoryView.el);
        },

        storyById: function(id) {
            $(ws.views._activeStoryView.el).find('video').remove(); // Remove any videos playing in the story view

            if (ws.views._activeStoryView.doPageTransition) { ws.views.switchTo(ws.views._activeStoryView.el); }

            var collection = ws.views._activeStoryView.collection;
            if (collection) {
                model = collection.get(id) || collection.getByCid(id) || ws.stories.get(id);

                //model.getFullContent();

                $.publish('/feeds/downloadstarted');
                model.getFullContent(model.attributes.href, {
                    success: function() {
                        $.publish('/feeds/downloadcomplete');
                    },
                    error: function() {
                        $.publish('/feeds/downloadcomplete');
                    }
                });

                if (model.isInLocalStore == true) {$.publish('/feeds/downloadcomplete');}

                ws.views._activeStoryView.options = {};
				ws.views._activeStoryView.options.referrer = 'byId';
                ws.views._activeStoryView.model   = model;

                // Set the change event here, in case the story hasn't loaded yet
                // Once story loads, the view is refreshed
                ws.views._activeStoryView.model.bind('change', ws.views._activeStoryView.render);
                ws.views._activeStoryView.render();

                ws.views._activeStoryView.doPageTransition = true;

                // Do metrics
                var contentFlavor = "STORY";
                _.each(model.get('media'), function(val,i) {
                    if (val.type == 'audio') {
                        contentFlavor = "AUDIO";
                    } else if (val.type == 'video') {
                        contentFlavor = "VIDEO";
                    }
                });

                ws.metrics.track("storypage", {
                    "pageName":  model.get('title'),
                    "contentId": model.get('bbcws_id'),
                    "url":       model.get('href'),
                    "contentFlavor":    "ws::"+contentFlavor
                });

            }

        }
    }),

    init: function() {
        $.publish("/app/init");
        // Start the views
        ws.views.init();

        //add lang
        if (typeof ws.config.lang !== "undefined" && ws.config.lang !== "") {
            $('html').attr('lang', ws.config.lang);
        }

        // check for localised extensions
        if (ws.config.jsonExt) {
            ws.jsonExt = ws.config.jsonExt;
        }

        // check for localised extensions
        if (ws.config.mobileRedirect) {
            ws.mobileRedirect = ws.config.mobileRedirect;
        }

        // check for latestitems limit
        if (ws.config.latestNews) {
            ws.latestNews.numOfItems = ws.config.latestNews.numOfItems;
        }

        // Load child browser
        ChildBrowser.prototype.install();//ChildBrowser.prototype.install();

        // Load date vocabs
        Date.replaceChars.longMonths    = ws.config.vocab.longMonths;
        Date.replaceChars.longDays      = ws.config.vocab.longDays;
        jQuery.timeago.settings.strings = ws.config.vocab.timeAgo;

        ws.connection.check(); // Check and update connection status

        // Check remote config file, and overrides the local one
        ws.checkRemoteConfig();

        // Create CSS ready node to check if stylesheets have loaded
        ws.cssReadyNode = $('body').append('<div id="css-ready"></div>');

        // Listen for PhoneGap events
        document.addEventListener("online", function() {
            // Going online
            ws.env.status = "online";
            $.publish("/status/online");
            ws.connection.check();
			ws.checkRemoteConfig();
            ws.metrics.processQueue();
        }, false);

        document.addEventListener("offline", function() {
            // Going offline
            ws.env.status = "offline";
            $.publish("/status/offline");
            ws.connection.check();
			ws.env.remoteConfig = { "appEnabled": true };
        }, false);

        document.addEventListener("resume", function() {
            // When app is resumed from the background
            $.publish("/status/resume");
            ws.connection.check();
			ws.env.remoteConfig = undefined;
            ws.checkRemoteConfig();
			if (_.isEmpty(ws.store('storyIndex'))) {
				ws.fetchInitialContent();
			}

            ws.metrics.processQueue();
        }, false);

        var supportsOrientationChange = "onorientationchange" in window,
        orientationEvent = supportsOrientationChange ? "orientationchange" : "resize";

        window.addEventListener(orientationEvent, function() {
        	//delay to allow the switch and get updated dom
        	_.delay(
        			function () {
        				var h = $('body').height();

		        	// reset heights and refresh scroller
		        	$.each(ws.views.scrollers._list, function(i, val) {
		        		var node = $('#'+ val.contId),
		        			cont = node.find('.ws-content'),
		        			footer = (node.siblings('.ws-footer').length) ? node.siblings('.ws-footer') : cont.find('.ws-footer'),
		        			header = node.siblings('.ws-header'),
			        	 	scrollWrapperHeight = h - (header.height());
			        	// if the footer has been moved to the main id tag,
		        		//it needs to be returned to the scrolling area
		    			if (cont.find('.ws-footer').length == 0) {
			        		footer.detach();
			        		$('#footer-placeholder').remove();
			        		cont.append(footer);
			        		footer = cont.find('.ws-footer');
			        	}
			    	 	node.height(scrollWrapperHeight).css('overflow', 'hidden');

			    	 	//refresh scroller
		        		//val.refresh(false);
			    	 	//ws.views.scrollers.refresh();
			    	 	val.refresh();
		        	});
        	}, 200);
        }, false);

     // Back Button
    	document.addEventListener("backbutton", function () {
    		try{
    			if ($('#main.active').is(':visible')){
    				if($('#main #ws-settings').length == 0 && $('#main #ws-categories').length == 0 && $('#ws-popular').length == 0 && $('#main #ws-categorised-list').length == 0) {
    					//exit app if not on one of the non-home home screens
    					navigator.app.exitApp();
    				}
    			}
    				if ($('#overlay.visible').is(':visible') || $('.overlay.visible').is(':visible')) {
        				//remove overlay with its own methods
        				if($('#overlay.visible .cancel').length) $('#overlay .cancel').trigger('click');
        				if ($('.overlay.visible .button button').length) $('.overlay .button button').trigger('click');
        				return;
    				}
					if ($('#picture-gallery').hasClass('active')) {
						ws.views._activeStoryView.pictureGalleryView.returnToStory();
					}
    				if (ws.currentGenre == 'homepage') {
        				ws.currentGenre = null;
        			}
        			ws.thisRouter.navigate('', true);

    				//hack until I can work out how to do this properly
    				$('#ws-tab-bar .ws-home').trigger('click');
        			try {
                    	$.each(ws.views.scrollers._list, function(i, val) {
                    		val.refresh(true);
                    	});
                    } catch (err) {}

    		} catch (err) {
    			// @TODO: potentially report this issue
    		}

        }, false);

        // Search button
        document.addEventListener("searchbutton", function () { return }, false);

        // Menu button
        document.addEventListener("menubutton", function () {
        	try {
        		ws.views.MainView.viewToRender = new ws.views.settingsView();
        		$('#ws-banner').find('span').text('');
        		ws.views.renderLastUpdated();
        		ws.views._activeMainView.render();

        	} catch (err) {}
        }, false);

        // Prevent window scroll
        $(window).bind("scroll", function (e) {
                $(window).scrollTop(0);
        });
        // Load storyIndex from localStorage
        // This keeps track of the editorial order and which genre index they should appear in
        ws.storyIndex = ws.store('storyIndex');

        // Check if genre/category order has been set, and if not, set default
        ws.genreOrder = ws.store('genreOrder');
        if (ws.genreOrder == null || ws.genreOrder === {}) {
            var finalList = {};
            _.each(ws.config.availableGenres, function(genre, genreId) {
                if (genre.enabled == true) {
                    finalList[genreId] = genre.selected;
                }
            });
            ws.genreOrder = finalList;
            ws.store('genreOrder', finalList);
        } else {

            // Check if new genres have been removed/added
            var availableGenreIds = _.keys(ws.config.availableGenres),
                orderedGenreIds   = _.keys(ws.genreOrder);


            // Check for genres which have been added to the config
            _.each(ws.config.availableGenres, function(genre, genreId) {
                if (_.indexOf(orderedGenreIds, genreId) == -1) {
                    ws.genreOrder[genreId] = genre.selected;
                }

                if (genre.enabled == false) {
                    delete ws.genreOrder[genreId];
                }
            });

            // Check for genres which have been removed from the config
            _.each(ws.genreOrder, function(genre, genreId) {
                if (_.indexOf(availableGenreIds, genreId) == -1) {
                    delete ws.genreOrder[genreId];
                }

            });
            ws.store('genreOrder', ws.genreOrder);

        }

        // check if default home tab has been set
		if (!ws.prefs('defaultHomeTab')) {
			ws.prefs('defaultHomeTab', ws.config.defaultHomeTab);
		}
		// check if default stats has been set
        if (!ws.prefs('shareStats')>=0) {
        	ws.prefs('shareStats', 1);
        }
		// check if default dialect has been set
		if (ws.prefs('defaultDialect') && ws.config.dialects) {
            if (!ws.service.match(ws.prefs('defaultDialect'))) {
                ws.loadNewService(ws.service.substring(0, ws.service.indexOf('-') + 1) + ws.prefs('defaultDialect'), 'init');
            }
        }

        // Set up collections for most popular stories
        ws.popularStories = {
            read:    new ws.models.mostPopular(),
            watched: new ws.models.mostWatched()
        }

        // Initial load of stories
        var storedStories = ws.store('stories') || {};
        //if (ws.env.status == "offline" && !_.isEmpty(storedStories)) {
        if (!_.isEmpty(storedStories)) {
			//ws.log('get stories from storage');

            ws.stories = new ws.models.StorySet(storedStories);
            ws.startApp();
        } else {
            //ws.log('get stories from online');
			var homepageUrl = ws.config.serviceRoot+'/index'+ws.jsonExt;
            //homepageUrl = 'tests/json/homepage_gallery_russian.app.json';

            if (ws.debug) { ws.log('[FETCHING] Getting index: '+ homepageUrl); }
            ws.stories = new ws.models.StorySet({ url: homepageUrl });
            ws.fetchInitialContent();
        }


    },

    fetchInitialContent: function() {

    	if (ws.env.remoteConfig) {

    		if (ws.env.remoteConfig.appEnabled == false) {return false;}

	        ws.stories.fetch({
	            success: function() {

                    // check for extra dialects and show an alert on first run of the app, notifying users of the option to switch in the settings
                    if (ws.config.dialects && !ws.prefs('defaultDialect')) {

                        _.each(ws.config.dialects, function(dialect, dialectId) {
                            if (ws.service.match(dialectId)) {
                                ws.prefs('defaultDialect', dialectId);
                            }
                        });

                        // only use the alert on mobile devices
                        if (navigator.network) {
                            navigator.notification.alert (
                                ws.config.vocab.changeDialectAlertTitle,
                                function(){
                                    ws.startApp();
                                },
                                ws.config.vocab.changeDialectAlertDesc,
                                ws.config.vocab.changeDialectAlertConfirm
                            );
                        } else {
                            ws.startApp();
                        }

                    } else {
                        ws.startApp();
                    }

	            }, error: function() {
					navigator.notification.alert(
						ws.config.vocab.connectionAlertMsg,
						function(){},
						ws.config.vocab.connectionAlertTitle,
						ws.config.vocab.connectionAlertBtn
					);
				}
	        });

        } else {
        	 _.delay(ws.fetchInitialContent, 100);
        }

    },

    checkRemoteConfig: function() {
        $.ajax({
            url:      ws.remoteConfigRoot+'/config.json',
            dataType: "json",
            useCache: true,
            success:  function(data) {
                ws.env.remoteConfig = data;
				if (ws.env.remoteConfig.appEnabled == false) {
					$.publish("/status/disabled");
				} else {
					$.publish("/status/enabled");
				}
            }, error: function() {
				if (ws.env.status == 'offline') {
					ws.env.remoteConfig = { "appEnabled": true };
					$.publish("/status/enabled");
				}
			}
        });

    },

    startApp: function() {
        // This should only be called once the initial data bootstrap
        // has been done and BackBone has enough data to go with
        if ($('#css-ready').css('z-index') == 1234 && ws.env.remoteConfig) {
            ws.thisRouter = new ws.router();
            try {
                Backbone.history.start();
            }catch(e) {
                //console.log(e);
            }

            ws.thisRouter.navigate('', true); // Always start on homepage at first load

            ws.stories.fetchAllStories({ skipHomepage: false }); // Load the rest of the genres (apart from the homepage)
        } else {
            //$('body').append('<p>'+$('#css-ready').css('z-index')+'</p>');
            _.delay(ws.startApp, 100);
        }

    }
};

// merge the objects
$.extend(ws,wsAndroid);
