ws = {
    service: 'brasil',  // This gets modified by the Grunt tasks
    version: '1.5.3',  // This gets modified by the Grunt tasks
    remoteConfigRoot:  'http://wsapps.bbc.co.uk/worldservice/apps/2',
    jsonExt:           '.app.esi',
    mobileRedirect:    '/mobile',
    debug:             true, // Dumps lots of messages to console
    enableOffline:     true,
    nativeVideoPlayer: true, // Use native video player to force landscape mode?
    models:            {},
    collections:       {},
    views:             {},
    latestNews: {
        numOfItems: 10
    },
    mirrors: {
        enabled:    false,
        interval:   90, // seconds
		list:       ["www.amidodgyornot.co.uk",
                    "www.inmwt.com",
                    "www.timetrumpet.com",
                    "www.sundaylovesongs.co.uk"]
    },
	currentGenre:	   null,
	currentHomeTab:	   null,
    env: {
        status:      "online",
        orientation: "portrait",
        network:     {},
        retina:      window.devicePixelRatio >= 2 ? true : false,
        width:       $(window).width(),
        height:      $(window).height(),
        allowRotate: false // This is used to temporarily allow rotation while videos are playing (doesn't apply to native player)
    },
    router: Backbone.Router.extend({
        routes: {
          "":                     "main",
          "story/:id":            "storyById",
          "story/url/*path":      "storyByUrl",
          "institutional/:page":  "institutional"
        },

        initialize: function() {
            ws.views._activeMainView   = new ws.views.MainView();
            ws.views._activeStoryView  = new ws.views.StoryView();
        },

        main: function() {
            $(ws.views._activeStoryView.el).find('video').remove(); // Remove any videos playing in the story view
            ws.views.switchTo(ws.views._activeMainView.el, true);
        },

        institutional: function(page) {
            this.storyByUrl(ws.config.pages[page], { disableSharing: true, disableNavigation: true });
        },

        storyByUrl: function(url, options) {;
            // This method loads any page by URL - provided that a JSON feed exists
            if (!options) { options = {}; }
            var storyModel = new ws.models.Story();

            ws.views._activeStoryView.options = options;
			ws.views._activeStoryView.options.referrer = 'byUrl';
            ws.views._activeStoryView.model   = storyModel;

            storyModel.bind('change', function() {
                ws.views._activeStoryView.render();

                // Do metrics
                var contentFlavor = "STORY";
                _.each(model.get('media'), function(val,i) {
                    if (val.type == 'audio') {
                        contentFlavor = "AUDIO";
                    } else if (val.type == 'video') {
                        contentFlavor = "VIDEO";
                    }
                });

                ws.metrics.track("storypage", {
                    "pageName":         storyModel.get('title'),
                    "contentId":        storyModel.get('bbcws_id'),
                    "url":              storyModel.get('href'),
                    "contentFlavor":    "ws::"+contentFlavor
                });
            });


            storyModel.getFullContent(url, {
                error: function() {
                    // If getting the feed failed (usually because it's a very old story,
                    // launch the Child Browser and go to the mobile version of the page

                    var mobileUrl = url.replace(ws.config.serviceRoot, ws.config.serviceRoot+ws.mobileRedirect);

                    // if simp or trad chinese service, build the mobile url using regex instead
                    if (url.match('/simp/') || url.match('/trad/')) {
                        //test for /simp/, /trad/
                        var r = new RegExp(/\/simp\/|\/trad\//);
                        var s = r.exec(url);
                        mobileUrl = url.replace(s, s+ws.mobileRedirect + '/');
                    }

                    //ws.log(mobileUrl);
                    //window.plugins.childBrowser.showWebPage(mobileUrl);
                    var ref = window.open(mobileUrl, '_blank', 'toolbarposition=top');

                    // Take the main window back by one
                    // The delay is only to hide the 'jump back' behind the Child Browser (once it has loaded)
                    _.delay(function() { history.back(); }, 1000);
                }
            });

            ws.views._activeStoryView.render();
            ws.views.switchTo(ws.views._activeStoryView.el);
        },

        storyById: function(id) {
            $(ws.views._activeStoryView.el).find('video').remove(); // Remove any videos playing in the story view

            if (ws.views._activeStoryView.doPageTransition) { ws.views.switchTo(ws.views._activeStoryView.el); }

            var collection = ws.views._activeStoryView.collection;

            if (collection) {
                model = collection.get(id) || collection.getByCid(id) || ws.stories.get(id);

                //model.getFullContent();

                $.publish('/feeds/downloadstarted');
                model.getFullContent(model.attributes.href, {
                    success: function() {
                        $.publish('/feeds/downloadcomplete');
                    },
                    error: function() {
                        $.publish('/feeds/downloadcomplete');
                    }
                });

                if (model.isInLocalStore == true) {$.publish('/feeds/downloadcomplete');}

                ws.views._activeStoryView.options = {};
				ws.views._activeStoryView.options.referrer = 'byId';
                ws.views._activeStoryView.model   = model;

                // Set the change event here, in case the story hasn't loaded yet
                // Once story loads, the view is refreshed
                ws.views._activeStoryView.model.bind('change', ws.views._activeStoryView.render);
                ws.views._activeStoryView.render();

                ws.views._activeStoryView.doPageTransition = true;

                // Do metrics
                var contentFlavor = "STORY";
                _.each(model.get('media'), function(val,i) {
                    if (val.type == 'audio') {
                        contentFlavor = "AUDIO";
                    } else if (val.type == 'video') {
                        contentFlavor = "VIDEO";
                    }
                });

                ws.metrics.track("storypage", {
                    "pageName":  model.get('title'),
                    "contentId": model.get('bbcws_id'),
                    "url":       model.get('href'),
                    "contentFlavor":    "ws::"+contentFlavor
                });

            }

        }
    }),

    init: function() {
        $.publish("/app/init");

        // Start the views
        ws.views.init();

        // check for localised extensions
        if (ws.config.jsonExt) {
            ws.jsonExt = ws.config.jsonExt;
        }

        // check for localised extensions
        if (ws.config.mobileRedirect) {
            ws.mobileRedirect = ws.config.mobileRedirect;
        }

        // check for latestitems limit
        if (ws.config.latestNews) {
            ws.latestNews.numOfItems = ws.config.latestNews.numOfItems;
        }

        // Load child browser
        ChildBrowser.install();

        // Load date vocabs
        Date.replaceChars.longMonths    = ws.config.vocab.longMonths;
        Date.replaceChars.longDays      = ws.config.vocab.longDays;
        jQuery.timeago.settings.strings = ws.config.vocab.timeAgo;

        ws.connection.check(); // Check and update connection status

        // Check remote config file, and overrides the local one
        ws.checkRemoteConfig();

        // Create CSS ready node to check if stylesheets have loaded
        ws.cssReadyNode = $('body').append('<div id="css-ready"></div>');

        // Listen for PhoneGap events
        document.addEventListener("online", function() {
            // Going online
            ws.env.status = "online";
            $.publish("/status/online");
            ws.connection.check();
			ws.checkRemoteConfig();
            ws.metrics.processQueue();
        }, false);

        document.addEventListener("offline", function() {
            // Going offline
            ws.env.status = "offline";
            $.publish("/status/offline");
            ws.connection.check();
			ws.env.remoteConfig = { "appEnabled": true };
        }, false);

        document.addEventListener("resume", function() {
            // When app is resumed from the background
            $.publish("/status/resume");
            //ws.stories.fetchAllStories();
            ws.connection.check();
			ws.env.remoteConfig = undefined;
            ws.checkRemoteConfig();
			if (_.isEmpty(ws.store('storyIndex'))) {
				ws.fetchInitialContent();
			}

            ws.metrics.processQueue();
        }, false);

        document.addEventListener("orientationchange", function() {

        });

        // Load storyIndex from localStorage
        // This keeps track of the editorial order and which genre index they should appear in
        ws.storyIndex = ws.store('storyIndex');

        // Check if genre/category order has been set, and if not, set default
        ws.genreOrder = ws.store('genreOrder');
        if (ws.genreOrder == null || ws.genreOrder === {}) {
            var finalList = {};
            _.each(ws.config.availableGenres, function(genre, genreId) {
                if (genre.enabled == true) {
                    finalList[genreId] = genre.selected;
                }
            });
            ws.genreOrder = finalList;
            ws.store('genreOrder', finalList);
        } else {

            // Check if new genres have been removed/added
            var availableGenreIds = _.keys(ws.config.availableGenres),
                orderedGenreIds   = _.keys(ws.genreOrder);


            // Check for genres which have been added to the config
            _.each(ws.config.availableGenres, function(genre, genreId) {
                if (_.indexOf(orderedGenreIds, genreId) == -1) {
                    ws.genreOrder[genreId] = genre.selected;
                }

                if (genre.enabled == false) {
                    delete ws.genreOrder[genreId];
                }
            });

           // Check for genres which have been removed from the config
            _.each(ws.genreOrder, function(genre, genreId) {
                if (_.indexOf(availableGenreIds, genreId) == -1) {
                    delete ws.genreOrder[genreId];
                }

            });
            ws.store('genreOrder', ws.genreOrder);

        }

		// check if default home tab has been set
		if (!ws.prefs('defaultHomeTab')) {
			ws.prefs('defaultHomeTab', ws.config.defaultHomeTab);
		}
		// check if default stats has been set
        if (!ws.prefs('shareStats')>=0) {
        	ws.prefs('shareStats', 1);
        }
		// check if default dialect has been set
		if (ws.prefs('defaultDialect') && ws.config.dialects) {
            if (!ws.service.match(ws.prefs('defaultDialect'))) {
                ws.loadNewService(ws.service.substring(0, ws.service.indexOf('-') + 1) + ws.prefs('defaultDialect'), 'init');
            }
        }

        // Set up collections for most popular stories
        ws.popularStories = {
            read:    new ws.models.mostPopular(),
            watched: new ws.models.mostWatched()
        }

        // Initial load of stories
        var storedStories = ws.store('stories') || {};
        //if (ws.env.status == "offline" && !_.isEmpty(storedStories)) {
        if (!_.isEmpty(storedStories)) {
			//ws.log('get stories from storage');

            ws.stories = new ws.models.StorySet(storedStories);
            ws.startApp();
        } else {
            //ws.log('get stories from online');
			var homepageUrl = ws.config.serviceRoot+'/index'+ws.jsonExt;
            //homepageUrl = 'tests/json/homepage_gallery_arabic.app.json';

            if (ws.debug) { ws.log('[FETCHING] Getting index: '+ homepageUrl); }
            ws.stories = new ws.models.StorySet({ url: homepageUrl });
            ws.fetchInitialContent();
        }



    },

    fetchInitialContent: function() {

    	if (ws.env.remoteConfig) {

    		if (ws.env.remoteConfig.appEnabled == false) {return false;}

	        ws.stories.fetch({
	            success: function() {

                    // check for extra dialects and show an alert on first run of the app, notifying users of the option to switch in the settings
                    if (ws.config.dialects && !ws.prefs('defaultDialect')) {

                        _.each(ws.config.dialects, function(dialect, dialectId) {
                            if (ws.service.match(dialectId)) {
                                ws.prefs('defaultDialect', dialectId);
                            }
                        });

                        // only use the alert on mobile devices
                        if (navigator.network) {
                            navigator.notification.alert (
                                ws.config.vocab.changeDialectAlertTitle,
                                function(){
                                    ws.startApp();
                                },
                                ws.config.vocab.changeDialectAlertDesc,
                                ws.config.vocab.changeDialectAlertConfirm
                            );
                        } else {
                            ws.startApp();
                        }

                    } else {
                        ws.startApp();
                    }

	            }, error: function() {
					navigator.notification.alert(
						ws.config.vocab.connectionAlertMsg,
						function(){},
						ws.config.vocab.connectionAlertTitle,
						ws.config.vocab.connectionAlertBtn
					);
				}
	        });

        } else {
        	 _.delay(ws.fetchInitialContent, 100);
        }

    },

    checkRemoteConfig: function() {
        $.ajax({
            url:      ws.remoteConfigRoot+'/config.json',
            dataType: "json",
            useCache: true,
            success:  function(data) {
                ws.env.remoteConfig = data;
				if (ws.env.remoteConfig.appEnabled == false) {
					$.publish("/status/disabled");
				} else {
					$.publish("/status/enabled");
				}
            }, error: function() {
				if (ws.env.status == 'offline') {
					ws.env.remoteConfig = { "appEnabled": true };
					$.publish("/status/enabled");
				}
			}
        });

    },

    startApp: function() {
        // This should only be called once the initial data bootstrap
        // has been done and BackBone has enough data to go with
        if ($('#css-ready').css('z-index') == 1234 && ws.env.remoteConfig) {
            ws.thisRouter = new ws.router();
            try {
                Backbone.history.start();
            }catch(e) {
                //console.log(e);
            }

            ws.thisRouter.navigate('', true); // Always start on homepage at first load

            ws.stories.fetchAllStories({ skipHomepage: false }); // Load the rest of the genres (apart from the homepage)
        } else {
            //$('body').append('<p>'+$('#css-ready').css('z-index')+'</p>');
            _.delay(ws.startApp, 100);
        }

    }
};


/* Preferences API */
/* This is mainly a convenience function to set/get preferences and persist them in localStorage */
ws.prefs = function(key, value) {
    var _prefsObject = ws.store('prefs') || {}

    if (key === undefined) {
        return _prefsObject;
    } else if (value === undefined) {
        // Read
        return _prefsObject[key];
    } else {
        // Write
        if (value === true)  { value = 1; } // Shorten booleans to save space
        if (value === false) { value = 0; }
        _prefsObject[key] = value;
        return ws.store('prefs', _prefsObject);
    }
}


/* Storage */
ws.store = function(key, value) {
    if (value === undefined) {
        // Read
        return JSON.parse(localStorage.getItem(key));
    } else {
        // Write
        try {

            localStorage.setItem(key, JSON.stringify(value));
            return true;

        } catch(err) {
            if((err.name).toUpperCase() == 'QUOTA_EXCEEDED_ERR'){
                $.publish("/store/full");
                //ws.log('localStorage quota exceeded');

                // Storage pruning should happen here
                ws.store.prune();

                return false;
            }
        }
    }
}
ws.store.prune = function() {
    // Pruning the storage removes any stories not in the storyIndex
	//ws.log('######### PRUNING ########');
    var storyIndex    = ws.store('storyIndex') || {},
        storyIndexIds = _.flatten(storyIndex);

	localStorage.removeItem('stories');

    _.forEach(_.keys(localStorage), function(val) {
        var thisId = val.match(/s(\d+)/) || '';
        //ws.log(val, thisId);
        if (!_.isUndefined(thisId[1])) {
            thisId = thisId[1];
            //ws.log(thisId);
            //if (_.indexOf(storyIndexIds, thisId) == -1) {
				//ws.log('##### PRUNING ' + thisId);
                localStorage.removeItem(val);
            //}
        }
    });
}
ws.store.getUsage = function() {
    return JSON.stringify(localStorage).length
}



/* Connection */
ws.connection = {
    check: function() {

	//hard code offline
	//	ws.env.status = "offline";$.publish("/status/offline"); return;


        if (!navigator.network) {
            // If Phonegap status is not available, try to do it
            // using the navigator.onLine way
            if (navigator.onLine) {
                ws.env.status = "online";
                $.publish("/status/online");
            } else {
                ws.env.status = "offline";
                $.publish("/status/offline");
            }
            return;
        }
        var networkState = navigator.network.connection.type;
        var states = {};

        states[Connection.UNKNOWN]  = "unknown";
        states[Connection.ETHERNET] = "ethernet";
        states[Connection.WIFI]     = "wifi";
        states[Connection.CELL_2G]  = "2g";
        states[Connection.CELL_3G]  = "3g";
        states[Connection.CELL_4G]  = "4g";
        states[Connection.CELL]     = "cell";
        states[Connection.NONE]     = "none";

        var networkType  = states[networkState] || "cell", // Fallback to 'cell' if nothing matches
            networkSpeed = networkType.match(/ethernet|wifi/) ? "fast" : "slow";

        // Check for changes, and if so, broadcast message
		if (ws.env.network != networkType) {
			if (networkType == 'none' || networkType == 'unknown') {
				ws.env.status = "offline";
				$.publish("/status/offline");
			} else {
				ws.env.status = "online";
				$.publish("/status/online");
			}
		}

        ws.env.network = {
            "type" : networkType,
            "speed": networkSpeed
        }

        //alert(ws.env.network.type);
        //console.log('hi', navigator.onLine, ws.env.status);
        return ws.env.network;
    }
}


/* Utilities */
ws.utils = {
    uid: function() {
        // Generates a unique sequental id for elements which need it
        if (!ws.utils.uid.counter) { ws.utils.uid.counter = 1; } else { ws.utils.uid.counter++; }
        return 'ws-gen-'+ws.utils.uid.counter;
    },
    truncate: function(text, len) {
        text = text.substring(0, len);
        //text = text.replace(/\w+$/, '') + ws.config.ellipses;
        text = text.replace(/\s+\S*$/, '') + ws.config.ellipses;
        return text;
    }
}

/* Logging */
ws.log = function(msg) {
    if (ws.debug) { console.log(msg, arguments); }
}


/* Vocab Utils */
ws.vocab = {
    numpf_ru: function(n, f, s, t) {
        // This handles Russian relative times
        // f - 1, 21, 31, ...
        // s - 2-4, 22-24, 32-34 ...
        // t - 5-20, 25-30, ...
        var n10 = n % 10;
        if ( (n10 == 1) && ( (n == 1) || (n > 20) ) ) {
            return f;
        } else if ( (n10 > 1) && (n10 < 5) && ( (n > 20) || (n < 10) ) ) {
            return s;
        } else {
            return t;
        }
    }
}

// reload the new configs to refresh the language, delete existing stories from the local storage and download new stories.
ws.loadNewService = function(newConfig, context) {

	var newConfigPath, newConfigCrossDomain;

    newConfigPath = ws.remoteConfigRoot;
    newConfigCrossDomain = true;

    $.ajax({
        url: newConfigPath+'/services/'+newConfig+'.js',
        dataType: 'script',
        crossDomain: newConfigCrossDomain, // needs to by dynamic
        cache: true,
        timeout: 5000,
        success: function(data) {

            ws.service = newConfig;

            //ws.log(ws.config.serviceRoot);

            ws.prefs('defaultDialect', newConfig.slice(newConfig.indexOf('-') + 1));

            if (ws.env.status == "online") {

                // clear out the stored stories
                _.forEach(_.keys(localStorage), function(val) {
                    var thisId = val.match(/s(\d+)/) || '';
                    if (!_.isUndefined(thisId[1])) {
                        thisId = thisId[1];
                        localStorage.removeItem(val);
                    }
                });

                //reload the stories with new dialect
                _.delay(function() { ws.stories.fetchAllStories(); }, 500);

                //console.log(ws.popularStories.read);
                ws.popularStories.read.url = ws.config.livestats.mostReadUrl;
                _.delay(function() { ws.popularStories.read.fetch(); }, 600);

                ws.popularStories.watched.url = ws.config.livestats.mostWatchedUrl;
                _.delay(function() { ws.popularStories.watched.fetch(); }, 700);

            }

            //reload date vocab
            Date.replaceChars.longMonths    = ws.config.vocab.longMonths;
            Date.replaceChars.longDays      = ws.config.vocab.longDays;
            jQuery.timeago.settings.strings = ws.config.vocab.timeAgo;

            ws.views.updateViewVocab(context);

        },
		error: function(data){
            // do nothing and revert
            var overlayNode = $('#overlay.loading');
            overlayNode.removeClass('loading').removeClass('visible');

            // notify user
            navigator.notification.alert(
                ws.config.vocab.connectionAlertMsg,
                function(){},
                ws.config.vocab.connectionAlertTitle,
                ws.config.vocab.connectionAlertBtn
            );

		}

    });
}

// choose different hosts
ws.redirect = function() {

    if (ws.mirrors.enabled) {

        //console.log('choose new domain', ws.mirrors.list.length);
        var mirrorIndex = Math.floor(Math.random() * ws.mirrors.list.length);
        //console.log(mirrorIndex);
        ws.mirrorUrl = ws.mirrors.list[mirrorIndex];

        //ws.log("http://" + mirrorUrl + "/worldservice/apps/1");
        ws.remoteConfigRoot = "http://" + ws.mirrorUrl + "/worldservice/apps/1";

        if (ws.config) {
            ws.config.serviceRoot =  "http://" + ws.mirrorUrl + ws.config.relativeRoot;
            //ws.log(ws.config.serviceRoot);
            _.delay(function(){ws.redirect();}, ws.mirrors.interval * 1000); // seconds delay
        }

    }
}

ws.trackView = function(view, url, title) {

    ws.metrics.track(view, {
        "pageName":         title || ws.config.metrics.sitecatalyst.title,
        "contentId":        "",
        "url":              url,
        "contentFlavor":    "ws::"+view
    });

}

/* This is called by PhoneGap when the orientation changes */
/* If true, orientation change is allowed */
function shouldRotateToOrientation(rotation) {
    return ws.env.allowRotate;
}

// Debugging grids
$(document).keypress(function(e) {
    switch(e.which) {
        case 104: $("html").toggleClass("debug-h-grid"); break; // h key
        case 118: $("html").toggleClass("debug-v-grid"); break; // v key
        case 119: $("body").toggleClass("force-320"); break;    // w key
        case 111:
            // o key
            if (ws.env.status == 'online') {
                ws.env.status = "offline";
                $.publish("/status/offline");
            } else {
                ws.env.status = "online";
                $.publish("/status/online");
            }
            break;
    }
});

$("body").delegate("#toggle-h-grid", "click", function() { $("html").toggleClass("debug-h-grid"); e.preventDefault(); });
$("body").delegate("#toggle-v-grid", "click", function() { $("html").toggleClass("debug-v-grid"); e.preventDefault(); });
