/******************************************
 * Copyright 2012 Aerian Studios - comments and queries to info@aerian.com
 * version 1.0
 * This Class creates a very smoothly scrolling element that works with touch events and scrollwheel events.
 * It needs a delegate object in the main application scope to be passed in to it. 
 * The delegate object handles all actions extra to scrolling within the application.
 * 
 ******************************************/
var AS = AS || {};
(function () {
//@TODO: move methods into a prototype.
// Utility class
AS.Vector2D = function(inX, inY) {
    this.x = (inX !== undefined) ? inX : 0;
    this.y = (inY !== undefined) ? inY : 0;
    
    this.copy = function() {
        return new AS.Vector2D(this.x, this.y);
    }

    this.size = function() {
        return Math.sqrt(this.x * this.x + this.y * this.y);
    }

    this.scale = function(scale) {
        this.x *= scale;
        this.y *= scale;
    }
};

AS.ASScrollView = function ($, container) {
	//console.log('scrollview created', this)
		var self = this,
	    delegate = false,
	    vendorPrefix = (/webkit/i).test(navigator.appVersion) ? 'webkit' :
	        (/firefox/i).test(navigator.userAgent) ? 'Moz' :
	        (/trident/i).test(navigator.userAgent) ? 'ms' :
	        'opera' in window ? 'O' : '',
	    
	    container = $(container);
	    container.css('overflow', 'hidden');
	    container.wrapInner('<div class="scroll-view-content"></div>');
	
	    var content = container.find('div.scroll-view-content'),
		    isTouch = 'ontouchstart' in window,
		    lastTouchTime,
		    lastTouch = new AS.Vector2D(),
		    velocity = new AS.Vector2D(0,0),
		    offset = new AS.Vector2D(0,0),
		    transitionEndTimeout = 0,
		    decelerationRate = 1,
		    disabled = false,
	    	tapTimeStamp = false,
	    	requestedVerticalScrollbar = false,// used to track if the setup had a scrollbar request
	    	requestedHorizontalScrollbar = false;// used to track if the setup had a scrollbar request
	    
	    this.lockX = false;
	    this.lockY = false;
	    this.showScrollBar = function (direction) {
	    	createScrollbar (direction);
    	};
    	this.shouldShowHorizontalScrollbar = false;
    	this.shouldShowVerticalScrollbar = false;
    	this.horzontalScrollbar = false;
    	this.verticalScrollbar = false;
    	this.scrollbarOpacity = 0.4;
	    this.hasInertia = true;
	    this.bounces = true;
	    this.touchScale = 1.0;
	    this.scrollToOffsetTime = 1.0;
	    this.use3dTransforms = true;// may cause slowness on some platforms
	    this.pageXSize = 0;
	    this.pageYSize = 0;
	    this.topOffset = 0;
	    this.leftOffset = 0;
	    this.bottomOffset = 0;
	    this.rightOffset = 0;
	    this.vFadeSwitch = this.shouldShowVerticalScrollbar;
	    this.hFadeSwitch = this.shouldShowHorizontalScrollbar;

	    //public functions
	    self.scrollToElement = function (toElement, withAnimation) {
	        var $el = container.find(toElement);

	        var newOffset = new AS.Vector2D(-($el.position().left), -($el.position().top));

	        self.scrollToOffset(newOffset, withAnimation);
	    }

	    self.scrollToOffset = function (toOffset, withAnimation) {
	    	if (delegate && typeof(delegate.scrollViewStartScroll) == 'function') {
                delegate.scrollViewStartScroll(self);
            }
	        withAnimation = withAnimation !== false;
	        var timeInSeconds, easing;
	        // stop something silly being passed in
	        if (typeof toOffset.copy == "undefined") { 
	        	if (typeof console !== "undefined") console.log('The offset was not an expected object in :', arguments.callee.caller.toString());
	        	//return false;
	        }
	        
	        if (toOffset.y > self.topOffset) toOffset.y = self.topOffset;
	        if (toOffset.y < (container.height() - content.height()) - self.bottomOffset) toOffset.y = (container.height() - content.height()) - self.bottomOffset;
	        if (toOffset.x > self.leftOffset) toOffset.x = self.leftOffset;
	        if (toOffset.x < (container.width() - content.width()) - self.rightOffset) toOffset.x = (container.width() - content.width()) - self.rightOffset;
	        
	        if (withAnimation) {
	            timeInSeconds = self.scrollToOffsetTime;
	            easing = 'ease-in-out';
	        }

	        if (isTouch) {
	            offset = toOffset.copy();
	            content.css('-' + vendorPrefix + '-transition', '-' + vendorPrefix + '-transform ' + timeInSeconds + 's ' + easing);
	            if (self.shouldShowHorizontalScrollbar) {
	            	if (self.horizontalScrollbar) self.horizontalScrollbar.css('-' + vendorPrefix + '-transition', '-' + vendorPrefix + '-transform ' + timeInSeconds + 's ' + easing);
	            }
	            if (self.shouldShowVerticalScrollbar) {
	            	if (self.verticalScrollbar) self.verticalScrollbar.css('-' + vendorPrefix + '-transition', '-' + vendorPrefix + '-transform ' + timeInSeconds + 's ' + easing);
	            }
	            render();

	            if (delegate) {
	                delegate.scrollViewDidScrollToWithAnimation(self, offset, timeInSeconds, easing);
	            }

	        } else {
	            if (withAnimation) {
	                container.animate({
	                    scrollTop : -toOffset.y,
	                    scrollLeft: -toOffset.x
	                    }, timeInSeconds * 1000);
	            } else {
	                container.scrollTop(-toOffset.y).scrollLeft(-toOffset.x);
	            }

	            render();

	            if (delegate) {
	                delegate.scrollViewDidScrollToWithAnimation(self, toOffset, timeInSeconds, easing);
	            }
	        }
	        
	        if (withAnimation && delegate && typeof(delegate.scrollViewDidFinishScrollToOffset) == 'function') {
	            transitionEndTimeout = window.setTimeout(function() {
	                delegate.scrollViewDidFinishScrollToOffset(self);
	            }, timeInSeconds * 1000);
	        }
	        
	    }

	    function isNumber(n) {
	        return !isNaN(parseFloat(n)) && isFinite(n);
	    }

	    /**
	    * <h2>Delegate protocol</h2>
	    * 
	    * <h3>Required methods:</h3>
	    * <ul><li>scrollViewDidScrollToWithAnimation(scrollView, offset, timeInSeconds, easing)</li></ul>
	    * 
	    * <h3>Optional methods:</h3>
	    * <ul>
	    * <li>scrollViewDidFinishScrolToOffset(scrollView)</li>
	    * <li>scrollViewDidFinishDecelerating(scrollView)</li>
	    * <li>scrollViewStartScroll(scrollView)</li>
	    * <li>scrollViewDidFinishTrackingFingers(scrollView)</li>
	    * </ul>
	    */
	    self.setDelegate = function(inDelegate) {
	        if (typeof inDelegate.scrollViewDidScrollToWithAnimation == 'function') {
	            delegate = inDelegate;
	        	if(typeof(delegate.scrollViewDidFinishSetup) == 'function') {
        	    	delegate.scrollViewDidFinishSetup(self);
        	    }
	        }
	    }
	    
	    /**
	     * <p>The rate at which the content decelerates after dragging.</p>
	     * <dl>
	     *  <dt>Min value</dt>
	     *  <dd>0.1</dd>
	     *  <dt>Max value</dt>
	     *  <dd>10</dd>
	     * </dl>
	     */
	    self.decelerationRate = function(newRate) {
	        if (isNumber(newRate) && newRate >= 0.1 && newRate <= 10) {
	            decelerationRate = newRate;
	        }
	    };

	    self.getContainer = function() {
	        return container;
	    };

	    self.getContent = function() {
	        return content;
	    };

	    self.getOffset = function() {
	        if (isTouch) {
	            return offset;
	        } else {
	            return new AS.Vector2D(-container.scrollLeft(), -container.scrollTop());
	        }
	    };

	    self.isTouchEnabled = function() {
	        return isTouch;
	    };

	    
	    self.disable = function () {
	    	disabled = true;
	    	// unbind events
	    	//container.unbind('touchstart');
	    	//container.unbind('touchmove');
	    	//container.unbind('touchend');
	    	

            try {            	
            	container.addClass('disabled');
            } catch (err) {}
	    };
	    self.enable = function () {
	    	disabled = false;
	    	try {	    		
	    		container.removeClass('disabled');
	    	} catch (err) {}
	    };
	    self.destroy = function () {
	    	_destroy();
	    };
	    self.refresh = function (withAnimation) {
	    	content = container.find('div.scroll-view-content'),
		    lastTouch = new AS.Vector2D(),
		    velocity = new AS.Vector2D(0,0),
		    offset = new AS.Vector2D(self.leftOffset, self.topOffset),
		    transitionEndTimeout = 0,
		    decelerationRate = 1,
		    self.enable();
	    	
	    	//scroll bar;
	    	var sbar;
	    	if (self.horzontalScrollbar || requestedHorizontalScrollbar) {
	    		if (! self.horizontalScrollbarWrapper) {
	    			createScrollbar('horizontal');
	    			return;
	    		}
	    		self.horizontalScrollbarWrapper.stop(true, true).fadeIn(400);//self.horizontalScrollbarWrapper.css('opacity',self.scrollbarOpacity);
	    		sbar = self.horzontalScrollbar;
	    		sbar.css({'width':'100%'});
				if (reorient) sbar.width(Math.max((container.width() * container.width() / content.width()), 8));
				self.hSBLength = sbar.width();
				self.hSBRatio = 1 / (container.width() * self.hSBLength);
				if (self.hSBRatio < 0.987) { 
					self.shouldShowHorizontalScrollbar = true;
					self.horizontalScrollbarWrapper.stop(true, true).delay(800).fadeOut(800);//self.horizontalScrollbarWrapper.delay(1000).css('opacity','0');
				} else {
					self.horizontalScrollbarWrapper.css('display', 'none');
					self.shouldShowHorizontalScrollbar = false;
					destroyScrollbar("horizontal");
				}
				
	    	}

	    	if (self.verticalScrollbar || requestedVerticalScrollbar) {
	    		if (! self.verticalScrollbarWrapper) {
	    			createScrollbar('vertical');
	    			return;
	    		} 
	    		self.verticalScrollbarWrapper.stop(true, true).fadeIn(400);//self.verticalScrollbarWrapper.css('opacity',self.scrollbarOpacity);
	    		sbar = self.verticalScrollbar;
				sbar.css({'height':'100%'});
				var tempHeight = Math.max((container.height() * container.height() / content.height()), 8);
				sbar.height(tempHeight);
				self.vSBLength = sbar.height();
				self.vSBRatio = 1 / (container.height() / (self.vSBLength));
				alert('vSBRatio: '+self.vSBRatio)
				if (self.vSBRatio < 0.96 ) { 
					self.shouldShowVerticalScrollbar = true;
					self.verticalScrollbarWrapper.stop(true, true).delay(800);//.fadeOut(800);//self.verticalScrollbarWrapper.delay(1000).css('opacity','0');
				} else {
					self.verticalScrollbarWrapper.css('display', 'none');
					self.shouldShowVerticalScrollbar = false;
					destroyScrollbar("vertical");
				}
	    	}

	    	self.scrollToOffset(new AS.Vector2D(self.leftOffset, self.topOffset), false);
	    }
	   
	    //private functions
	    
	    function createScrollbar (direction) {
	    	var sbar,
	    		sbarW,
	    		scrollBarId,
				styles = {};
	    	if (direction == "horizontal") {
	    		if (self.lockX) {//Check if should have a scroll bar in this direction
	    			destroyScrollbar(direction);
		    		return;
	    		}
	    		scrollBarId = 'as-scrollbar-horizontal';
	    		styles = {
					'bottom':'1px',
					'left':'2px',
					'right':(self.lockY) ? '2px' : '5px'
	    		}
	    	} else {
	    		if (self.lockY) {
		    		destroyScrollbar(direction);
		    		return;
		    	}
	    		scrollBarId = 'as-scrollbar-vertical';
	    		styles = {
					'top':'2px',
					'right':'1px',
					'bottom':(self.lockX) ? '2px' : '5px'
	    		}
	    	}
			if (! $('#' + self.contId +  '-' + scrollBarId + '-wrapper').length) {
				// Create the scrollbar wrapper
				sbarW = '<div id="' + self.contId +  '-' + scrollBarId + '-wrapper" class="' + scrollBarId + '-wrapper as-scrollbar-wrapper"></div>';
				container.append(sbarW);
				sbarW = container.find('.' + scrollBarId + '-wrapper');
				sbarW.css(styles);
				sbarW.css('position','absolute','pointer-events','none','overflow','hidden');
				// add to base object for utility
				self[direction + 'ScrollbarWrapper'] = sbarW;
	
				// Create the scrollbar itself
				sbar = '<div class="' + scrollBarId + ' as-scrollbar"></div>';
				sbarW.append(sbar);
				sbar = sbarW.find('.'+scrollBarId);
				// add to base object for utility
				self[direction + 'Scrollbar'] = sbar;
				sbar.css({'position':'absolute', 'pointer-events':'none'});
				if (self.use3dTransforms) {
		            sbar.css('-' + vendorPrefix + '-transform', 'translate3d(0,0,0)');
		        } else {
		            sbar.css('-' + vendorPrefix + '-transform', 'translate(0,0)');
		        }
				
				
				// set scrollbar width or height according to which direction
				if (direction == "horizontal") {
					requestedHorizontalScrollbar = true;
					sbar.css({'width':'100%'});
					self.hSBLength = sbar.outerWidth();
					sbar.width(Math.max((container.outerWidth() * container.outerWidth() / self.content.outerWidth()), 8));
					self.hSBMaxScroll = self.hSBLength - sbar.outerWidth();
					self.hSBRatio = 1 / (container.width() / (self.hSBLength));
					if (self.hSBRatio < 1) {
						self.shouldShowHorizontalScrollbar = true;
					} else {
						self.horizontalScrollbarWrapper.css('display', 'none');
						self.shouldShowHorizontalScrollbar = false;
						destroyScrollbar("horizontal");
					}
				} else {
					requestedVerticalScrollbar = true;
					sbar.css({'height':'100%'});
					self.vSBLength = sbar.outerHeight();
					sbar.height(Math.max((container.outerHeight() * container.outerHeight() / content.outerHeight()), 8));
					self.vSBMaxScroll = self.vSBLength - sbar.outerHeight();
					self.vSBRatio = 1 / (container.height() / (self.vSBLength));
					if (self.vSBRatio < 1) {
						self.shouldShowVerticalScrollbar = true;
					} else {
						self.verticalScrollbarWrapper.css('display', 'none');
						self.shouldShowVerticalScrollbar = false;
						destroyScrollbar("vertical");
					}
				}
			}
			self.scrollToOffset(new AS.Vector2D(self.leftOffset, self.topOffset), false);
		}
	    
	    function destroyScrollbar (direction) {
    		try {
    			self[direction + 'Scrollbar'].remove();
    			self[direction + 'Scrollbar'] = null;
    			if (direction == "horizontal") {
    				self.horizontalScrollbarWrapper.remove();
    				self.shouldShowHorizontalScrollbar = false;
    				self.horizontalScrollbarWrapper = false;
    				self.horizontalScrollbar = false;
    			} else {
    				self.verticalScrollbarWrapper.remove();
    				self.shouldShowVerticalScrollbar = false;
    				self.verticalScrollbarWrapper = false;
    				self.verticalScrollbar = false;
    			}
    		} catch (err) {}
	    }
	    
	    function render() {
	    	if (! disabled ) {
	    		if (self.use3dTransforms) {
		            content.css('-' + vendorPrefix + '-transform', 'translate3d(' + (offset.x) + 'px, ' + (offset.y) + 'px, 1px)');
		            if (self.showScrollBar) {
		            	if (self.shouldShowHorizontalScrollbar) {
			            	if (self.horizontalScrollbar) self.horizontalScrollbar.css('-' + vendorPrefix + '-transform', 'translate3d(' +  (-((offset.x - self.leftOffset) * self.hSBRatio)) + 'px, 0px, 1px)');
		            	}
		            	if (self.shouldShowVerticalScrollbar) {
		            		if (self.verticalScrollbar) self.verticalScrollbar.css('-' + vendorPrefix + '-transform', 'translate3d(0px, ' + (-((offset.y - self.topOffset) * self.vSBRatio)) + 'px, 1px)');
		            	}
		            }
		        } else {
		            content.css('-' + vendorPrefix + '-transform', 'translate(' + (offset.x) + 'px, ' + (offset.y) + 'px)');
		            if (self.showScrollBar) {
		            	if (self.shouldShowHorizontalScrollbar && self.horizontalScrollbar) self.horizontalScrollbar.css('-' + vendorPrefix + '-transform', 'translate(' + (-((offset.x - self.leftOffset) * self.hSBRatio)) + 'px, 0px)');
		            	if (self.shouldShowVerticalScrollbar && self.verticalScrollbar) self.verticalScrollbar.css('-' + vendorPrefix + '-transform', 'translate(0px, ' + (-((offset.y - self.topOffset) * self.vSBRatio)) + 'px)');
		            }
		        }
	    	}
	        
	    }//render
	    
	    if (!isTouch) {
	        container.scroll(function(e) {
	            if (delegate) {
	                delegate.scrollViewDidScrollToWithAnimation(self, new AS.Vector2D(-container.scrollLeft(), -container.scrollTop()));
	            }
	        });

	        container.bind('mousewheel DOMMouseScroll', function(e) {
	        	if (! disabled) {
	        		e.preventDefault();
		            var delta = 0,
		            event = e.originalEvent;

		            if (!event) // For IE. 
		                event = window.event;
		            if (event.wheelDelta) { // IE/Opera. 
		                delta = event.wheelDelta/120;
		            } else if (event.detail) { // Mozilla case.
		                delta = -event.detail;
		            }

		            delta *= 30;

		            var currentOffset = self.getOffset();
		            currentOffset.x += delta;

		            self.scrollToOffset(currentOffset, false);
	        	}
	            
	        });
	    } else {
	    	
	    		container.bind('touchstart', function(e) {
	    			//console.log('touchstart '+self.contId);
		            e = e.originalEvent;
		            
		            // Prevent multiple events firing!
	    			/*if (tapTimeStamp == e.timestamp) return;
    	    		tapTimeStamp = e.timeStamp;*/

		            if (transitionEndTimeout) {
		                window.clearTimeout(transitionEndTimeout);
		            }
	
		            var mString = window.getComputedStyle(content.get(0)).webkitTransform;
		            var m = new WebKitCSSMatrix();
		            m.setMatrixValue(mString);
	
		            offset.x = m.m41;
		            offset.y = m.m42;
		            content.css('-' + vendorPrefix + '-transition', '');
		            
		            // don't do anything there after if disabled
		            if (disabled) return e;
		            
		            //== Scrollbar - this is a hack for asynchronously loaded content
		           if (self.shouldShowHorizontalScrollbar) {
		            	if (self.hSBRatio != (1 / (container.width() / (self.hSBLength))) || self.hSBLength != Math.max((container.width() * container.width() / content.width())-2, 8)) {
		            		self.refresh(false);
		            	}
		            	try {
		            		self.horizontalScrollbarWrapper.stop(true, true).fadeIn(400);
			            	self.horizontalScrollbar.css('-' + vendorPrefix + '-transition', '');
		            	} catch (err) {}
		            	
		            	self.hFadeSwitch = true;
		            } 
		            if (self.shouldShowVerticalScrollbar) {
		            	if (self.vSBRatio != (1 / (container.height() / (self.vSBLength))) || Math.abs(self.vSBLength - Math.max(Math.round(container.height() * container.height() / content.height())-2, 8)) > 2) {
		            		self.refresh(false);
		            	}
		            	try {
		            		self.verticalScrollbarWrapper.stop(true, true).fadeIn(400);
			            	self.verticalScrollbar.css('-' + vendorPrefix + '-transition', '');
		            	} catch (err) {
		            		
		            	}
		            	
		            	self.vFadeSwitch = true;
		            } 
		            
		            render();
	
		            if (delegate) {
		                delegate.scrollViewDidScrollToWithAnimation(self, offset);
		            }
	
		            velocity.y = velocity.x = 0;
		            
		            lastTouch.y = e.targetTouches[0].pageY;
		            lastTouch.x = e.targetTouches[0].pageX;
	        	
	    		});

	    		container.bind('touchmove', function(e) {
	    			if (disabled) return e;
		            e = e.originalEvent;
		            
		            // Prevent multiple events firing!
	    			if (tapTimeStamp == e.timestamp) return;
    	    		tapTimeStamp = e.timeStamp;
    	    		
		            //e.preventDefault();
		            var d = new Date(),
		            	ms = d.getMilliseconds();
		            
	
		            var oldOffset = offset.copy();
		            var pageY = e.targetTouches[0].pageY;
		            var pageX = e.targetTouches[0].pageX;
		            
		            if (!self.lockY) offset.y = offset.y + ((pageY - lastTouch.y) * self.touchScale);
		            if (!self.lockX) offset.x = offset.x + ((pageX - lastTouch.x) * self.touchScale);
	
		            if (!self.lockY) velocity.y = (lastTouch.y - pageY) * 0.75 * self.touchScale;
		            if (!self.lockX) velocity.x = (lastTouch.x - pageX) * 0.75 * self.touchScale;
		            
		            var shouldDelegateOffset = false;
		            if (self.bounces) {
		                if (!self.lockY) {
		                    if (offset.y > self.topOffset || offset.y < (container.height() - content.height()) - self.bottomOffset) {
		                    	// reduce the offset + response to moving 
		                        offset.y = offset.y - ((pageY - lastTouch.y) / (self.touchScale * 1.5));
		                        
		                        if (self.bottomOffset != 0 && offset.y < (container.height() - content.height()) - self.bottomOffset) {
		                        	shouldDelegateOffset = true;
		                        }

		                        if (self.topOffset != 0 && offset.y > self.topOffset) {
		                        	shouldDelegateOffset = true;
		                        }
		                    }
		                }
	
		                if (!self.lockX) {
		                    if (offset.x > self.leftOffset || offset.x < (container.width() - content.width()) - self.rightOffset) {
		                        
		                    	offset.x = offset.x - ((pageX - lastTouch.x) / (self.touchScale * 1.5));
		                    }
		                    //@TODO: add left right offset checks
		                }
	
		            } else {
		                if (offset.y > self.topOffset) offset.y = self.topOffset;
		                if (offset.y < (container.height() - content.height()) - self.bottomOffset) offset.y = (container.height() - content.height()) - self.bottomOffset;
		                if (offset.x > self.leftOffset) offset.x = self.leftOffset;
		                if (offset.x < (container.width() - content.width()) - self.rightOffset) offset.x = (container.width() - content.width()) - self.rightOffset;
		            }
	
		            lastTouch.y = pageY;
		            lastTouch.x = pageX;
		            lastTouchTime = ms;
	
		            render();
		            if (shouldDelegateOffset && delegate && typeof delegate.scrollViewDidEnterOffsetZone == "function") {
	                	delegate.scrollViewDidEnterOffsetZone(self);
	                }
		            if (delegate && (oldOffset.y != offset.y || oldOffset.x != offset.x)) {
		                delegate.scrollViewDidScrollToWithAnimation(self, offset);
		            }
	        	
	    		});

	    		container.bind('touchend', function(e) {
	    			// do nothing if disabled
	    			if (disabled) return e;
		            //console.log('touchend:'+ self.contId)
		            self.vFadeSwitch = false;
		            if (!self.hasInertia /*|| numFingers > 0*/) return;
		            var offsetScale = 20.1 / decelerationRate,
		            	d = new Date(),
		            	ms = d.getMilliseconds(),
		            	dt = ms - lastTouchTime;
		            if (dt > 200) { 
		            	velocity.scale(0);
		            } else {
		            	velocity.scale(10 / dt);
		            }
		            
	
		            var target = new AS.Vector2D(offset.x - (offsetScale * velocity.x), offset.y - (offsetScale * velocity.y));

		            if (target.y > self.topOffset) {
		            	target.y = self.topOffset;
		            	if (velocity.y == 0) velocity.y = 1;
		            }
		            if (target.y < (container.height() - content.height()) - self.bottomOffset) target.y = ((container.height() - content.height())) - self.bottomOffset;
	
		            if (target.x > self.leftOffset) {
		            	target.x = self.leftOffset;
		            	if (velocity.x == 0) velocity.x = 1;
		            }
		            if (target.x < (container.width() - content.width()) - self.rightOffset) target.x = (container.width() - content.width()) - self.rightOffset;
		            
		            //@TODO investigate pageYSize
		            if (self.pageXSize > 1) {
		                target.x = parseInt((target.x - (self.pageXSize / 2)) / self.pageXSize) * self.pageXSize;
		            }
		            
		            if (self.pageYSize > 1) {
		                target.y = parseInt((target.y - (self.pageYSize / 2)) / self.pageYSize) * self.pageYSize;
		            }
		            
		            var delta = new AS.Vector2D(Math.abs(target.x - offset.x), Math.abs(target.y - offset.y));
	
		            var timeScale = 0.0019;
		            var timeInSeconds = (1.8 + Math.sqrt(delta.x * delta.x + delta.y * delta.y) * timeScale) / decelerationRate;
		            
		            var shouldDelegate = false;
		            if (offset.y > self.topOffset ||
		                offset.y < (container.height() - content.height()) - self.bottomOffset ||
		                offset.x > self.leftOffset ||
		                offset.x < (container.width() - content.width()) - self.rightOffset) {
		                timeInSeconds /= 1.8;
		                
		                if (! self.lockY) {
		                	if (self.bottomOffset != 0 && offset.y < (container.height() - content.height()) - self.bottomOffset) {
		                		shouldDelegate = true;
		                	} else {
		                		if (self.topOffset != 0 && offset.y > self.topOffset) {
		                			shouldDelegate = true;
		                		}
		                	}
		                }
		                if (! self.lockX) {
		                	if (self.leftOffset != 0 && offset.x > self.leftOffset) {
		                		shouldDelegate = true;
		                	}
			                if (self.rightOffset != 0 && offset.x < (container.width() - content.width()) - self.rightOffset) {
			                	shouldDelegate = true;
			                }
		                }
		                
		                
		                if (shouldDelegate && delegate && typeof delegate.scrollViewDidReleaseOffsetZone == "function") {
		                	delegate.scrollViewDidReleaseOffsetZone(self);
		                }
		                
		            }
	
		            if (velocity.size() > 0 || self.pageXSize > 1 || self.pageYSize > 1) {
	
		                self.decelerating = true;
		                transitionEndTimeout = window.setTimeout(function() {
		                    self.decelerating = false;
		                    if (delegate && typeof(delegate.scrollViewDidFinishDecelerating) == 'function') {
		                        delegate.scrollViewDidFinishDecelerating(self);
		                    }
		                    if (self.shouldShowHorizontalScrollbar) {
		                    	//@TODO: insert functional delay and fade switch
		                    	self.horizontalScrollbarWrapper.stop(true, true).delay(1000).fadeOut(800);
		                    }
		                    if (self.shouldShowVerticalScrollbar) {
		                    	try {
		                    		// font-size animate is a hack to give a stoppable delay with a callback
		                    		self.verticalScrollbarWrapper.stop(true, true).animate({'font-size':'1em'},1000, function () {
			                    		if(!self.vFadeSwitch) {
			                    			self.verticalScrollbarWrapper.stop(true, true).fadeOut(800);
			                    		}
			                    	});
		                    	} catch (err) {}
		                    	
		                    }
		                    
		                }, timeInSeconds * 1000);
	
		                var easing = 'cubic-bezier(.14, .81, .36, 1)';
		                content.css('-' + vendorPrefix + '-transition', '-' + vendorPrefix + '-transform ' + timeInSeconds + 's ' + easing);
		                if (self.shouldShowHorizontalScrollbar) {
			            	 self.horizontalScrollbar.css('-' + vendorPrefix + '-transition', '-' + vendorPrefix + '-transform ' + timeInSeconds + 's ' + easing);
			            } 
			            if (self.shouldShowVerticalScrollbar) {
			            	self.verticalScrollbar.css('-' + vendorPrefix + '-transition', '-' + vendorPrefix + '-transform ' + timeInSeconds + 's ' + easing);
			            }
		                offset = target.copy();
	
		                render();
	
		                if (delegate) {
		                    delegate.scrollViewDidScrollToWithAnimation(self, offset, timeInSeconds, easing);
		                }
		            } else {
		                if (delegate && typeof(delegate.scrollViewDidFinishTrackingFingers) == 'function') {
		                    delegate.scrollViewDidFinishTrackingFingers(self);
		                }
		            }
	        	
	    		});
	    }
	    //destroy function to remove all active memory elements
	    function _destroy() {
	    	if (!isTouch) {
		        container.unbind('scroll');
		        container.unbind('mousewheel DOMMouseScroll');
		    } else {
		        container.unbind('touchstart');
		        container.unbind('touchmove');
		        container.unbind('touchend');
		    }//istouch
	    	delegate = null;
		    isTouch = null;
		    lastTouch = null;
		    velocity = null;
		    offset = null;
		    transitionEndTimeout = 0;
		    decelerationRate = 0;
		    disabled = false;
		    requestedVerticalScrollbar = false;
		    requestedHorizontalScrollbar = false;
	    	self = {};
	    	if (self.shouldShowVerticalScrollbar) self.destroyScrollbar("vertical");
	    	if (self.shouldShowHorizontalScrollbar) self.destroyScrollbar("horizontal");
	    	var cHtml = content.html();
	    	content.remove();
	    	container.html(cHtml);
	    	container = null;
	    	content = null;
	    }

	    self.scrollToOffset(new AS.Vector2D(self.leftOffset, self.topOffset), false);
};
})();