/* Models */
ws.models.Story = Backbone.Model.extend({

    idAttribute: "bbcws_id", // Set ID to the Topcat2 ID

    sync: function(method, model, options) {
        var isInStore = !_.isEmpty(ws.store('s'+model.id));
        //console.log(model.id, isInStore);
        if (isInStore) {
            model.isInLocalStore = true;
            switch(method) {
              case 'create':
              case 'update':
              case 'delete':
              case 'read':
                var storyData = ws.store('s'+model.id);
                model.attributes = storyData || {};
                return model;
            }
        } else {
            return Backbone.sync(method, model, options);
        }
    },
    defaults: {
        title:   "",
        summary: "",
        body:    "",
        media:   {},
        vGenre:  "",
        teasers: {
            '104x59': { 'src': 'images/placeholder_104x59.png' }
        }
    },
    initialize: function(options) {
		//ws.log('[INITIALIZE] story model');
        _.bindAll(this, "getFullContent", "storeOffline");

        if (!options) { options = {}; }
        this.url = options.url || null;

        var modelInstance = this;

        this.bind("change", function() {
			//ws.log('[INITIALIZE - BIND CHANGE EVENT] story model');
			//ws.log(modelInstance);
            if (ws.enableOffline && modelInstance.offlineable === true) {
                // If the model's data changes, and it is an offlineable story,
                // then resave the data to localStorage
                modelInstance.storeOffline();
            }
        });

        // If model.teaser hasn't been set already,
        // set 104x59 as default teaser (might change in the future)
        if (!modelInstance.has('teaser')) {
            var teasers = modelInstance.get('teasers'),
                teaserSrc;

            if (_.isEmpty(teasers)) {
                teasers = modelInstance.defaults.teasers;
            }

            if (!_.isEmpty(teasers['104x59']['base64']) && teasers['104x59']['base64'].match('data:image')) {
                teaserSrc = teasers['104x59']['base64'];
            }  else {
                teaserSrc = teasers['104x59']['src'];
            }

            modelInstance.unset('teasers');
            modelInstance.set({ teaser: teaserSrc });

            // Get Base64 equivalent of the teaser for offline storage
            // This is only a fallback should Base64 not be in the main feeds
            if (ws.enableOffline && !teaserSrc.match('data:image') && teaserSrc != this.defaults.teasers['104x59']['src']) {
                //ws.log('[INDEX] Download B64 image: '+teaserSrc);
                var b64Teaser = teaserSrc.replace('/104x59/','/104x59_b64/');
                $.get(b64Teaser, function(b64) {
                    modelInstance.set({ teaser: b64 });
                });
            }
        }


    },

    getFullContent: function(url, options) {
		//ws.log('[GETFULLCONTENT]');

		// check if app enabled via config.json
		if (ws.env.remoteConfig.appEnabled == false) { return false; }

        if (!options) { options = {}; }

        // Gets the full content of the story
        var self = this,
            key  = this.id;


        this.url = url || this.get('href');
        this.url = this.url.replace('.shtml', ws.jsonExt);
        //this.url = 'http://delta.bbc.co.uk/russian/mobapptest/111101_putin_award_journalists.app.json';

		// redirect story model to retrieve story data from mirrored feed
		if (ws.config.mirrorFeed == true && this.url.substring(0,4) === 'http') {

			var currStoryDomain = this.url.split('//')[1].split('/')[0];
			var mirroredDomain = ws.config.serviceRoot.split('//')[1].split('/')[0];

			this.url = this.url.replace(currStoryDomain, mirroredDomain);
		}

        this.fetch({
            success: options.success ? options.success : null,
            error:   options.error ? options.error : null
        });
    },

    storeOffline: function() {
		//ws.log('[STOREOFFLINE] storing ');
        if (this.has('body')) { this.set({ offline: 1 }); }
        ws.store('s'+this.id, this.toJSON());
    },

    isOffline: function() {
        return !_.isEmpty(ws.store('s'+this.id));
    },

    parse: function(response) {
        var self = this;

        _.each(response.media, function(val, key) {
            // Get Base64 of an image if it doesn't come in a feed - this is slower
            if (val.type == 'image' && _.isEmpty(val.base64) && ws.enableOffline && self.offlineable === true) {
               //ws.log('[STORY] Download B64 image: '+self.url);
               var b64Source = val.src.replace(/104x59|144x81|112x63/,  '/104x59_b64/')
                                      .replace('224x280','144x180_b64')
                                      .replace(/304x171|336x189|464x261|624x351|976x549/, '/304x171_b64/');

                $.get(b64Source, function(b64) {
                    media[key].src = b64;
                    self.set({ media: media });
                    self.trigger('change');
                });
            }


            // Check that the poster image for a video is ImageChefed to 304x171 size
            if (val.type == 'video') {
                if (!val.poster.match('/worldservice/ic/')) {
                    val.poster = val.poster.replace('/worldservice/assets/', '/worldservice/ic/304x171/worldservice/assets/');

                    response.media[key] = val;
                }
            }
        });

        return response;
    }
});


ws.models.StorySet = Backbone.Collection.extend({
    model:    ws.models.Story,
    value:    null,
    initialize: function(options) {
        if (!options) { options = {}; }
        this.url     = options.url || ws.config.serviceRoot+'/index'+ws.jsonExt;
        this.genreId = options.genreId || "homepage";
		//ws.log("StorySet", options) ;
    },

    fetchAllStories: function(options) {
		//ws.log("####### fetchAllStories ########")

		// check if app enabled via config.json
		if (ws.env.remoteConfig.appEnabled == false) { return false; }

        if (!options) { options = {}; }
        var collectionInstance = this;
        $.publish('/feeds/downloadstarted');


        // Get homepage stories ????????????????????????
		/*
        if (options.skipHomepage != true) {
            var homepageStories = new ws.models.StorySet({url: ws.config.serviceRoot+'/index'+ws.jsonExt });
            homepageStories.fetch({
                headers: { BBCWSVer: ws.version },
                success: function(result) {

                }
            });
            delete homepageStories;
        };*/



        // Get list of genres availables and the user sorted order
        var availableGenres = ws.config.availableGenres,
            genreSavedOrder = ws.genreOrder;

		// add homepage/topnews genre to download
        if (options.skipHomepage != true) {
			availableGenres['homepage'] = {
				path: '/',
				selected: true,
				enabled: true
			};
        };

        // If Rolling News is enabled, add as another genre to download
        if (ws.config.rollingNews.enabled) {
            availableGenres[ws.config.rollingNews.id] = {
                path:     ws.config.rollingNews.path,
                selected: true,
                enabled:  true
            };
            genreSavedOrder[ws.config.rollingNews.id] = true;
        }

        // Get stories from other indices
        var totalGenresToDownload  = 0, // counter of how many feeds have started downloaded
            fetchedCollectionCount = 0; // counter of how many feeds have completed their download

        //ws.log("[AVAILABLE GENRES]",availableGenres)

        _.each(availableGenres, function(genre, genreId) {
            if (genreSavedOrder[genreId] != true && genreId != 'homepage') { return; }

			_.defer(function(){
				//ws.log("[DEFER]")

	            var indexUrl      = ws.config.serviceRoot + genre.path + '/index'+ws.jsonExt; //'tests/json/homepage_gallery_russian.app.json';
	                tmpCollection = new ws.models.StorySet({ url: indexUrl, genreId: genreId });

	            totalGenresToDownload++;

				tmpCollection.fetch({
	                headers: { BBCWSVer: ws.version },
	                success: function(data) {

	                    fetchedCollectionCount++;
	                    if (fetchedCollectionCount == totalGenresToDownload) {
	                        // All feeds have been downloaded at this point
	                        ws.store('lastUpdated', new Date());
	                        ws.store.prune(); // Remove old stories from localStorage
	                        if (ws.enableOffline) { ws.stories.storeStoriesOffline(); }

							_.delay(function() {ws.views.renderLastUpdated(); }, 500);

	                        $.publish('/feeds/downloadcomplete');
	                    }
	                    //console.log('works: ', data);
	                },
					error: function() {
						fetchedCollectionCount++;
						// hack to fix bug when re-loading app in offline, to re-render the homepage
						if (fetchedCollectionCount == totalGenresToDownload) {
							//ws.log('error');
							$.publish('/feeds/downloadcomplete');
						}
					}
	            });
	            delete tmpCollection;

			});//
        });


    },

    fetchGenre: function(genreId) {
		$.publish('/feeds/downloadstarted');
        // Fetches additional stories from a particular genre
        var indexUrl      = (genreId=='homepage') ? ws.config.serviceRoot + '/index'+ws.jsonExt : ws.config.serviceRoot + ws.config.availableGenres[genreId].path + '/index'+ws.jsonExt,
            tmpCollection = new ws.models.StorySet({ url: indexUrl, genreId: genreId });

        //ws.log("[AVAILABLE GENRES fetchGenre]",genreId,indexUrl)

        tmpCollection.fetch({
			headers: { BBCWSVer: ws.version },
			success: function() {
				//ws.log('#####FETCH GENRE '+genreId+' SUCCESS #########');

				var genreLUKey = 'lastUpdated'+genreId;
				ws.store(genreLUKey, new Date());
				_.delay(function() {ws.views.renderLastUpdated(genreId); }, 500);
				tmpCollection.trigger('add');
				$.publish('/feeds/downloadcomplete');

				//need to move to view.
				//ws.views.scrollers.refresh();
			},
			error: function() {
				$.publish('/feeds/downloadcomplete');
				//ws.views.scrollers.refresh();
			}
		}); // The stories will be filtered to ws.stories in the parse() method below
		if ($('body').hasClass('offline')) {
			//console.log('error with tmpCollection');
			tmpCollection = ws.stories.getByGenre(genreId);
		}
		return tmpCollection;
    },

    getByGenre: function(genreId) {
        // This returns a collection of stories in editorial order for a partular genre location
        var storyIndex = ws.store('storyIndex')[genreId],
            filtered   = []
            self       = this;

        var newStorySet = new ws.models.StorySet();
		//ws.log("[getByGenre newStorySet]", genreId, this)

        _.each(storyIndex, function(val) {
            var story = self.get(val);

            if (story) {
                story.collection = newStorySet;
                filtered.push(story);
            }
        });
        newStorySet.add(filtered);

        return newStorySet;
    },

    sortByMostRecent: function() {
        var collectionInstance = this;
        var sorted = collectionInstance.sortBy(function(story) {
           return story.get("updated");
        }).reverse();

        var newList = new ws.models.StorySet();
        newList.add(sorted);

        return newList;
    },

    storeStoriesOffline: function() {
        //ws.log('[STORE STORIES OFFLINE]');
        self = this;

        // Keep a copy of the whole collection in offline storage
        ws.store('stories', this.toJSON());

        // Saves the story models to offline storage
        _.each(_.keys(ws.config.availableGenres), function(genre, i) {
            var genreStories = self.getByGenre(genre);

            genreStories.forEach(function(storyModel, k) {
                if (k < ws.config.storiesToOffline) {
					//ws.log('########### store this '+ genre + ' ' + k);
                    storyModel.offlineable = true;
                    storyModel.getFullContent();
					storyModel.trigger('change');
                    //storyModel.storeOffline();
                }

            });
        });


    },

    parse: function(response) {

		//ws.log('[PARSE] storyset collection');

        // Filter out stories which have no title set
        response = _.filter(response, function(entry) {
            return !_.isEmpty(entry.title);
        });

        // Keep a record of all story IDs from each feed
        var storyIndex = ws.store('storyIndex') || {},
            storyIds   = _.pluck(response, 'bbcws_id');

        storyIndex[this.genreId] = storyIds;
        ws.storyIndex = storyIndex;
        ws.store('storyIndex', storyIndex);


        // Remove items from the core ws.stories which aren't listed in storyIndex
        var storyIndexIds = _.flatten(storyIndex),
            toRemove      = _.difference(ws.stories.pluck('bbcws_id'), storyIndexIds);

        //console.log(ws.stories.pluck('bbcws_id'), storyIndexIds);
        ws.stories.remove(toRemove);

        // Filter items to the core ws.stories object
        ws.stories.remove(storyIds);
        ws.stories.add(response, {silent: true});
        ws.stories.trigger("add");
        ws.stories.trigger("change");

        return response;
    }
});


/* Live stats */
ws.models.liveStatsBaseClass = Backbone.Collection.extend({
    model: ws.models.Story,
    fetch: function(options) {
        options || (options = {});
        var collection = this;
        var success = options.success;
        var entries = [];
        var self = this;
        $.get(this.url, function(data) {
            _.each(self.parse(data), function(story, i) {
                entries.push(story);
            });
			collection.reset();
            collection.add(entries);
            collection.trigger('change'); // For some reason, this manual trigger is needed
            if (success) {
				success(collection);
				//$.publish('/feeds/downloadcomplete');
				//var genreLUKey = 'lastUpdated'+this.tab;
				//ws.store(genreLUKey, new Date());
			}
        });
    }
});

ws.models.mostPopular = ws.models.liveStatsBaseClass.extend({
    initialize: function() {
        this.url   = ws.config.livestats.mostReadUrl;
		this.tab   = 'read';
        this.parse = ws.config.livestats.mostReadParse || ws.models._parseLiveStats;
    }
});


ws.models.mostWatched = ws.models.liveStatsBaseClass.extend({
    initialize: function() {
        this.url   = ws.config.livestats.mostWatchedUrl;
		this.tab   = 'watched';
        this.parse = ws.config.livestats.mostWatchedParse || ws.models._parseLiveStats;
    }
});


ws.models._parseLiveStats = function(data) {
    var isResponsiveFeed;
    // This function parses the livestats include and
    // returns it as a nice little object

    var lsDom       = $(data),
        outputList  = [];

    lsDom.find('li a').each(function() {
        var $el = $(this),
            href  = $el.attr('href'),
            title;

        title = $el.text();

        outputList.push({
            "title": title,
            "href":  href
        });
    });

    return outputList;
}