ws.viewsAndroid = {
    scrollers:    {},
    templates:    {},
    init: function() {
        // Set underscore template settings to accept the curlies {variable}
        _.templateSettings = { interpolate : /\{(.+?)\}/g };

        // Loads the CSS files and prepares the layout
        $("body").attr("id", ws.service);
        if (ws.env.retina) { $("body").addClass("retina"); }
        ws.views.loadCSS();
        ws.views.loadTemplates();
        ws.views.scrollers.init();

        // Event subscriptions
        $.subscribe("/status/offline", function(){
            // Update UI to show offline status
            // ws.views.notify("Lost connection<br/>gone offline");
			// ws.log('gone offline');
            $('body').addClass('offline');

            $('#ws-refresh, .pulldown').removeClass('activity');
        });

        $.subscribe("/status/online", function(){
            // Update UI to show online status
            // ws.views.notify("Gone online");
			// ws.log('gone online');
            $('body').removeClass('offline');
        });

        $.subscribe("/status/disabled", function(){
            // Update UI to show offline status
            // ws.views.notify("Lost connection<br/>gone offline");
			// ws.log('Disabled');
            $('body').addClass('disabled');
        });

        $.subscribe("/status/enabled", function(){
            // Update UI to show online status
            // ws.views.notify("Gone online");
			// ws.log('Enabled');
            $('body').removeClass('disabled');
        });

        $.subscribe("/feeds/downloadstarted", function() {
			// ws.log('download started');
            $('#ws-refresh').addClass('activity');
            ws.env.downloading = true;
            // After 15 seconds, timeout activity spinners
            _.delay(function() {
                $('#ws-refresh, .pulldown').removeClass('activity');
            }, 15000);
        });

        $.subscribe("/feeds/downloadcomplete", function() {
			// ws.log('download complete');
            ws.env.downloading = false;
            $('#ws-refresh, .pulldown').removeClass('activity');
            $('#overlay.loading div').empty();
            $('#overlay.loading').removeClass('loading').removeClass('visible');
			// ws.views.renderLastUpdated();
            ws.views.scrollers.refresh();
        });


        // Convert header buttons
        $('.ws-header button').each(function() {
            var node  = $(this),
                label = node.text();

            node.html('<span>'+label+'</span><span class="end"></span>');
        });


        // Don't allow scrolling of the main app (headers/footers)
        document.addEventListener('touchmove', function(e){
            e.preventDefault();
        });

    },
    loadCSS: function() {
        var cssRoot = "styles/",
            cssHtml,

        cssHtml = '<link href="'+cssRoot+'core.css" type="text/css" rel="stylesheet" />'
                + '<link href="'+cssRoot+ws.config.dir+'.css" type="text/css" rel="stylesheet" />'
                + '<link href="'+cssRoot+'type.css" type="text/css" rel="stylesheet" />'
                + '<link href="'+cssRoot+'services/'+ws.service+'.css" type="text/css" rel="stylesheet" />'
                + '<link href="'+cssRoot+'services/'+ws.service+'-android.css" type="text/css" rel="stylesheet" />'
                + '<link href="'+cssRoot+'android.css" type="text/css" rel="stylesheet" />'
                + '<link href="'+cssRoot+ws.config.dir+'-android.css" type="text/css" rel="stylesheet" />';


        $("head").append(cssHtml);
    },

    loadTemplates: function() {
        ws.views.templates.storyRow =  $.ajax({ type:'GET', url:'templates/story_row.html', async:false }).responseText;

		// Render pulldown HTML once here, since the markup never changes
		var pulldownTemplate = _.template($.ajax({ type:'GET', url:'templates/pulldown.html', async:false }).responseText);
		ws.views.templates.pulldown = pulldownTemplate(ws.config);

		// Add to config object, so it can easily be inserted into other templates alongside the vocab
		ws.config.pulldown = ws.views.templates.pulldown;

        $("[data-template]").each(function() {
            var node = $(this),
                templateName = node.attr("data-template"),
                templatePath = "templates/"+templateName+".html",
                templateContent = $.ajax({ type:'GET', url:templatePath, async:false }).responseText;

            node.html(templateContent);
        });
    },
    switchTo: function(to, reverse) {
		// ws.log('[SWITCHTO]');
        // ws.log($(to).siblings('.page'));
        // var from = $('.page.active');
        var from = $(to).siblings('.active');
        $(to).addClass('active slide in');
        from.addClass('slide out');

        // Helps to refresh scrollers here, in case the
        // destination page has the pull-to-refresh showing
        ws.views.scrollers.refresh(this.el);

        if (reverse == true) {
            $(to).addClass('reverse');
            from.addClass('reverse');
        }
        _.delay(function() {
            from.removeClass('active reverse slide out');
            $(to).removeClass('reverse slide in');
            $(to).addClass('active'); // Be double sure that the destination page goes active
        }, 350);
    },

	renderLastUpdated: function(genreId) {
		var genre = null;

		if (genreId) {genre=genreId;}
		else if (ws.currentHomeTab=='rolling_news') {genre='rolling_news';}
		else if (ws.currentHomeTab=='latest_news') {genre='latest_news';}
		else if (ws.currentPopularTab) {genre=ws.currentPopularTab;}
		else {}

		var updated = new Date();
		if (genre && ws.store('lastUpdated'+genre)) {updated=ws.store('lastUpdated'+genre);}
		else if (ws.store('lastUpdated')) {updated=ws.store('lastUpdated');}
		else {}

		//updated = (genre && ws.store('lastUpdated'+genre)) ? ws.store('lastUpdated'+genre) : ws.store('lastUpdated'),
		niceDate = new Date(Date.parse(updated)).format(ws.config.shortDateTime);

		// ws.log(genre,".renderLastUpdated")
		_.delay(function() {$('.pulldown .time').text(niceDate);}, 500);

	},

    updateViewVocab: function(context) {

        ws.config.pulldown = ws.views.templates.pulldown;

        if (context != 'init') {
            // re-render views
            ws.views._activeMainView.viewToRender = new ws.views.settingsView();
            ws.views._activeMainView.render();

            // rebuild bottom nav
            $('.ws-home a').html('<span></span>' + ws.config.vocab.tabNews);
            $('.ws-popular a').html('<span></span>' + ws.config.vocab.tabPopular);
            $('.ws-categories a').html('<span></span>' + ws.config.vocab.tabCategories);
            $('.ws-settings a').html('<span></span>' + ws.config.vocab.tabSettings);

        } else {
            $('#ws-view-chooser a[data-tabtarget="#ws-top-news-list"] span').html(ws.config.vocab.topNews);
        }

    },

    launchLiveAV: function(playlist) {

        if (ws.env.status == 'online' && ws.env.remoteConfig.appEnabled == true && ws.config.liveVideo) {
			if (ws.nativeVideoPlayer == true) {
				//PhoneGap.exec("LandscapePlayer.play", {url: playlist });
				window.plugins.videoPlayer.play(playlist);
			}
		}

    },

    scrollers: {
        _list: {},
        _refreshCounter: 0,
        init: function() {
            $('#scroll-to-top').bind('tap', function() {
                var activeScroller = $('.page.active .ws-scrollable').attr('id');
                ws.views.scrollers._list[activeScroller].scrollTo(0, 0, 200);
		//ws.views.scrollers._list[activeScroller].scrollToOffset(new AS.Vector2D(0,0), false);
            });
        },
        drawScrollablePage: function(selector) {
            var node        = $(selector),
                isVisible   = node.is(':visible'),
                contentNode = node.find('.ws-content'),
                id = contentNode.attr('id') ? contentNode.attr('id') : ws.utils.uid();
            if (!isVisible) { node.show(); } // Needed to properly calculate heights
            var pulldownNode        = contentNode.find('.pulldown'),
                pulldownLabel       = pulldownNode.find('.label'),
                pullDownOffset      = pulldownNode.height(),
                scrollWrapperHeight = $(window).height() - node.find(".ws-header").height();


            //ws.log(node.find(".ws-header").height(), node.find(".ws-footer").height());
            contentNode.height(scrollWrapperHeight).css('overflow', 'hidden');
            if (!isVisible) { node.hide(); } // Re-hide previously hidden content

            // Draw ASScrollView
            contentNode.attr('id', id);
            contentNode.wrapInner('<div class="ws-content" />').removeClass('ws-content');
            // Destroy existing scroller
            if (ws.views.scrollers._list[id]) { ws.views.scrollers._list[id].destroy(); }


            var scroller = new iScroll(id, {
                desktopCompatibility:false,
                useTransition: true,
                vScrollbar: true,
				scrollbarClass: 'scrollbar-',
                hScrollbar: false,
                hideScrollbar: true,
                lockDirection: true,
                topOffset: pullDownOffset,
                onScrollStart: function() {
                    contentNode.addClass('scrolling');
                    pulldownNode = contentNode.find('.pulldown');
                },
                onScrollMove: function() {
                	if (this.y >= -48 && this.lastYMove == -48 && this.last2YMove == -48 && !pulldownNode.hasClass('release-to-refresh')) {
                        pulldownNode.addClass('release-to-refresh');
                        pulldownLabel.html(ws.config.vocab.releaseToRefresh);
                        this.minScrollY = 0;
                        this.hasPulldownAction = true;
                    } else if (this.y < -48 && pulldownNode.hasClass('release-to-refresh')) {
                        pulldownNode.removeClass('release-to-refresh');
                        pulldownLabel.html(ws.config.vocab.pullToRefresh);
                        this.minScrollY = -pullDownOffset;
                        this.hasPulldownAction = false;
                    }
                	this.last2YMove = this.lastYMove;
            		this.lastYMove = this.y;
                },
                onScrollEnd: function() {
                    contentNode.removeClass('scrolling');

                    if (this.hasPulldownAction && pulldownNode.is(":visible")) {
                        //ws.log('pull down starts here');
                        pulldownNode.removeClass('release-to-refresh').addClass('activity');
                        if (!$('body').hasClass('offline')) {
							//_.delay(function() { ws.stories.fetchAllStories(); }, 500);
							if (ws.currentGenre) {
								//ws.log('refresh categories');
								_.delay(function() {ws.stories.fetchGenre(ws.currentGenre); }, 500);
							} else if (ws.currentPopularTab=='read' || ws.currentPopularTab=='watched') {
								// check if app enabled via config.json
								if (ws.env.remoteConfig.appEnabled == false) { return false; }
								//ws.log('refresh most read and most watched');
								ws.popularStories.read.fetch({
									success: function(data) {
										$.publish('/feeds/downloadcomplete');
										var genreLUKey = 'lastUpdatedread';
										ws.store(genreLUKey, new Date());
										_.delay(function() {ws.views.renderLastUpdated('read'); }, 500);
									}
								});
								ws.popularStories.watched.fetch({
									success: function(data) {
										$.publish('/feeds/downloadcomplete');
										var genreLUKey = 'lastUpdatedwatched';
										ws.store(genreLUKey, new Date());
										_.delay(function() {ws.views.renderLastUpdated('watched'); }, 500);
									}
								});
							} else {
								// ws.log('refresh everything including rolling news');
								_.delay(function() { ws.stories.fetchAllStories(); }, 500);
							}
                         } else {
							// end pulldown anim after a few seconds
                        	_.delay(function() {
					            $('#ws-refresh, .pulldown').removeClass('activity');
					            ws.views.scrollers.refresh();
                        	}, 1500);
                        }
						_.delay(function() {
							$('#ws-refresh, .pulldown').removeClass('activity');
							ws.views.scrollers.refresh();
						}, 10000);
                        this.hasPulldownAction = false;
                    }
                },
                bounce: false,
                fixedScrollbar: true

            });
            scroller.contId = id;
            scroller.doLayout = function() {
                // Recalculates the scrolling height region
                // ws.log('doing layout');
                var scrollWrapperHeight = $(window).height() - node.find(".ws-header").height();
                contentNode.height(scrollWrapperHeight);
            }

            ws.views.scrollers._list[id] = scroller;
            return scroller;
        },
        refresh: function(sourceEl) {
            // Forces a refresh on all the iScroll instances
            //ws.log('refreshing scrollers ');
            $.each(ws.views.scrollers._list, function(i, val) {
                val.options.topOffset = $(val.wrapper).find('.pulldown:visible').height();
                if (val.options.topOffset > 0) {
                    val.hasPulldown = true;
                    //val.scrollTo(0, -val.options.topOffset, 0);
                } else {
                    val.hasPulldown = false;
                }

                val.refresh();
            });
        }
    }
}




/*******************************************************************************
 * Backbone driven views /* Main and Story views here
 ******************************************************************************/
ws.viewsAndroid.MainView = Backbone.View.extend({
    el: "#main",
    tapTimeStamp: {},//this is a hack to prevent tapclick event happening 3 times
    events: {//tap events appear to be happening 3 times, so changed to click
        "click #ws-view-chooser li": 	"handleViewTab",
        "click #ws-tab-bar li":      	"handleBottomTabBar",
        "tap #ws-refresh":         "refresh",
        "click #ws-live button":     "launchLiveStream",
        "swiperight #ws-home":		"rightSwipeTab",
        "swipeleft #ws-home":		"leftSwipeTab",
        "click #ws-banner span":		"handleBandTap"
    },
    initialize: function() {
    	//ws.log('[INIT MAINVIEW]');
        var self = this;
        _.bindAll(this, "render"); // to solve the 'this' issue
        // this.collection.bind("change", this.render);
        this.node = $(this.el);

        this.template = _.template($(this.el).html());

        // body click
        $('body').bind('click', function (e) {
        	e.stopPropagation();
        	self.showMenu(e);
        	//close live menu if open
        	if (!$(e.target).is('#ws-live span')) {
        		$('#avSelectOverlay').addClass('closed').removeClass('visible').css({'display':'none'});
            	$('#ws-live').removeClass('active');
        	}

        });


        this.render();
    },
    showMenu: function (e, direction) {
    	var menu = $('#ws-sliding-menu'),
    		targ = (typeof e.target !== "undefined") ? e.target : e;
    	if ($(targ).is('#ws-menu-button *')) {
    		if(typeof direction === "undefined") {
	    		direction = (!menu.hasClass('closed')) ? "up" : "down";
	    	}
    	} else {
    		direction = "up";
		}

    	if (direction == "up") {
    		/*menu.stop(true,true).slideUp(200, function () {
    			menu.addClass('closed');
    		});*/

    		menu.css({'-webkit-transition':'height ease-out 300ms', 'height': '0px'});
    		_.delay(function () {
    			menu.addClass('closed');
    		}, 350);
    	} else {
    		//close live menu if open
        	$('#avSelectOverlay').addClass('closed').removeClass('visible').css({'display':'none'});
        	$('#ws-live').removeClass('active');
    		/*menu.slideDown(400, function () {
    			menu.stop(true,true).removeClass('closed');
    		});*/
        	menu.css({'display':'block','-webkit-transition':'height ease-out 500ms', 'height': '67px'});
    		_.delay(function () {
    			menu.removeClass('closed');
    		}, 550);
    	}
    },
    rightSwipeTab: function(e) {
    	var node   = $(e.currentTarget),
        	target,
        	active = $('#ws-view-chooser li.active');
    	if (active.find('a').attr("data-tabtarget") == "#ws-latest-list") {
    		target = active.siblings('li').eq(0).find('a').attr('data-tabtarget');
    	} else {
    		return;
    	}

	    active.removeClass('active');
	    active.siblings('li').eq(0).addClass('active');

	    $(target).parent('.tab-group').children().hide();
	    $(target).show();

		ws.views.renderLastUpdated();
	    ws.views.scrollers.refresh(this.el);
    },
    leftSwipeTab: function(e) {
    	var node   = $(e.currentTarget),
	    	target,
	    	active = $('#ws-view-chooser li.active');
		if (active.find('a').attr("data-tabtarget") == "#ws-top-news-list") {
			target = active.siblings('li').eq(0).find('a').attr('data-tabtarget');
		} else {
			return;
		}

	    active.removeClass('active');
	    active.siblings('li').eq(0).addClass('active');
	    $('#ws-home .tab-group').children().hide();
	    $(target).show();

		ws.views.renderLastUpdated();
	    ws.views.scrollers.refresh(this.el);
    },
    handleViewTab: function(e) {
        var node   = $(e.currentTarget),
            target = node.find('a').attr('data-tabtarget');

        node.parent().find('li').removeClass('active');
        $(target).parent('.tab-group').children().hide();
        $(target).show();

        node.addClass('active');
		ws.views.renderLastUpdated();
        ws.views.scrollers.refresh(this.el);
    },
    handleBandTap: function (e) {
    	this.viewToRender = new ws.views.homeView({ collection: ws.stories });
    	$(this.el).removeClass('category-view-active');
        $(this.el).removeClass(function(index, css){
            return (css.match (/\bcurrent-genre-\S+/g) || []).join(' ');
        });
		$('#ws-banner').find('span').text('');
		//up navigation
		$('#main .up-navigation').removeClass('active');

		ws.views.renderLastUpdated();
        this.render();
    },
    handleBottomTabBar: function(e) {
        var self   = this,
            node   = $(e.currentTarget),
            target = node.attr('data-tabtarget');/*,
            scrollEl =  ws.views.scrollers._list[$(this.el).find('.ws-scrollable').attr('id')];*/

        // remove live av dropdown if it exists and is visible
        if ($('#avSelectOverlay').hasClass('visible')) {
        	console.log(3)
            $('#avSelectOverlay').removeClass('visible').addClass('closed').css({'display':'none'});
            $('#ws-live').removeClass('active');
        }

        switch(target) {
            case "#ws-home":
            	this.viewToRender = new ws.views.homeView({ collection: ws.stories });
            	break;
            case "#ws-popular":
            	this.viewToRender = new ws.views.popularView();
            	break;
            case "#ws-categories":
            	this.viewToRender = new ws.views.categoryListView();
            	break;
            case "#ws-settings":
            	this.viewToRender = new ws.views.settingsView();
            	break;
        }

        $(this.el).removeClass('category-view-active');
        $(this.el).removeClass(function(index, css){
            return (css.match (/\bcurrent-genre-\S+/g) || []).join(' ');
        });
		$('#ws-banner').find('span').text('');

		ws.views.renderLastUpdated();
        this.render();
    },

    goToCategoryView: function() {

		// check if app enabled via config.json
		if (ws.env.remoteConfig.appEnabled == false) { return false; }

        var self = this;
		if (ws.currentGenre != null) {
			this.vGenre = ws.config.availableGenres[ws.currentGenre] ? ws.config.availableGenres[ws.currentGenre].title : "";
			$('#ws-banner').find('span').text(this.vGenre);
            $(this.el).addClass('current-genre-' + ws.currentGenre);
		}
        $(this.el).addClass('category-view-active');

        $('#ws-tab-bar li').removeClass('active');
        $('#ws-tab-bar .ws-categories').addClass('active'); // Activate relevant tab
        //$('#ws-categorised-list').addClass('active');

        _.delay(function() { ws.views.renderLastUpdated(ws.currentGenre); }, 50);
        self.scroller.scrollTo(0,0,0);
	    //self.scroller.scrollToOffset(new AS.Vector2D(0,0),false);
		ws.views.scrollers.refresh(self.el); //this seems to work better than self.scroller.refresh();
            //self.scroller.refresh();
		//up navigation
		$('#main .up-navigation').removeClass('active');
    },

    backToMainView: function() {
        var self = this,
            backButton = $(this.el).find('.back');
        //if (!backButton.hasClass('enabled')) return false;
        //up navigation
		$('#main .up-navigation').removeClass('active');

        $(this.el).removeClass('category-view-active');
        // remove category class if it exists
        $(this.el).removeClass(function(index, css){
            return (css.match (/\bcurrent-genre-\S+/g) || []).join(' ');
        });

        //on back button change tab to main only if we're back on the homepage, not the catgory list
		if ($('#ws-categories').length < 1) {
			$('#ws-tab-bar li').removeClass('active');
			$('#ws-tab-bar .ws-home').addClass('active');
		}

        backButton.removeClass('enabled');

		// reset env variables
        ws.currentGenre = null;
        ws.currentHomeTab = null;
		ws.currentPopularTab = null;
        $('#ws-banner').find('span').text('');

        _.delay(function() {
			$('#ws-categorised-list').remove();
			//ws.views.renderLastUpdated();
			//ws.views.scrollers.refresh();
            ws.views.scrollers.refresh(self.el);
            //self.scroller.scrollTo(0,0,0);
		}, 500);

        return false; //prevent propagation

    },

    refresh: function() {
    	this.showMenu({},"up");
    	console.log(4)
    	$('#avSelectOverlay').addClass('closed').removeClass('visible').css({'display':'none'});
    	$('#ws-live').removeClass('active');
		if (ws.env.status == 'online') {
			$('#ws-refresh').addClass('activity');
			ws.stories.fetchAllStories();
		}
		//up navigation
		$('#main .up-navigation').removeClass('active');
    },

    launchLiveStream: function(e, direction) {
        //ws.log('[launchLive]');
    	this.showMenu({},"up");
        if (ws.env.status != 'online' || ws.env.remoteConfig.appEnabled != true || !ws.config.liveVideo) {
            return;
        }

        var thisTarget = $('#ws-live'),
        	avSelect;

        thisTarget.addClass('active');

        if ($('#avSelectOverlay').length > 0) {
        	avSelect = $('#avSelectOverlay');
    		var targ = $(e.target);
	    	if (targ.is(thisTarget) || targ.is('*',thisTarget)) {
	    		if(typeof direction === "undefined") {
		    		direction = (!avSelect.hasClass('closed')) ? "up" : "down";
		    	}
	    	} else {
	    		direction = "up";
			}
	    	if (direction == "up") {
	    		/*avSelect.stop(true,true).slideUp(200, function () {
	    			avSelect.addClass('closed').removeClass('visible');
	    			thisTarget.removeClass('active');
	    		});*/
	    		avSelect.css({'-webkit-transition':'height ease-out 300ms', 'height': '0'});
	    		_.delay(function () {
	    			avSelect.removeClass('visible').addClass('closed').css('display','none');
	    			thisTarget.removeClass('active');
	    		}, 350);
	    	} else {
	    		/*avSelect.addClass('visible').slideDown(400, function () {
	    			$('#avSelectOverlay').find('li').removeClass('active');
	    			avSelect.stop(true,true).removeClass('closed');
	    		});*/
	    		avSelect.css({'display':'block','-webkit-transition':'height ease-out 500ms', 'height': '67px'});
	    		_.delay(function () {
	    			avSelect.removeClass('closed').addClass('visible');
	    			avSelect.find('li').removeClass('active').css('display','');
	    		}, 550);
	    	}

            /*if ($('#avSelectOverlay').hasClass('visible')) {
                $('#avSelectOverlay').removeClass('visible');
                thisTarget.removeClass('active');
                return false;
            } else {
                $('#avSelectOverlay').find('li').removeClass('active');
                $('#avSelectOverlay').addClass('visible');
            }*/

        } else {
            var self =  this;
            avSelect = $('<div id="avSelectOverlay" class="closed" style="height:0;"><ul><li class="live-video">'+ ws.config.vocab.liveTv +'<span></span></li><li class="live-audio">'+ ws.config.vocab.liveRadio +'<span></span></li></ul></div>');

            $('.ws-header').after(avSelect);
            avSelect = $('#avSelectOverlay');

            //_.delay(function(){$('#avSelectOverlay').addClass('visible');}, 50);

            /*$('#avSelectOverlay').bind('tap', function(e) {
                $('#avSelectOverlay').removeClass('visible');
                thisTarget.removeClass('active');
                return false;
            });*/
            /*avSelect.slideUp().addClass('visible').slideDown(400, function () {
    			$('#avSelectOverlay').find('li').removeClass('active');
    			avSelect.stop(true,true).removeClass('closed');
    		});*/

            avSelect.css({'display':'block','-webkit-transition':'height ease-out 500ms', 'height': '67px'});

    		_.delay(function () {

    			avSelect.removeClass('closed').addClass('visible').find('li').removeClass('active');
    		}, 550);

            $('#avSelectOverlay').find('.live-video').bind('tap', function(e) {
                //ws.log('[launchLiveVideo]');
                $(this).addClass('active');
                //$('#avSelectOverlay').addClass('closed');
                //thisTarget.removeClass('active');
                ws.views.launchLiveAV(ws.config.liveVideo);
                //avSelect.stop(true,true).slideUp(200, function () {
        			//avSelect.addClass('closed').removeClass('visible').css('display','none');
        			thisTarget.removeClass('active');
        		//});
                //return false;
            });

            $('#avSelectOverlay').find('.live-audio').bind('tap', function(e) {
                ws.log('[launchLiveAudio]');
                $(this).addClass('active');
               // $('#avSelectOverlay').addClass('closed').removeClass('visible').css('display','none');
                //thisTarget.removeClass('active');
                ws.views.launchLiveAV(ws.config.liveRadio);
                //avSelect.stop(true,true).slideUp(200, function () {
        			//avSelect.addClass('closed').removeClass('visible');
        			thisTarget.removeClass('active');
        		//});
                //return false;
            });
        }
    },

    render: function() {
        var self = this;
        //up navigation
		$('#main .up-navigation').removeClass('active');
        // Create iScroll if it doesn't exist already
        var scrollableNodeId = $(this.el).find('.ws-scrollable').attr('id');
        if (_.isUndefined(scrollableNodeId)) {
            this.node.html(this.template(ws.config)); // Render template and mixin vocab
            this.scroller = ws.views.scrollers.drawScrollablePage(this.el);
            this.contentNode = this.node.find('.ws-content');
        }


        // If no subview has been specified, default to the Home one
        if (!this.viewToRender) { this.viewToRender = new ws.views.homeView({ collection: ws.stories }); }

        // Highlight appropiate tab button
        var viewId = this.viewToRender.el.id;
        $('#ws-tab-bar li').removeClass('active');
        $('#ws-tab-bar .'+viewId).addClass('active'); // Activate relevant tab


        // Render subview
        this.contentNode.html(this.viewToRender.el);
        this.viewToRender.render();

        _.delay(function() {
            //self.scroller.refresh();
            ws.views.scrollers.refresh();
            self.scroller.scrollTo(0,0,0);
	    $('#ws-sliding-menu').removeClass('start-closed').slideUp(0);
        }, 100);

        return this;
    }
});


ws.viewsAndroid.generalStoryList = Backbone.View.extend({
    //tagName:   'ul',
    //className: 'ws-entries',
    initialize: function() {
        _.bindAll(this, "render"); // to solve the 'this' issue

		this.collection.bind('reset', this.render);
        //this.collection.bind('change', this.render);
        this.collection.bind('add', this.render);

        $(this.el).empty(); // Clear the element when starting
    },
    render: function() {
		//ws.log('[GENERAL STORY LIST] rendering view');
        var self        = this,
            node        = $(this.el),
            storyIdList = []; // Keep track of stories already listed, so to prevent duplicates


        node.html(ws.views.templates.pulldown + '<ul class="ws-entries" data-groupgenre="'+this.collection.genreId+'"></ul>');
		var listNode = node.find('.ws-entries');

        this.collection.forEach(function(story) {
            // If 'filterGenre' option is specified, only display those items
            if (!_.isUndefined(self.options.filterGenre)) {
                var filterGenre = self.options.filterGenre;
                if (filterGenre != story.get('genre') && filterGenre != story.get('topic')) return;
            }

            var storyEntry = new ws.views.StoryListRow({ model: story });

            // Check if story has been displayed already, otherwise skip
            if (_.indexOf(storyIdList, story.id) == -1) {
                listNode.append(storyEntry.render().el);
                storyIdList.push(story.id);
             }

        });

        if ($(this.el).attr('id') == "ws-categorised-list") {
        	//up navigation
    		$('.up-navigation').addClass('active');
        	// add footer
			$("#ws-page-footer").remove();
			$('#ws-categorised-list').append('<div id="ws-page-footer">BBC &copy; 2012</div>');
        }
        ws.views.scrollers.refresh();
        return this;
    }
});


ws.viewsAndroid.StoryView = Backbone.View.extend({
    el:               "#story",
    doPageTransition: true,
    events: {
        "click #story-navigation .prev":	"previous", // Tap doesn't work too well on next and previous either
        "click #story-navigation .next":	"next",
        "swiperight .ws-content":			"previous",
        "swipeleft .ws-content":			"next",
        "tap .up-navigation":				"back",
        "tap .ws-content":					"moveShareTools",
        "tap .video-container":				"playVideo",
		"tap .audio-container":				"playAudio",
        "tap .ws-text-size a":				"handleTextSize",
        "tap .share":						"shareTools",
        "tap .picture-gallery-container":   "openpicturegallery",
        "tap .ws-story-toolbar":            "preventDefault" // prevent auto zoom in ios5+
    },
    initialize: function() {
        _.bindAll(this, "render", "previous", "next", "back");
        var self = this;

        if (typeof(this.model) == 'undefined') {
            this.model = new ws.models.Story();
        }
        this.template = _.template($(this.el).html());
    },
    preventDefault: function(e) {
        e.preventDefault();
    },
    catchSwipeLeft: function() {
        if (ws.config.dir == 'ltr') {
            $(this.el).find('#story-navigation .next').trigger('tap');
        } else {
            $(this.el).find('#story-navigation .prev').trigger('tap');
        }
    },
    catchSwipeRight: function() {
        if (ws.config.dir == 'ltr') {
            $(this.el).find('#story-navigation .prev').trigger('tap');
        } else {
            $(this.el).find('#story-navigation .next').trigger('tap');
        }
    },
    previous: function() {

		if (this.options.disableNavigation == true) {return false;}

        var self = this;
        var slideto = 'slidetoright';//ws.config.dir == 'ltr' ? 'slidetoright' : 'slidetoleft';
        $(this.el).find('.prev, .next').removeClass('disabled');
        this.storyPosition--;

		//exception for the homepage needs to be refactored so that all next/previous navigation uses ids instead of collections
		/*if (ws.currentGenre == 'homepage' || ws.currentGenre == 'latestnews') {

            var currentList = ws.currentGenre == 'homepage' ? ws.homepageList : ws.latestList;

			if (this.storyPosition < 0) {
				this.storyPosition = 0;
				$(this.el).find('.prev').addClass('disabled');
			} else {

				var potentialPrevStory = this.storyPosition;

				if (ws.env.status == 'offline') {
					potentialPrevStory = null;
					//catch if offline and the next story doesn't exist
					$.each(currentList, function(i) {
						if (i <= self.storyPosition && ws.store('s'+currentList[i])) {
							potentialPrevStory = i;
						}
						return (i <= self.storyPosition);
					});
				}

				if (potentialPrevStory != null) {

					//go to the previous homepage story
					var destinationStoryId = currentList[potentialPrevStory];
					this.doPageTransition = false;

					//ws.log('go to previous story - ' + potentialPrevStory + destinationStoryId);

					$(this.el).find('.ws-scrollable').addClass(slideto);

					_.delay(function() {
						ws.thisRouter.navigate('story/' + destinationStoryId, true);
						$(this.el).find('.ws-scrollable').removeClass(slideto);
					}, 350);

				} else {
					this.storyPosition++;
					$(this.el).find('.prev').addClass('disabled');
				}
			}

		} else {*/

			if (this.storyPosition < 0) {
				this.storyPosition = 0;
				$(this.el).find('.prev').addClass('disabled');
			} else if (ws.env.status == 'offline' && !ws.store('s'+this.collection.at(this.storyPosition).id)) {
				this.storyPosition++;
				$(this.el).find('.prev').addClass('disabled');
			} else {
				this.model = this.collection.at(this.storyPosition);
				this.doPageTransition = false;

				//ws.log('go to previous story - ' + this.storyPosition, this.model);

				$(this.el).find('.ws-scrollable').addClass(slideto);

				_.delay(function() {
					var destinationStoryId = self.model.id || self.model.cid;
					ws.thisRouter.navigate('story/' + destinationStoryId, true);
					$(this.el).find('.ws-scrollable').removeClass(slideto);
				}, 350);
			}

		//}

        return false; //prevent propagation

    },

    next: function() {

		if (this.options.disableNavigation == true) {return false;}

        var self = this;
        var slideto = 'slidetoleft';//ws.config.dir == 'ltr' ? 'slidetoleft' : 'slidetoright';
        $(this.el).find('.prev, .next').removeClass('disabled');
        this.storyPosition++;

		//exception for the homepage needs to be refactored so that all next/previous navigation uses ids instead of collections
		/*if (ws.currentGenre == 'homepage' || ws.currentGenre == 'latestnews') {

            var currentList = ws.currentGenre == 'homepage' ? ws.homepageList : ws.latestList;

			if (this.storyPosition > currentList.length-1) {
				this.storyPosition = currentList.length-1;
				$(this.el).find('.next').addClass('disabled');
			} else {

				var potentialNextStory = this.storyPosition;

				if (ws.env.status == 'offline') {
					potentialNextStory = null;
					//catch if offline and the next story doesn't exist
					$.each(currentList, function(i) {
						if (i >= self.storyPosition && ws.store('s'+currentList[i])) {
							potentialNextStory = i;
							return (potentialNextStory == null);
						}
					});

				}

				if (potentialNextStory != null) {

					//go to the next homepage story
					var destinationStoryId = currentList[potentialNextStory];
					this.doPageTransition = false;

					//ws.log('go to next story - ' + potentialNextStory, destinationStoryId);

					$(this.el).find('.ws-scrollable').addClass(slideto);

					_.delay(function() {
						ws.thisRouter.navigate('story/' + destinationStoryId, true);
						$(this.el).find('.ws-scrollable').removeClass(slideto);
					}, 350);

				} else {
					this.storyPosition--;
					$(this.el).find('.next').addClass('disabled');
				}

			}

		} else {*/

			if (this.storyPosition > this.collection.length-1) {
				//catch if the last story of the collection
				this.storyPosition = this.collection.length-1;
				$(this.el).find('.next').addClass('disabled');
			} else if (ws.env.status == 'offline' && (!ws.store('s'+this.collection.at(this.storyPosition).id) || this.storyPosition > ws.config.storiesToOffline-1)) {
				//catch if offline and the next story doesn't exist
				this.storyPosition--;
				$(this.el).find('.next').addClass('disabled');
			} else {
				//go to the next story
				this.model = this.collection.at(this.storyPosition);
				this.doPageTransition = false;

				//ws.log('go to next story - ' + this.storyPosition, this.model);

				$(this.el).find('.ws-scrollable').addClass(slideto);

				_.delay(function() {
					var destinationStoryId = self.model.id || self.model.cid;
					ws.thisRouter.navigate('story/' + destinationStoryId, true);
					$(this.el).find('.ws-scrollable').removeClass(slideto);
				}, 350);
			}

		//}

		return false; //prevent propagation

    },

    back: function() {

		/*if (this.options.referrer == 'byUrl') {
			// for stories accessed by related links
			window.history.back();
		} else {*/
			if (ws.currentGenre == 'homepage' || ws.currentGenre == 'latestnews' || ws.currentGenre == 'rollingnews') {
				ws.currentGenre = null;
			}
			ws.thisRouter.navigate('', true);
		//}

        return false; //prevent propagation
    },

    playVideo: function(e) {
        if (ws.env.status == 'online' && ws.env.remoteConfig.appEnabled == true) {
			if (ws.nativeVideoPlayer == true) {
				var videoSrc = $(e.currentTarget).attr('data-source');
				//console.log('videoSrc: '+videoSrc)
				//cordova.exec("LandscapePlayer.play", {"url": videoSrc });
				// window.plugins.videoPlayer.play(videoSrc);
                window.videoPlayer.play(videoSrc);
			} else {
				var node = $(e.currentTarget).find('video')[0].play();
			}
		}
    },

	playAudio: function(e) {
        if (ws.env.status == 'online' && ws.env.remoteConfig.appEnabled == true) {
			if (ws.nativeVideoPlayer == true) {
				var audioSrc = $(e.currentTarget).attr('data-source');
				//PhoneGap.exec("LandscapePlayer.play", {url: audioSrc });
				// window.plugins.videoPlayer.play(audioSrc);
                window.videoPlayer.play(audioSrc);
			} else {
				var node = $(e.currentTarget).find('audio')[0].play();
			}
		}
	},

    handleRelatedLink: function(e) {
        var url       = $(e.target).attr('href'),
            mobileUrl = url.replace(ws.config.serviceRoot, ws.config.serviceRoot+ws.mobileRedirect);

        window.plugins.childBrowser.showWebPage(mobileUrl);
        e.preventDefault();
    },

    handleTextSize: function(e) {
        var action        = _.isUndefined(e) ? '' : $(e.target).parent().attr('data-action'),
            sizeNode      = $(this.el).find('.ws-text-size'),
            storyTextSize = ws.prefs('textSize') || 1;
        sizeNode.find('.disabled').removeClass('disabled');
        switch(action) {
            case "smaller":
                storyTextSize = storyTextSize-1 < 1 ? 1 : storyTextSize-1;
                break;
            case "larger":
                storyTextSize = storyTextSize+1 > 5 ? 5 : storyTextSize+1;
                break;
        }

        if (storyTextSize >= 5) { sizeNode.find('.larger').addClass('disabled'); }
        if (storyTextSize <= 1) { sizeNode.find('.smaller').addClass('disabled'); }
        $(this.el).find('.ws-story-content').attr('class', 'ws-story-content text-size-'+storyTextSize);
        ws.views.scrollers.refresh(this.el);

        ws.prefs('textSize', storyTextSize);

        if (!_.isUndefined(e)) {
        	e.preventDefault();
        	e.stopPropagation();
    	}
    },
    openpicturegallery: function(e) {
        //ws.log('[OPEN PICTURE GALLERY]');

        if (ws.env.status == 'online' && ws.env.remoteConfig.appEnabled == true) {
            if (!this.pictureGalleryView) {
                this.pictureGalleryView = new ws.viewsAndroid.PictureGallery({gallery: this.gallery}).render();
            } else {
                this.pictureGalleryView.options.gallery = this.gallery;
                this.pictureGalleryView.render();
            }
            ws.views.switchTo('#picture-gallery');
        }

        return false;
    },
    moveShareTools: function (e) {
    	var placeholder = '<div id="footer-placeholder">&nbsp;</div>',
			shareToolsBar;
    	var tgt = $(e.target);
    	if (tgt.is('.ws-footer *')) {
    		return e;
    	} else if ($('#story > .ws-footer').length) {
    		shareToolsBar =  $('#story').find('.ws-footer').detach();
    		$('#footer-placeholder').replaceWith(shareToolsBar);
    	} else {
    		shareToolsBar = $(this.el).find('.ws-footer').clone();
        	$(this.el).find('.ws-footer').replaceWith(placeholder);

        	$('#story').append(shareToolsBar);
        	shareToolsBar = $('#story').find('.ws-footer');
        	//make the placeholder hte same height as the sharetools
        	$('#footer-placeholder').height(shareToolsBar.height()).css({'padding-top':shareToolsBar.height()+'px'});
    	}

    },
    shareTools: function(e) {
        // Sharetools should probably have its own view code, instead of here
        var self = this;

        window.plugins.socialsharing.share(
            self.model.get('title'),
            null,
            null,
            self.model.get('href')
        );

        if (!_.isUndefined(e)) { e.preventDefault(); e.stopPropagation();}
    },

    hideShareTools: function() {
    	$('.ws-footer').fadeOut();
    },

    showShareTools: function() {
	    $('.ws-footer').fadeIn(800);
    },

    render: function() {
        // ws.log('[STORY] Rendering view')


        var self = this,
            node = $(this.el);

        if (typeof(this.model) == 'undefined') return false;
        this.hideShareTools();
        var storyData = this.model.toJSON();

        // Mix in the vocab object
        storyData.vocab = ws.config.vocab;

        // Prettify the output
        if (_.isUndefined(storyData.updated)) {
            storyData.updated = "";
        } else {
            storyData.updated = new Date(Date.parse(storyData.updated)).format(ws.config.dateFormat);
        }

        // Apply vernacular genre
        var genreId = storyData.genre;
        //storyData.vGenre = ws.config.availableGenres[genreId] ? ws.config.availableGenres[genreId].title : "";

        if (ws.genreContext == "latest_stories") {
            // show 'latest stories'
            storyData.vGenre = ws.config.vocab.latestNews;
        } else if (ws.genreContext == "most_read" || ws.genreContext == "most_watched" || ws.genreContext == "top_stories") {
            // show current stories' genre
            storyData.vGenre = "";
            //storyData.vGenre = ws.config.availableGenres[genreId] ? ws.config.availableGenres[genreId].title : "";
            if (ws.config.availableGenres[genreId]) {
                storyData.vGenre = ws.config.availableGenres[genreId].title ? ws.config.availableGenres[genreId].title : "";
            }
            if (storyData.vGenre == "") {
                if (ws.config.availableGenres[storyData.topic]) {
                    //check topics if no genre matches
                    storyData.vGenre = ws.config.availableGenres[storyData.topic] ? ws.config.availableGenres[storyData.topic].title : "";
                }
                if (genreId == "rolling_news") { storyData.vGenre = ws.config.rollingNews.title; }
            }
        } else {
            // show contextual category user visited from
            storyData.vGenre = ws.config.availableGenres[ws.genreContext].title;
        }

        $(this.el).html(this.template(storyData));

        // Add class to first paragraph
        $(this.el).find('.bodytext p:first').addClass('ingress');

        // Insert any rich media
        var storyMedia = this.model.get('media');

        // Set font size
        var storyTextSize = ws.prefs('textSize') || 1;
        $(this.el).find('.ws-story-content').attr('class', 'ws-story-content text-size-'+storyTextSize);

        // Force update of connection status - this updates ws.env.network
        ws.connection.check();

        _.each(storyMedia, function(val, i) {
            var html = '';
            switch(val.type) {
                case "image":
                    // Image media
                    var size = val.size.split('x'),
                        src  = val.base64 || val.src;

                    html = '<div class="module image img-w'+size[0]+'">'
                             + '<img src="'+src+'" width="'+size[0]+'" height="'+size[1]+'"/>';
                    if (val.caption) { html += '<p class="caption">'+val.caption+'</p>' }
                    html += '</div>';
                    break;

                case "video":
                    // Video media
                    var playerAttributes = 'controls webkit-playsinline-DISABLED ';
                    var mp4Sources = _.select(val.sources, function(src){ return src.type == "video/mp4"; });
                    mp4Sources = _.sortBy(mp4Sources, function(src){ return parseInt(src.size); });

                    // Switch video quality depending on what type of connection we're on
                    // 3G: Low quality | Wifi: Medium quality
                    var videoSrc = ws.env.network.speed == "fast" ? mp4Sources[1] : mp4Sources[0];

                    // If user tapped on the teaser image, and it's a Media Asset Page, start autoplay
                    if (storyData.type == 'av_instance' && self.clickedOnImage) {
                        playerAttributes += 'autoplay ';
                        if (ws.nativeVideoPlayer == true) {
                            //PhoneGap.exec("LandscapePlayer.play", {url: videoSrc.src});
                            window.videoPlayer.play(videoSrc.src);
                        }
                    }


                    if (typeof(videoSrc) != 'undefined') {
                        //html = '<video src="'+videoSrc.src+'" poster="'+val.poster+'" width="304" height="171"></video>'
                        var duration = val.duration,
                            minutes = parseInt(duration/60) % 60,
                            seconds = parseInt(duration % 60, 10),
                            niceDuration = (minutes<10 ? "0" + minutes : minutes) + ":" + (seconds<10 ? "0" + seconds : seconds);


                        html = '<div class="video-container" data-source="'+videoSrc.src+'">'
                             + '<img src="'+val.poster+'" width="304" height="171" />';

                        if (!ws.nativeVideoPlayer) {
                            html += '<video src="'+videoSrc.src+'" poster="'+val.poster+'" width="304" height="171" '+playerAttributes+'></video>';
                        }

                        html += '<span class="cta">'+niceDuration+'</span><span class="warning">'+ws.config.vocab.offlineVideoCta+'</span></div>';

                    }
                    break;


                case "picturegallery":
                    //ws.log('picturegallery');
                    //$(self.el).find('#'+val.id).remove();

                    html = '<div class="picture-gallery-container"><img src="http://ichef.bbci.co.uk/wsimagechef/ic/304x171/'+ val.images[0].src.replace('http://wscdn.bbc.co.uk/','') + '" />';

                    html += '<span class="cta"><span class="icon"></span><span>'+ws.config.vocab.launchGallery+'</span></span><span class="warning">'+ws.config.vocab.galleryOfflineMessage+'</span></div>';

                    self.gallery = val.images;

					break;

				case "audio":
					// Audio media
					var playerAttributes = 'controls webkit-playsinline-DISABLED ';
					var mp3Sources = _.select(val.sources, function(src){ return src.type == "audio/mpeg";});
					mp3Sources = _.sortBy(mp3Sources, function(src){ return parseInt(src.size) });

					if (val.poster == '') {val.poster = 'images/608x342_blank.png';}

					var audioSrc = mp3Sources[0];

					if (typeof(audioSrc) != 'undefined') {
                        var duration = val.duration,
                            minutes = parseInt(duration/60) % 60,
                            seconds = parseInt(duration % 60, 10),
                            niceDuration = (minutes<10 ? "0" + minutes : minutes) + ":" + (seconds<10 ? "0" + seconds : seconds);

                        html = '<div class="audio-container" data-source="'+audioSrc.src+'">'
							 + '<img src="'+val.poster+'" width="144" height="81" />';

						html += '<span class="cta">'+niceDuration+'</span><span class="warning">'+ws.config.vocab.offlineVideoCta+'</span></div>';
					}
					break;

            }

            // Replace media node in body area
            $(self.el).find('#'+val.id).html(html);
        });

        // handle special stories
        if (ws.config.specials) {
            var thisurl = storyData.href;
            _.each(ws.config.specials, function(special,i) {
                if (special.url == thisurl) {
                    $(self.el).find(special.targetEl).html(special.html);
                }
            });
        }

        // check after json loaded if story body has been populated, if not show message
        _.delay(function() {
            // handle empty pages
            if (ws.env.downloading == false && $(self.el).find('.bodytext').children().length < 1) {
                $(self.el).find('.bodytext').append('<p class="ingress">' + ws.config.vocab.storyError + '</p></div>');
                //ws.log('this story is empty');
            }
        },1000);

        // If there's related stories, insert them in
        if (_.size(storyData.related)) {
            var output = '';
            _.forEach(storyData.related, function(val, key) {
                if (typeof(val) == 'object') {
                    // New related items format (post Nov 3 2011)
                    var date = new Date(Date.parse(val.updated)).format(ws.config.dateFormat);

                    output += '<li><a href="#story/url/'+val.href+'">'+val.title+'</a><div class="date">'+date+'</div></li>';
                } else {
                    // Original JSON format
                    output += '<li><a href="#story/url/'+key+'">'+val+'</a><div class="date"></div></li>';
                }
            });
            node.find('.related ul').html(output);
			// catch related link tap event to prevent when app is online
			node.find('.related ul li').bind('tap', function(e) {
				if (ws.env.status == 'offline') { e.preventDefault(); }
			});
        } else {
            node.find('.related').remove();
        }


        // Disable prev/next buttons if at the edge of a collection needs to be reworked
        if (this.collection) {

			//ws.log('ws.currentGenre', ws.currentGenre, ws.homepageList, storyData);

			/*if (ws.currentGenre == 'homepage' || ws.currentGenre == 'latestnews') {

                var currentList = ws.currentGenre == 'homepage' ? ws.homepageList : ws.latestList;

                //ws.log(currentList);

				this.storyPosition = _.indexOf(currentList, storyData.bbcws_id);

				if (this.storyPosition <= 0) { node.find('.prev').addClass('disabled'); }
				if (this.storyPosition >= currentList.length-1) { node.find('.next').addClass('disabled'); }

				//ws.log("this.storyPosition " + this.storyPosition);

				// this if checks homepage array for stories stored offline and enables/disables next/previous buttons when appropriate
				if (ws.env.status == 'offline' && this.storyPosition > 0 && this.storyPosition < currentList.length) {

					node.find('.next').addClass('disabled');
					node.find('.prev').addClass('disabled');

					// loop through array, if stories from list are stored in array enable next/prev
					$.each(currentList, function(i) {
						// if index > than current story position and story is saved then we can enable next
						if (i > self.storyPosition && ws.store('s'+currentList[i]) && node.find('.next').hasClass('disabled')) {
							node.find('.next').removeClass('disabled');
						}
						// if index < than current story position and story is saved then we can enable prev
						if (i < self.storyPosition && ws.store('s'+currentList[i]) && node.find('.prev').hasClass('disabled')) {
							node.find('.prev').removeClass('disabled');
						}
					});

				}

			} else {*/
	        	var ind = 0,
					len = this.collection.length,
					listHTML = '<li class="pip-nav"></li>',
					navUl = $('#story-navigation');

				for (; ind < len; ind++) {
					navUl.append(listHTML);
				}


				if (this.storyPosition <= 0) {
					//node.find('.prev').addClass('disabled');
				}
				if (this.storyPosition >= this.collection.length-1) { node.find('.next').addClass('disabled'); }

				if (ws.env.status == 'offline' && this.storyPosition > 0 && this.storyPosition < this.collection.length) {
					//ws.log('disable next previous if offline');

					if (!ws.store('s'+this.collection.at(this.storyPosition + 1).id) || this.storyPosition >= ws.config.storiesToOffline-1) {
						//ws.log('the next one doesnt exist!');
						node.find('.next').addClass('disabled');
					}

					if (!ws.store('s'+this.collection.at(this.storyPosition - 1).id)) {
						//ws.log('the previous one doesnt exist!');
						node.find('.prev').addClass('disabled');
					}
				}
				navUl.find('li').eq((this.storyPosition) + 2).addClass('selected');
			//}

        }


		if (this.options.referrer == 'byUrl') { this.options.disableNavigation = true; }

        // Check if any controls need to be disabled
        if (this.options.disableNavigation) { node.find('#story-navigation').remove(); }
        if (this.options.disableSharing)    { node.find('.share').remove(); }

        this.handleTextSize();

        ws.views.scrollers.drawScrollablePage(this.el);
        //ws.views.scrollers.refresh();
       // this.showShareTools();

        return this;
    }
});

// Extend PictureGallery view for android
ws.viewsAndroid.PictureGallery = ws.views.PictureGallery.extend({
    events: {
        "tap .done": 		"returnToStory",
        "tap .next": 		"nextImage",
        "tap .previous": 	"previousImage",
        "tap .captions":  	"toggle",
        "tap li img": 		"toggleFurniture",
        "tap li p": 		"toggleFurniture",
        "tap li": 			"toggleFurniture"
    },
	createScroller: function() {
        var self = this;
        self.scroller = new iScroll('pg-wrapper', {
            snap: 'li',
            vScroll: self.vScroll,
            hScroll: self.hScroll,
            momentum: false,
            hScrollbar: false,
            vScrollbar: false,
            desktopCompatibility: true,
            bounce: false,
            onScrollEnd: function () {
                //ws.log(this.currPageY);
                self.checkNavPosition();
                self.loadImage(self.currImageIndex); //incase you've swipped too fast
            }
        });
        _.delay(function() {
            self.scroller.refresh();
            if (ws.config.dir == 'rtl') {
                if (self.orientation == 'responsive') {
                    self.scroller.scrollToPage(self.currImageIndex, 0, 0);
                } else {
                    self.scroller.scrollToPage(0, self.currImageIndex, 0);
                }
            }
            $(self.el).find('.pg-overlay').addClass('hidden');
            //self.checkNavPosition();
            self.prepareCaptions();
        },1000);
    },
    createCaptionScroller: function(i) {
    	//ws.log('createCaptionScroller-Android');

    	var self = this;

		self.captionScrolls[i] = new iScroll('captionScroll' + i, {
            vScroll: true,
            hScroll: false,
            zoom: false,
            desktopCompatibility: true,
            bounce: false
        });

    },
    returnToStory: function(e) {
        //ws.log('back');

        if ($(this.el).hasClass('hideFurniture')) {
        	this.toggleFurniture(e);
        	return false;
        }

        var self = this;
        $(window).unbind('orientationchange');
        $(window).unbind('offline');
        $(window).unbind('online');
        self.unbind();

        ws.views.switchTo('#story', true);
        _.delay(function() {
            self.scroller.destroy(); // improves performance :)
            $(this.el).empty();
            $(this.el).removeClass('showDetails');
        },750);
        e.stopImmediatePropagation();
        return false;
    },
    toggle: function(e) {
        // hide controls and captions
        //ws.log('toggle');
        if ($(this.el).hasClass('hideFurniture')) {
        	this.toggleFurniture(e);
        	return false;
        }

        $(this.el).toggleClass('showDetails');
        if ($(this.el).hasClass('showDetails')) {
            $(this.el).find('.captions span').html(ws.config.vocab.galleryHideCaptions);
        } else {
            $(this.el).find('.captions span').html(ws.config.vocab.galleryShowCaptions);
        }
        e.stopImmediatePropagation();
        return false;
    },
    toggleFurniture: function(e) {
    	//ws.log('toggle furniture');

        $(this.el).toggleClass('hideFurniture');

        e.stopImmediatePropagation();
        return false;
    }
})


/*******************************************************************************
 * Subviews of "Main" go here (Home, Popular, Categories, Settings)
 ******************************************************************************/
ws.viewsAndroid.homeView = Backbone.View.extend({
    id: 'ws-home',
    events: {
        "click #ws-view-chooser li": "handleViewTab"
    },
    initialize: function() {
		//ws.log('[INIT HOMEVIEW]');
        _.bindAll(this, "handleViewTab", "render"); // to solve the 'this' issue
        this.node      = $(this.el);
        this.template  = _.template($.ajax({ type:'GET', url:'templates/home-android.html', async:false }).responseText);
        ws.currentGenre = null;
		ws.currentHomeTab = null;
		ws.currentPopularTab = null;

        this.topStoriesView = new ws.views.StoryList({
            id: "ws-top-news-list",
            groupgenre: "top_stories",
            collection: this.collection,
            showFullStoryList: false,
            showTopStories:    true
        });

        this.recentStoriesView = new ws.views.mostRecentList({
            id: "ws-latest-list",
            collection: this.collection
        });
    },
    handleViewTab: function(e) {
        var node   = $(e.currentTarget),
            target = node.find('a').attr('data-tabtarget');

        node.parent().find('li').removeClass('active');
        $(target, this.node).parent('.tab-group').children().hide();
        $(target, this.node).show();

		ws.currentHomeTab=null;
		if (target=='#ws-latest-list') {
			if (ws.config.rollingNews.enabled) {
				ws.currentHomeTab = 'rolling_news';
			} else {
				ws.currentHomeTab = 'latest_news';
			}
		}

        node.addClass('active');
		ws.views.renderLastUpdated();
        //ws.views.scrollers.refresh(this.el);
    },
    render: function() {
    	//up navigation
		$('#main .up-navigation').removeClass('active');
		//ws.log('[HOME] rendering view');
    	//add homeview tracking
    	ws.trackView('INDEX', ws.config.serviceRoot);

        this.node.html(this.template(ws.config)); // Render template and mixin vocab

		ws.activeStoryListViewId = this.topStoriesView.cid;

        this.node.find('#ws-top-news-list').replaceWith(this.topStoriesView.render().el);
        this.node.find('#ws-latest-list').replaceWith(this.recentStoriesView.render().el);

        // Set default 'Top Stories/Recent Stories' tab
        if (ws.prefs('defaultHomeTab') == "latestnews") {
            this.node.find('a[data-tabtarget="#ws-latest-list"]').trigger('click');
        } else {
            this.node.find('a[data-tabtarget="#ws-top-news-list"]').trigger('click');
        }

        return this;
    }
});


ws.viewsAndroid.categoryListView = Backbone.View.extend({
    id: 'ws-categories',
    initialize: function() {
		//ws.log('[INIT CATEGORY LIST VIEW]');
        _.bindAll(this, "goToCategory", "render"); // to solve the 'this' issue
    },
    goToCategory: function(genreId) {

		// check if app enabled via config.json
		if (ws.env.remoteConfig.appEnabled == false) { return false; }

		//ws.log('from categorised list');
        var node = $(this.el);
        var renderedList = new ws.views.generalStoryList({
            collection: ws.stories.fetchGenre(genreId),
            id: 'ws-categorised-list'
        }).render().el;

        $('#ws-categorised-list').remove();
        node.after(renderedList);

        _.delay(function() { ws.views._activeMainView.goToCategoryView(); }, 50);
    },
	render: function() {
		//ws.log('[CATEGORY LIST] rendering view');
		//up navigation
		$('#main .up-navigation').removeClass('active');
        var self       = this,
            node       = $(this.el),
            listNode   = $('<ul id="ws-categorylist" class="active"></ul>'),
            output     = "",
            genreOrder = ws.store('genreOrder');

        _.each(genreOrder, function(selected, genreId) {
            var classname = selected ? 'is-offline' : '';

            var genre        = ws.config.availableGenres[genreId],
                tempNode     = $('<li class="'+classname+'">'+genre.title+'</li>');

            tempNode.bind('tap', function() {
				if (!$(this).hasClass('is-offline') && ws.env.status == 'offline') { return false; }
                self.goToCategory(genreId);
				ws.currentGenre = genreId;
				ws.currentHomeTab = null;

            });

            listNode.append(tempNode);
        });

        node.html(listNode);
		//node.html(listNode).prepend(ws.views.templates.pulldown);

        ws.views.scrollers.refresh(node);

		// add footer
		$("#ws-page-footer").remove();
		$('#ws-categorylist').after('<div id="ws-page-footer">BBC &copy; 2012</div>');
        return this;
    }
});


ws.viewsAndroid.popularView = Backbone.View.extend({
    id: 'ws-popular',
    events: {
        "click #ws-view-chooser li": "handleViewTab"
    },
    initialize: function() {
		//ws.log('[INIT POPULAR VIEW]');
        _.bindAll(this, "handleViewTab", "render"); // to solve the 'this' issue
        this.node      = $(this.el);
        this.template  = _.template($.ajax({ type:'GET', url:'templates/popular-android.html', async:false }).responseText);

        this.mostReadView    = new ws.views.mostPopularList({ collection: ws.popularStories.read, id: 'ws-most-read', attributes: {'data-groupgenre': 'most_read'}  });
        this.mostWatchedView = new ws.views.mostPopularList({ collection: ws.popularStories.watched, id: 'ws-most-watched', attributes: {'data-groupgenre': 'most_watched'}  });
    },
    handleViewTab: function(e) {

		if (ws.popularStories.read.length < 1 && ws.env.status == 'offline') { return false; }

        var node   = $(e.currentTarget),
            target = node.find('a').attr('data-tabtarget');

        node.parent().find('li').removeClass('active');
        $(target).parent('.tab-group').children().hide();
        $(target).show();
		if (target == '#ws-most-read') {
			ws.currentPopularTab = 'read';
		} else {
			ws.currentPopularTab = 'watched';
		}

        node.addClass('active');
        ws.views.scrollers.refresh(this.el);
    },
    switchOff: function(feature) {
        if (feature === 'mostRead') { this.$('.most-read-tab').hide(); }
        if (feature === 'mostWatched') { this.$('.most-watched-tab').hide(); }
    },
    render: function() {
		//ws.log('[POPULAR] rendering view');
    	//up navigation
		$('#main .up-navigation').removeClass('active');
        this.node.html(this.template(ws.config)); // Render template and mixin vocab

		this.node.find('.ws-status-message').hide();

        // Hide tabs if any of the feeds are switched off
        if (ws.config.livestats.mostRead === false || ws.config.livestats.mostWatched === false) {
            this.switchOff('mostRead');
            this.switchOff('mostWatched');
        }

		// if offline the livestats won't show if it hasn't been looked at already online, so this displays message in that case
		if (ws.popularStories.read.length < 1 && ws.env.status == 'offline') {

			this.node.find('.ws-status-message').show();
			this.node.find('#ws-view-chooser li:first').addClass('active');

		} else if (ws.env.remoteConfig.appEnabled == false) {

			this.node.find('#ws-view-chooser li:first').addClass('active');

		} else {

			if (ws.popularStories.read.length == 0) {
				ws.popularStories.read.fetch({
					success: function(data) {
						$.publish('/feeds/downloadcomplete');
						var genreLUKey = 'lastUpdatedread';
						ws.store(genreLUKey, new Date());
					}
				});
			}
			if (ws.popularStories.watched.length == 0) {
				ws.popularStories.watched.fetch({
					success: function(data) {
						$.publish('/feeds/downloadcomplete');
						var genreLUKey = 'lastUpdatedwatched';
						ws.store(genreLUKey, new Date());
					}
				});
			}

			this.node.find('#ws-most-read').replaceWith(this.mostReadView.render().el);
			this.node.find('#ws-most-watched').replaceWith(this.mostWatchedView.render().el);

			this.node.find('a[data-tabtarget="#ws-most-read"]').trigger('click');
		}

        return this;
    }
});


ws.viewsAndroid.settingsView = Backbone.View.extend({
    id: 'ws-settings',
    events: {
        'tap .genre-list li': 'toggleGenre',
        'tap .share-stats'  : 'toggleStats',
		//'tap .set-default-tab'  : 'setDefaultTab',//moved into init
        'tap #select-default-home-tab' : 'openPopOverSetDefaultTab',
        'tap .#select-default-dialect'  : 'openPopOverSetDialect',
        'tap #category-order' : 'showCategoryOrder',
        'tap #order-category-popover .button button' : 'hideCategoryOrder',
        'tap #order-category-popover' : 'hideCategoryOrder'
    },
    initialize: function() {
    	var self = this,
    		tapTimeStamp = false,
    		dialectStr = ws.service;

    	_.bindAll(this, "showCategoryOrder", "saveSortOrder", "openPopOverSetDefaultTab", "setDialect");
        this.node = $(this.el);
        this.template  = _.template($.ajax({ type:'GET', url:'templates/settings-android.html', async:false }).responseText);

    },
    render: function() {
    	//up navigation
		$('#main .up-navigation').removeClass('active');
        var node = this.node,
            self = this;

        var viewVars = ws.config;
        viewVars.version = ws.version;
        node.html(this.template(viewVars));

        // Create sortable list
        var genreListHtml = '<li class="topnews selected locked">'+ws.config.vocab.topNews+'<span class="handle" /></li>',
            numListHtml   = '<li>1</li>',
            genreOrder    = ws.store('genreOrder'),
            count         = 2;


        _.each(genreOrder, function(selected, genreId) {
            var genre     = ws.config.availableGenres[genreId],
                className = selected === true ? 'selected' : '';

            numListHtml   += '<li>'+count+'</li>';
            genreListHtml += '<li id="'+genreId+'" class="'+className+' sortable">'+genre.title+'<span class="handle" /></li>';
            count++;
        });
        // make sure settings is at least fullscreen
        var screenH = ($('.ws-scrollable').height());
        $('#ws-settings').css({'min-height': screenH + 'px'});

        node.find('.genre-num').html(numListHtml);
        node.find('.genre-list').html(genreListHtml);


        $('#order-category-popover').css({'height':screenH + 'px'});

        if (ws.prefs('shareStats')) { node.find('.share-stats').addClass('active'); }

		if (ws.prefs('defaultHomeTab') == 'topnews') {
			node.find('.set-default-tab#top-news').addClass('active');
		} else {
			node.find('.set-default-tab#latest-news').addClass('active');
		}

        // set the dialect
		if (ws.config.dialects) {
	        _.each(ws.config.dialects, function(i, k) {
	            //ws.log(i, k);
				node.find('#select-default-dialect-popover ul').append('<li id="' + k + '" class="set-dialect set-default-tab"><span>' + i + '</span></li>')
	        });
			$('#' + ws.prefs('defaultDialect')).addClass('active');
		}


		//catch legal list link tap event to prevent when app is online
		node.find('.legal-list li').bind('click', function(e) {
			if (ws.env.status == 'offline') { e.preventDefault(); }
		});

        return this;
    },
    toggleStats: function(e) {
        var node = $(e.target);
        node.toggleClass('active');

        node.hasClass('active') ? ws.prefs('shareStats', 1) : ws.prefs('shareStats', 0)
    },
    toggleGenre: function(e) {

		if (ws.env.remoteConfig.appEnabled == false) { return false; }

		var node = $(e.target);
		if (node.hasClass('locked')) return;
		node.toggleClass('selected');

		if (node.hasClass('selected')) {
			// If the user selects a genre which is not in the local story index,
			// go off and fetch the index from the server
			var genreId = node.attr('id');
			if (_.isUndefined(ws.storyIndex[genreId])) { ws.stories.fetchGenre(genreId); }
		}

		this.saveSortOrder();

    },
    openPopOverSetDefaultTab : function (e) {
    	var popover = $('#select-default-tab-popover').detach(),
    		popoverMargin = 0,
    		popbox = false,
    		self = this;
    	$('#main').append(popover);
    	popover = $('#select-default-tab-popover');
    	popbox = popover.find('.pop-over');
    	popoverMargin = ($(window).height() - popbox.outerHeight()) / 2;
    	popbox.css({'margin-top': popoverMargin + 'px'});
    	popover.delay(300).addClass('visible');

    	popover.bind('tap', function (e) {
    		e.stopPropagation();
    		if ($(e.target).attr('id') != 'select-default-tab-popover') {
    			return;
    		}
    		_.delay(function () {
    			var popover = $('#select-default-tab-popover').removeClass('visible').delay(600).detach();
    			$('#ws-settings').append(popover);

    		}, 300);
    	});

    	$('#select-default-tab-popover .pop-over').bind('click', function (e) {
    		e.stopPropagation();
        	_.delay(function () { self.setDefaultTab(e); }, 300);
        });
    },
	setDefaultTab: function(e) {
		// store 'topnews' for top news, 'latestnews' for latest news
		// ws.config.defaultHomeTab set in service.js needs to be overriden if this exists
		var node = $(e.target);
		if (node.is('span')) {
			var par = node.parent();
			if (!par.hasClass('active')) {
				par.siblings(".active").removeClass('active');
				par.addClass('active');
				if ($('.set-default-tab.active').attr('id') == 'top-news') {
					ws.prefs('defaultHomeTab', 'topnews');
				} else {
					ws.prefs('defaultHomeTab', 'latestnews');
				}
			}
		}
		//replace the overlay
		if (node.is('button')) {
			var popover = $('#select-default-tab-popover').removeClass('visible').delay(600).detach();
			$('#ws-settings').append(popover);
		}
	},
    saveSortOrder: function(el) {
        var node = $(this.el);
        var sortedItems = node.find('.genre-list').sortable('toArray'),
            finalList = {};

        _.forEach(sortedItems, function(genreId) {
            finalList[genreId] = node.find('#'+genreId).hasClass('selected')
        });

        ws.genreOrder = finalList;
        ws.store('genreOrder', finalList);
    },
    openPopOverSetDialect : function (e) {
    	var popover = $('#select-default-dialect-popover').detach(),
    		popoverMargin = 0,
    		popbox = false,
    		self = this;
    	$('#main').append(popover);
    	popover = $('#select-default-dialect-popover');
    	popbox = popover.find('.pop-over');
    	popoverMargin = ($(window).height() - popbox.outerHeight()) / 2;
    	popbox.css({'margin-top': popoverMargin + 'px'});
    	popover.delay(300).addClass('visible');

    	popover.bind('tap', function (e) {
    		e.stopPropagation();
    		if ($(e.target).attr('id') != 'select-default-dialect-popover') {
    			return;
    		}
    		_.delay(function () {
    			var popover = $('#select-default-dialect-popover').removeClass('visible').delay(600).detach();
    			$('#ws-settings').append(popover);

    		}, 300);
    	});

    	$('#select-default-dialect-popover .pop-over').bind('tap', function (e) {
    		e.stopPropagation();
        	self.setDialect(e);
        });
    },
    setDialect: function(e) {
    	var self = this;
        if (ws.env.remoteConfig.appEnabled == false) { return false; }

        var node = $(e.target);
		if (node.is('span')) {
			var par = node.parent();
			//if (!par.hasClass('active')) {
				par.siblings(".active").removeClass('active');
				par.addClass('active');

				var str = ws.service.replace(ws.prefs('defaultDialect'), par.attr('id'));
				self.dialectStr = str;
				console.log('change:',self.dialectStr);

			//}
		}
		//replace the overlay
		if (node.is('button')) {
			var popover = $('#select-default-dialect-popover').removeClass('visible').delay(600).detach();
			$('#ws-settings').append(popover);
			ws.service = self.dialectStr;
			console.log('set:',self.dialectStr);
			//change the dialect
	        if (ws.prefs('defaultDialect') && ws.config.dialects && ws.env.status == "online") {
	            if (!ws.service.match(ws.prefs('defaultDialect'))) {
		            var overlayNode = $('#overlay');
		            overlayNode.empty().append('<div></div>').addClass('loading').addClass('visible');
		            ws.loadNewService(ws.service, 'settings');
	            }
	        }
		}
    },
    showCategoryOrder: function (e) {
    	// Should probably be in its own view
		var self = this;
    	if ($('#order-category-popover').addClass('visible')) {
    		var contain = $('#order-category-popover'),
	    		viewContainerId = this.node.parents('.ws-scrollable').attr('id'),
	    		orderer = contain.removeClass('hidden'),
	    		scroller = {},
	    		headerHeight = this.node.parents('.ws-scrollable').find('.fullpage .ws-settings-header').height(),
	    		footerHeight = contain.find('.button').height();

    		$('#order-category-popover-popover').css({'height': (contain.height() - (footerHeight)) + "px"});

    		scroller = new iScroll('order-category-popover-popover', {
                desktopCompatibility:false,
                useTransition: true,
                vScrollbar: true,
                hScrollbar: false,
                fadeScrollbar: true,
                hideScrollbar: true,
                hScroll: false,
                bounce: false,
                fixedScrollbar: true,
				scrollbarClass: 'scrollbar-',
                lockDirection: true
    		});
    		scroller.contId = false;

	    	orderer.data('scroller',scroller);
	    	ws.views.scrollers._list['order-category-popover'] = scroller;
	    	// disable parent container scrolling
	    	ws.views.scrollers._list[viewContainerId].disable();

	    	contain.find('.genre-list').sortable({
	            opacity: 1,
	            // delay: 100,
	            items:  '.sortable',
	            handle: '.handle',
	            containment: 'parent',
	            axis:   'y',
	            create: function(event, ui) {
	            	//ws.views.scrollers._list[$(this).parents('.ws-scrollable').attr('id')].disable();
	            },
	            start: function(event, ui) {
	            	scroller.disable();
	                //ws.views.scrollers._list[this.viewContainerId].disable();
	            },
	            stop: function(event, ui) {
	            	scroller.enable();
	                //ws.views.scrollers._list[this.viewContainerId].enable();
	            },
	            update: function(event, ui) {
	                self.saveSortOrder(this);
	                //self.render();
	            }
	        });

	    	//var sortHandle = contain.find('.sortable span.handle');
	    	//disable scrolling on touches
		    /*sortHandle.live('touchstart', function (e) {
		    	scroller.disable();
		    }).live('touchend', function (e) {
		    	scroller.enable();
		    });*/
    	} else {
    		return false;
    	}
    	scroller.refresh();

    	$('#order-category-popover-popover').css({'height': ($('#order-category-popover-popover').height() + footerHeight) - 3 + "px"});// -3 for the border

    },
    hideCategoryOrder: function (e) {
    	var target = $(e.target);
    	e.stopPropagation();
    	var viewContainerId = this.node.parents('.ws-scrollable').attr('id')
		scroller = ws.views.scrollers._list['order-category-popover'];
    	ws.views.scrollers._list[viewContainerId].enable();

    	_.delay(function () {// delay to prevent the elements below firing a touch event
    		if (target.is('#order-category-popover') || target.is('#order-category-popover .button button')) {
    			$('#order-category-popover').removeClass('visible');
    			//remove scroller
    			scroller.destroy();
    			$('#order-category-popover').data("scroller",{});
    			scroller = null;
    		}
    	}, 300);
    }
});




/*******************************************************************************
 * Mini views go here
 ******************************************************************************/

$.extend(ws.views,ws.viewsAndroid);
