ws.views = {
    scrollers:    {},
    templates:    {},
    init: function() {
        // Set underscore template settings to accept the curlies {variable}
        _.templateSettings = { interpolate : /\{(.+?)\}/g };

        // Loads the CSS files and prepares the layout
        $("body").attr("id", ws.service);
        if (ws.env.retina) { $("body").addClass("retina"); }
        ws.views.loadCSS();
        ws.views.loadTemplates();
        ws.views.scrollers.init();

        // Event subscriptions
        $.subscribe("/status/offline", function(){
            // Update UI to show offline status
            //ws.views.notify("Lost connection<br/>gone offline");
			//ws.log('gone offline');
            $('body').addClass('offline');

            $('#ws-refresh, .pulldown').removeClass('activity');
        });

        $.subscribe("/status/online", function(){
            // Update UI to show online status
            //ws.views.notify("Gone online");
			//ws.log('gone online');
            $('body').removeClass('offline');
        });

        $.subscribe("/status/disabled", function(){
            // Update UI to show offline status
            //ws.views.notify("Lost connection<br/>gone offline");
			//ws.log('Disabled');
            $('body').addClass('disabled');
        });

        $.subscribe("/status/enabled", function(){
            // Update UI to show online status
            //ws.views.notify("Gone online");
			//ws.log('Enabled');
            $('body').removeClass('disabled');
        });

        $.subscribe("/feeds/downloadstarted", function() {
			//ws.log('download started');
            $('#ws-refresh').addClass('activity');
            ws.env.downloading = true;
            // After 15 seconds, timeout activity spinners
            _.delay(function() {
                $('#ws-refresh, .pulldown').removeClass('activity');
            }, 15000);
        });

        $.subscribe("/feeds/downloadcomplete", function() {
			//ws.log('download complete');
            ws.env.downloading = false;
            $('#ws-refresh, .pulldown').removeClass('activity');
            $('#overlay.loading div').empty();
            $('#overlay.loading').removeClass('loading').removeClass('visible');
			//ws.views.renderLastUpdated();
            ws.views.scrollers.refresh();
        });


        // Convert header buttons
        $('.ws-header button').each(function() {
            var node  = $(this),
                label = node.text();

            node.html('<span>'+label+'</span><span class="end"></span>');
        });


        // Don't allow scrolling of the main app (headers/footers)
        document.addEventListener('touchmove', function(e){
            e.preventDefault();
        });

    },
    loadCSS: function() {
        var cssRoot = "styles/",
            cssHtml,
            cacheBuster = new Date().getTime();

        cssHtml = '<link href="'+cssRoot+'core.css?'+cacheBuster+'" type="text/css" rel="stylesheet" />'
                + '<link href="'+cssRoot+ws.config.dir+'.css?'+cacheBuster+'" type="text/css" rel="stylesheet" />'
                + '<link href="'+cssRoot+'type.css?'+cacheBuster+'" type="text/css" rel="stylesheet" />'
                + '<link href="'+cssRoot+'services/'+ws.service+'.css?'+cacheBuster+'" type="text/css" rel="stylesheet" />';


        $("head").append(cssHtml);
    },

    loadTemplates: function() {
        ws.views.templates.storyRow =  $.ajax({ type:'GET', url:'templates/story_row.html', async:false }).responseText;

		// Render pulldown HTML once here, since the markup never changes
		var pulldownTemplate = _.template($.ajax({ type:'GET', url:'templates/pulldown.html', async:false }).responseText);
		ws.views.templates.pulldown = pulldownTemplate(ws.config);

		// Add to config object, so it can easily be inserted into other templates alongside the vocab
		ws.config.pulldown = ws.views.templates.pulldown;

        $("[data-template]").each(function() {
            var node = $(this),
                templateName = node.attr("data-template"),
                templatePath = "templates/"+templateName+".html",
                templateContent = $.ajax({ type:'GET', url:templatePath, async:false }).responseText;

            node.html(templateContent);
        });
    },
    switchTo: function(to, reverse) {
		//ws.log('[SWITCHTO]');
        //ws.log($(to).siblings('.page'));
        //var from = $('.page.active');
        var from = $(to).siblings('.active');
        $(to).addClass('active slide in');
        from.addClass('slide out');

        // Helps to refresh scrollers here, in case the
        // destination page has the pull-to-refresh showing
        ws.views.scrollers.refresh(this.el);

        if (reverse == true) {
            $(to).addClass('reverse');
            from.addClass('reverse');
        }
        _.delay(function() {
            from.removeClass('active reverse slide out');
            $(to).removeClass('reverse slide in');
            $(to).addClass('active'); // Be double sure that the destination page goes active
        }, 350);
    },

	renderLastUpdated: function(genreId) {
		var genre = null;

		if (genreId) {genre=genreId;}
		else if (ws.currentHomeTab=='rolling_news') {genre='rolling_news';}
		else if (ws.currentHomeTab=='latest_news') {genre='latest_news';}
		else if (ws.currentPopularTab) {genre=ws.currentPopularTab;}
		else {}

		var updated = new Date();
		if (genre && ws.store('lastUpdated'+genre)) {updated=ws.store('lastUpdated'+genre);}
		else if (ws.store('lastUpdated')) {updated=ws.store('lastUpdated');}
		else {}

		//updated = (genre && ws.store('lastUpdated'+genre)) ? ws.store('lastUpdated'+genre) : ws.store('lastUpdated'),
		niceDate = new Date(Date.parse(updated)).format(ws.config.shortDateTime);

		//ws.log(genre,".renderLastUpdated")
		_.delay(function() {$('.pulldown .time').text(niceDate);}, 500);

	},

    updateViewVocab: function(context) {

        ws.config.pulldown = ws.views.templates.pulldown;

        if (context != 'init') {
            // re-render views
            ws.views._activeMainView.viewToRender = new ws.views.settingsView();
            ws.views._activeMainView.render();

            // rebuild bottom nav
            $('.ws-home a').html('<span></span>' + ws.config.vocab.tabNews);
            $('.ws-popular a').html('<span></span>' + ws.config.vocab.tabPopular);
            $('.ws-categories a').html('<span></span>' + ws.config.vocab.tabCategories);
            $('.ws-settings a').html('<span></span>' + ws.config.vocab.tabSettings);

        } else {
            $('#ws-view-chooser a[data-tabtarget="#ws-top-news-list"] span').html(ws.config.vocab.topNews);
        }

    },

    launchLiveAV: function(playlist) {

        if (ws.env.status == 'online' && ws.env.remoteConfig.appEnabled == true && ws.config.liveVideo) {
			if (ws.nativeVideoPlayer == true) {
                cordova.plugins.LandscapePlayer.playVideo(playlist);
			}
		}

    },

    scrollers: {
        _list: {},
        _refreshCounter: 0,
        init: function() {
            $('#scroll-to-top').bind('tap', function() {
                var activeScroller = $('.page.active .ws-scrollable').attr('id');
                ws.views.scrollers._list[activeScroller].scrollTo(0, 0, 200);
            });
        },
        drawScrollablePage: function(selector) {
            var node        = $(selector),
                isVisible   = node.is(':visible'),
                contentNode = node.find('.ws-content');

            /*if (contentNode.length == 0) {
                node.wrapInner('<div class="ws-content" />');
                contentNode = node.find('.ws-content');
            }*/
            var id = contentNode.attr('id') ? contentNode.attr('id') : ws.utils.uid();


            if (!isVisible) { node.show(); } // Needed to properly calculate heights
            var pulldownNode        = contentNode.find('.pulldown'),
                pulldownLabel       = pulldownNode.find('.label'),
                pullDownOffset      = pulldownNode.height(),
                scrollWrapperHeight = $(window).height() - node.find(".ws-header").height() - node.find(".ws-footer").height();


            //ws.log(node.find(".ws-header").height(), node.find(".ws-footer").height());
            contentNode.height(scrollWrapperHeight).css('overflow', 'hidden');
            if (!isVisible) { node.hide(); } // Re-hide previously hidden content

            // Draw iScroll
            contentNode.attr('id', id);
            contentNode.wrapInner('<div class="ws-content" />').removeClass('ws-content');
            if (ws.views.scrollers._list[id]) { ws.views.scrollers._list[id].destroy(); }


            var scroller = new iScroll(id, {
                desktopCompatibility:false,
                useTransition: true,
                vScrollbar: true,
                scrollbarClass: 'scrollbar-',
                topOffset: pullDownOffset,
                onScrollStart: function() {
                    contentNode.addClass('scrolling');
                    pulldownNode = contentNode.find('.pulldown');
                },
                onScrollMove: function() {
                    if (this.y > 5 && !pulldownNode.hasClass('release-to-refresh')) {
                        pulldownNode.addClass('release-to-refresh');
                        pulldownLabel.html(ws.config.vocab.releaseToRefresh);
                        this.minScrollY = 0;
                        this.hasPulldownAction = true;
                    } else if (this.y < 5 && pulldownNode.hasClass('release-to-refresh')) {
                        pulldownNode.removeClass('release-to-refresh');
                        pulldownLabel.html(ws.config.vocab.pullToRefresh);
                        this.minScrollY = -pullDownOffset;
                        this.hasPulldownAction = false;
                    }
                },
                onScrollEnd: function() {
                    contentNode.removeClass('scrolling');

                    if (this.hasPulldownAction && pulldownNode.is(":visible")) {
                        //ws.log('pull down starts here');
                        pulldownNode.removeClass('release-to-refresh').addClass('activity');
                        if (!$('body').hasClass('offline')) {
							//_.delay(function() { ws.stories.fetchAllStories(); }, 500);
							if (ws.currentGenre) {
								//ws.log('refresh categories');
								_.delay(function() {ws.stories.fetchGenre(ws.currentGenre); }, 500);
							} else if (ws.currentPopularTab=='read' || ws.currentPopularTab=='watched') {
								// check if app enabled via config.json
								if (ws.env.remoteConfig.appEnabled == false) { return false; }
								//ws.log('refresh most read and most watched');
								ws.popularStories.read.fetch({
									success: function(data) {
										$.publish('/feeds/downloadcomplete');
										var genreLUKey = 'lastUpdatedread';
										ws.store(genreLUKey, new Date());
										_.delay(function() {ws.views.renderLastUpdated('read'); }, 500);
									}
								});
								ws.popularStories.watched.fetch({
									success: function(data) {
										$.publish('/feeds/downloadcomplete');
										var genreLUKey = 'lastUpdatedwatched';
										ws.store(genreLUKey, new Date());
										_.delay(function() {ws.views.renderLastUpdated('watched'); }, 500);
									}
								});
							} else {
								// ws.log('refresh everything including rolling news');
								_.delay(function() { ws.stories.fetchAllStories(); }, 500);
							}
                         } else {
							// end pulldown anim after a few seconds
                        	_.delay(function() {
					            $('#ws-refresh, .pulldown').removeClass('activity');
					            ws.views.scrollers.refresh();
                        	}, 1500);
                        }
						_.delay(function() {
							$('#ws-refresh, .pulldown').removeClass('activity');
							ws.views.scrollers.refresh();
						}, 10000);
                        this.hasPulldownAction = false;
                    }
                }
            });
            scroller.doLayout = function() {
                // Recalculates the scrolling height region
                //ws.log('doing layout');
                var scrollWrapperHeight = $(window).height() - node.find(".ws-header").height() - node.find(".ws-footer").height();
                contentNode.height(scrollWrapperHeight);
            }

            ws.views.scrollers._list[id] = scroller;
            return scroller;
        },
        refresh: function(sourceEl) {
            // Forces a refresh on all the iScroll instances
            //ws.log('refreshing scrollers ');
            $.each(ws.views.scrollers._list, function(i, val) {
                val.options.topOffset = $(val.wrapper).find('.pulldown:visible').height();
                if (val.options.topOffset > 0) {
                    val.hasPulldown = true;
                    //val.scrollTo(0, -val.options.topOffset, 0);
                } else {
                    val.hasPulldown = false;
                }

                val.refresh();
            });
        }
    }
}




/* Backbone driven views
/* Main and Story views here
************************************** */
ws.views.MainView = Backbone.View.extend({
    el: "#main",
    events: {
        "tap #ws-view-chooser li": "handleViewTab", // duplication??? this funciton already in homeview and popularview
        "tap #ws-tab-bar li":      "handleBottomTabBar",
        "tap .back":               "backToMainView",
        "tap #ws-refresh":         "refresh",
        "tap #ws-live button":     "launchLiveStream"
    },
    initialize: function() {
		//ws.log('[INIT MAINVIEW]');
        var self = this;
        _.bindAll(this, "render"); // to solve the 'this' issue
        //this.collection.bind("change", this.render);
        this.node = $(this.el)

        this.template = _.template($(this.el).html());

        this.render();
    },

    handleViewTab: function(e) {
        var node   = $(e.currentTarget),
            target = node.find('a').attr('data-tabtarget');

        node.parent().find('li').removeClass('active');
        $(target).parent('.tab-group').children().hide();
        $(target).show();

        node.addClass('active');
		ws.views.renderLastUpdated();
        ws.views.scrollers.refresh(this.el);
    },

    handleBottomTabBar: function(e) {
        var self   = this,
            node   = $(e.currentTarget),
            target = node.attr('data-tabtarget');

        // remove live av dropdown if it exists and is visible
        if ($('#avSelectOverlay').hasClass('visible')) {
            $('#avSelectOverlay').removeClass('visible');
            $('#ws-live').removeClass('active');
        }

        switch(target) {
            case "#ws-home":       this.viewToRender = new ws.views.homeView({ collection: ws.stories }); break;
            case "#ws-popular":    this.viewToRender = new ws.views.popularView(); break;
            case "#ws-categories": this.viewToRender = new ws.views.categoryListView(); break;
            case "#ws-settings":   this.viewToRender = new ws.views.settingsView(); break;
        }

        $(this.el).removeClass('category-view-active');
        $(this.el).removeClass(function(index, css){
            return (css.match (/\bcurrent-genre-\S+/g) || []).join(' ');
        });
		$('#ws-banner').find('span').text('');

		ws.views.renderLastUpdated();
        this.render();
    },

    goToCategoryView: function() {

		// check if app enabled via config.json
		if (ws.env.remoteConfig.appEnabled == false) { return false; }

        var self = this;
		if (ws.currentGenre != null) {
			this.vGenre = ws.config.availableGenres[ws.currentGenre] ? ws.config.availableGenres[ws.currentGenre].title : "";
			$('#ws-banner').find('span').text(this.vGenre);
            $(this.el).addClass('current-genre-' + ws.currentGenre);
		}
        //this.enableBackButton();
        $(this.el).addClass('category-view-active');

        $('#ws-tab-bar li').removeClass('active');
        $('#ws-tab-bar .ws-categories').addClass('active'); // Activate relevant tab
        //$('#ws-categorised-list').addClass('active');

        //Don't think we need this delay it causes a jump
		//_.delay(function() {
	        _.delay(function() { ws.views.renderLastUpdated(ws.currentGenre); }, 50);
            self.scroller.scrollTo(0,0,0);
			ws.views.scrollers.refresh(self.el); //this seems to work better than self.scroller.refresh();
            //self.scroller.refresh();
        //}, 500);
    },

    backToMainView: function() {
        var self = this,
            backButton = $(this.el).find('.back');
        //if (!backButton.hasClass('enabled')) return false;

        $(this.el).removeClass('category-view-active');
        // remove category class if it exists
        $(this.el).removeClass(function(index, css){
            return (css.match (/\bcurrent-genre-\S+/g) || []).join(' ');
        });

        //on back button change tab to main only if we're back on the homepage, not the catgory list
		if ($('#ws-categories').length < 1) {
			$('#ws-tab-bar li').removeClass('active');
			$('#ws-tab-bar .ws-home').addClass('active');
		}

        backButton.removeClass('enabled');

		// reset env variables
        ws.currentGenre = null;
        ws.currentHomeTab = null;
		ws.currentPopularTab = null;
        $('#ws-banner').find('span').text('');

        _.delay(function() {
			$('#ws-categorised-list').remove();
			//ws.views.renderLastUpdated();
			//ws.views.scrollers.refresh();
            ws.views.scrollers.refresh(self.el);
            //self.scroller.scrollTo(0,0,0);
		}, 500);

        return false; //prevent propagation

    },

    refresh: function() {
		if (ws.env.status == 'online') {
			$('#ws-refresh').addClass('activity');
			ws.stories.fetchAllStories();
		}
    },

    launchLiveStream: function(e) {
        //ws.log('[launchLive]');

        if (ws.env.status != 'online' || ws.env.remoteConfig.appEnabled != true || !ws.config.liveVideo) {
            return;
        }

        var thisTarget = $('#ws-live');

        thisTarget.addClass('active');

        if ($('#avSelectOverlay').length > 0) {

            if ($('#avSelectOverlay').hasClass('visible')) {
                $('#avSelectOverlay').removeClass('visible');
                thisTarget.removeClass('active');
                return false;
            } else {
                $('#avSelectOverlay').find('li').removeClass('active');
                $('#avSelectOverlay').addClass('visible');
            }

        } else {

            var self =  this,
                        avSelectHtml = '<div id="avSelectOverlay"><ul><li class="live-video">'+ ws.config.vocab.liveTv +'<span></span></li><li class="live-audio">'+ ws.config.vocab.liveRadio +'<span></span></li></ul></div>';

            $('.ws-header').after(avSelectHtml);
            _.delay(function(){$('#avSelectOverlay').addClass('visible');}, 50);

            $('#avSelectOverlay').bind('tap', function(e) {
                $('#avSelectOverlay').removeClass('visible');
                thisTarget.removeClass('active');
                return false;
            });

            $('#avSelectOverlay').find('.live-video').bind('tap', function(e) {
                //ws.log('[launchLiveVideo]');
                $(this).addClass('active');
                $('#avSelectOverlay').removeClass('visible');
                thisTarget.removeClass('active');
                ws.views.launchLiveAV(ws.config.liveVideo);
                return false;
            });

            $('#avSelectOverlay').find('.live-audio').bind('tap', function(e) {
                ws.log('[launchLiveAudio]');
                $(this).addClass('active');
                $('#avSelectOverlay').removeClass('visible');
                thisTarget.removeClass('active');
                ws.views.launchLiveAV(ws.config.liveRadio);
                return false;
            });
        }
    },

    render: function() {
        //ws.log('[MAIN] Rendering view');
        var self = this;

        // Create iScroll if it doesn't exist already
        var scrollableNodeId = $(this.el).find('.ws-scrollable').attr('id');
        if (_.isUndefined(scrollableNodeId)) {
            this.node.html(this.template(ws.config)); // Render template and mixin vocab
            this.scroller = ws.views.scrollers.drawScrollablePage(this.el);
            this.contentNode = this.node.find('.ws-content');
        }


        // If no subview has been specified, default to the Home one
        if (!this.viewToRender) { this.viewToRender = new ws.views.homeView({ collection: ws.stories }); }

        // Highlight appropiate tab button
        var viewId = this.viewToRender.el.id;
        $('#ws-tab-bar li').removeClass('active');
        $('#ws-tab-bar .'+viewId).addClass('active'); // Activate relevant tab

        // Hide Most Popular tab, if both features are off
        if (ws.config.livestats.mostRead === false && ws.config.livestats.mostWatched === false) {
            $(this.el).find('.ws-popular').hide();
        }

        // Render subview
        this.contentNode.html(this.viewToRender.el);
        this.viewToRender.render();

        _.delay(function() {
            //self.scroller.refresh();
            ws.views.scrollers.refresh();
            self.scroller.scrollTo(0,0,0);
        }, 100);

        //ws.log('home done');
        return this;
    }
});


ws.views.StoryView = Backbone.View.extend({
    el:               "#story",
    doPageTransition: true,
    events: {
        "tap #story-navigation .prev":      "previous", // Tap doesn't work too well on next and previous either
        "tap #story-navigation .next":      "next",
        "swiperight .ws-content":			"catchSwipeRight",
        "swipeleft .ws-content":			"catchSwipeLeft",
        "tap .back":						"back",
        "tap .video-container":				"playVideo",
		"tap .audio-container":				"playAudio",
        "tap .ws-text-size a":				"handleTextSize",
        "tap .share":						"shareTools",
        "tap .picture-gallery-container":   "openpicturegallery",
        "tap .ws-story-toolbar":            "preventDefault" // prevent auto zoom in ios5+
    },
    initialize: function() {
        _.bindAll(this, "render", "previous", "next", "back");
        var self = this;

        if (typeof(this.model) == 'undefined') {
            this.model = new ws.models.Story();
        }

        this.template = _.template($(this.el).html());
    },
    preventDefault: function(e) {
        e.preventDefault();
    },
    catchSwipeLeft: function() {
        if (ws.config.dir == 'ltr') {
            $(this.el).find('#story-navigation .next').trigger('tap');
        } else {
            $(this.el).find('#story-navigation .prev').trigger('tap');
        }
    },
    catchSwipeRight: function() {
        if (ws.config.dir == 'ltr') {
            $(this.el).find('#story-navigation .prev').trigger('tap');
        } else {
            $(this.el).find('#story-navigation .next').trigger('tap');
        }
    },
    previous: function() {

		if (this.options.disableNavigation == true) {return false;}

        var self = this;
        var slideto = ws.config.dir == 'ltr' ? 'slidetoright' : 'slidetoleft';
        $(this.el).find('.prev, .next').removeClass('disabled');
        this.storyPosition--;

		//exception for the homepage needs to be refactored so that all next/previous navigation uses ids instead of collections
		if (ws.currentGenre == 'homepage' || ws.currentGenre == 'latestnews') {

            var currentList = ws.currentGenre == 'homepage' ? ws.homepageList : ws.latestList;

			if (this.storyPosition < 0) {
				this.storyPosition = 0;
				$(this.el).find('.prev').addClass('disabled');
			} else {

				var potentialPrevStory = this.storyPosition;

				if (ws.env.status == 'offline') {
					potentialPrevStory = null;
					//catch if offline and the next story doesn't exist
					$.each(currentList, function(i) {
						if (i <= self.storyPosition && ws.store('s'+currentList[i])) {
							potentialPrevStory = i;
						}
						return (i <= self.storyPosition);
					});
				}

				if (potentialPrevStory != null) {

					//go to the previous homepage story
					var destinationStoryId = currentList[potentialPrevStory];
					this.doPageTransition = false;

					//ws.log('go to previous story - ' + potentialPrevStory + destinationStoryId);

					$(this.el).find('.ws-scrollable').addClass(slideto);

					_.delay(function() {
						ws.thisRouter.navigate('story/' + destinationStoryId, true);
						$(this.el).find('.ws-scrollable').removeClass(slideto);
					}, 350);

				} else {
					this.storyPosition++;
					$(this.el).find('.prev').addClass('disabled');
				}
			}

		} else {

			if (this.storyPosition < 0) {
				this.storyPosition = 0;
				$(this.el).find('.prev').addClass('disabled');
			} else if (ws.env.status == 'offline' && !ws.store('s'+this.collection.at(this.storyPosition).id)) {
				this.storyPosition++;
				$(this.el).find('.prev').addClass('disabled');
			} else {
				this.model = this.collection.at(this.storyPosition);
				this.doPageTransition = false;

				//ws.log('go to previous story - ' + this.storyPosition, this.model);

				$(this.el).find('.ws-scrollable').addClass(slideto);

				_.delay(function() {
					var destinationStoryId = self.model.id || self.model.cid;
					ws.thisRouter.navigate('story/' + destinationStoryId, true);
					$(this.el).find('.ws-scrollable').removeClass(slideto);
				}, 350);
			}

		}

        return false; //prevent propagation

    },

    next: function() {

		if (this.options.disableNavigation == true) {return false;}

        var self = this;
        var slideto = ws.config.dir == 'ltr' ? 'slidetoleft' : 'slidetoright';
        $(this.el).find('.prev, .next').removeClass('disabled');
        this.storyPosition++;

		//exception for the homepage needs to be refactored so that all next/previous navigation uses ids instead of collections
		if (ws.currentGenre == 'homepage' || ws.currentGenre == 'latestnews') {

            var currentList = ws.currentGenre == 'homepage' ? ws.homepageList : ws.latestList;

			if (this.storyPosition > currentList.length-1) {
				this.storyPosition = currentList.length-1;
				$(this.el).find('.next').addClass('disabled');
			} else {

				var potentialNextStory = this.storyPosition;

				if (ws.env.status == 'offline') {
					potentialNextStory = null;
					//catch if offline and the next story doesn't exist
					$.each(currentList, function(i) {
						if (i >= self.storyPosition && ws.store('s'+currentList[i])) {
							potentialNextStory = i;
							return (potentialNextStory == null);
						}
					});

				}

				if (potentialNextStory != null) {

					//go to the next homepage story
					var destinationStoryId = currentList[potentialNextStory];
					this.doPageTransition = false;

					//ws.log('go to next story - ' + potentialNextStory, destinationStoryId);

					$(this.el).find('.ws-scrollable').addClass(slideto);

					_.delay(function() {
						ws.thisRouter.navigate('story/' + destinationStoryId, true);
						$(this.el).find('.ws-scrollable').removeClass(slideto);
					}, 350);

				} else {
					this.storyPosition--;
					$(this.el).find('.next').addClass('disabled');
				}

			}

		} else {

			if (this.storyPosition > this.collection.length-1) {
				//catch if the last story of the collection
				this.storyPosition = this.collection.length-1;
				$(this.el).find('.next').addClass('disabled');
			} else if (ws.env.status == 'offline' && (!ws.store('s'+this.collection.at(this.storyPosition).id) || this.storyPosition > ws.config.storiesToOffline-1)) {
				//catch if offline and the next story doesn't exist
				this.storyPosition--;
				$(this.el).find('.next').addClass('disabled');
			} else {
				//go to the next story
				this.model = this.collection.at(this.storyPosition);
				this.doPageTransition = false;

				//ws.log('go to next story - ' + this.storyPosition, this.model);

				$(this.el).find('.ws-scrollable').addClass(slideto);

				_.delay(function() {
					var destinationStoryId = self.model.id || self.model.cid;
					ws.thisRouter.navigate('story/' + destinationStoryId, true);
					$(this.el).find('.ws-scrollable').removeClass(slideto);
				}, 350);
			}

		}

		return false; //prevent propagation

    },

    back: function() {

		/*if (this.options.referrer == 'byUrl') {
			// for stories accessed by related links
			window.history.back();
		} else {*/
			if (ws.currentGenre == 'homepage' || ws.currentGenre == 'latestnews' || ws.currentGenre == 'rollingnews') {
				ws.currentGenre = null;
			}
			ws.thisRouter.navigate('', true);
		//}

        return false; //prevent propagation
    },

    playVideo: function(e) {
        if (ws.env.status == 'online' && ws.env.remoteConfig.appEnabled == true) {
			if (ws.nativeVideoPlayer == true) {
				var videoSrc = $(e.currentTarget).attr('data-source');
                cordova.plugins.LandscapePlayer.playVideo(videoSrc);
			} else {
				var node = $(e.currentTarget).find('video')[0].play();
			}
		}
    },

	playAudio: function(e) {
        if (ws.env.status == 'online' && ws.env.remoteConfig.appEnabled == true) {
			if (ws.nativeVideoPlayer == true) {
				var audioSrc = $(e.currentTarget).attr('data-source');
                cordova.plugins.LandscapePlayer.playVideo(audioSrc);
			} else {
				var node = $(e.currentTarget).find('audio')[0].play();
			}
		}
	},

    handleRelatedLink: function(e) {
        var url       = $(e.target).attr('href'),
            mobileUrl = url.replace(ws.config.serviceRoot, ws.config.serviceRoot+ws.mobileRedirect);

        window.plugins.childBrowser.showWebPage(mobileUrl);
        e.preventDefault();
    },

    handleTextSize: function(e) {
        var action        = _.isUndefined(e) ? '' : $(e.target).parent().attr('data-action'),
            sizeNode      = $(this.el).find('.ws-text-size'),
            storyTextSize = ws.prefs('textSize') || 1;

        sizeNode.find('.disabled').removeClass('disabled');
        switch(action) {
            case "smaller":
                storyTextSize = storyTextSize-1 < 1 ? 1 : storyTextSize-1;
                break;
            case "larger":
                storyTextSize = storyTextSize+1 > 5 ? 5 : storyTextSize+1;
                break;
        }

        if (storyTextSize >= 5) { sizeNode.find('.larger').addClass('disabled'); }
        if (storyTextSize <= 1) { sizeNode.find('.smaller').addClass('disabled'); }
        $(this.el).find('.ws-story-content').attr('class', 'ws-story-content text-size-'+storyTextSize);
        ws.views.scrollers.refresh(this.el);

        ws.prefs('textSize', storyTextSize);

        if (!_.isUndefined(e)) { e.preventDefault(); }
    },

    openpicturegallery: function(e) {
        //ws.log('[OPEN PICTURE GALLERY]');

        if (ws.env.status == 'online' && ws.env.remoteConfig.appEnabled == true) {
            if (!this.pictureGalleryView) {
                this.pictureGalleryView = new ws.views.PictureGallery({gallery: this.gallery}).render();
            } else {
                this.pictureGalleryView.options.gallery = this.gallery;
                this.pictureGalleryView.render();
            }
            ws.views.switchTo('#picture-gallery');
        }

        return false;
    },

    shareTools: function() {
        // Sharetools should probably have its own view code, instead of here

        var self = this,
            overlayNode = $('#overlay'),
            output = $('<div class="sharetools"></div>');

        _.each(ws.config.sharing, function(val, i) {
            if (val.enabled === false) { return; }
            var tmpNode = $('<button class="'+i+'">'+val.title+'</button>');
            tmpNode.click(function() {
                var shareUrlTemplate = _.template(val.shareUrl),
                    shareUrl         = shareUrlTemplate({title: self.model.get('title'), url: self.model.get('href')});

                overlayNode.removeClass('visible');
                if (val.urlRefactor) {
                    shareUrl = shareUrl.replace('bbc.co.uk','bbc.co.uk/'); // in some instances the story url needs to be edited superficially so its not blocked
                }

                // window.location.href = shareUrl;
                window.open(shareUrl, '_system')
            });
            output.append(tmpNode);
        });

        output.append('<button class="cancel">'+ws.config.vocab.cancel+'</button>');
        //output += '</div>';

        overlayNode.empty().append(output).addClass('visible');

        overlayNode.find('.cancel').click(function() {
            overlayNode.removeClass('visible');
            _.delay(function(){overlayNode.empty()},200);
        });
    },

    render: function() {
        //ws.log('[STORY] Rendering view')


        var self = this,
            node = $(this.el);

        if (typeof(this.model) == 'undefined') return false;

        var storyData = this.model.toJSON();

        // Mix in the vocab object
        storyData.vocab = ws.config.vocab;

        // Prettify the output
        if (_.isUndefined(storyData.updated)) {
            storyData.updated = "";
        } else {
            storyData.updated = new Date(Date.parse(storyData.updated)).format(ws.config.dateFormat);
        }

        $(this.el).removeClass(function(index, css){
            return (css.match (/\bsource-genre-\S+/g) || []).join(' ');
        });

        // Apply vernacular genre
        var genreId = storyData.genre;
        //storyData.vGenre = ws.config.availableGenres[genreId] ? ws.config.availableGenres[genreId].title : "";

        if (this.options.referrer == 'byUrl') {
            storyData.vGenre = "";
            if (ws.config.availableGenres[genreId]) {
                storyData.vGenre = ws.config.availableGenres[genreId].title ? ws.config.availableGenres[genreId].title : "";
                $(this.el).addClass('source-genre-' + genreId);
            }
        } else if (ws.genreContext == "latest_stories") {
            // show 'latest stories'
            storyData.vGenre = ws.config.vocab.latestNews;
            $(this.el).addClass('source-genre-' + ws.genreContext);
        } else if (ws.genreContext == "most_read" || ws.genreContext == "most_watched" || ws.genreContext == "top_stories") {
            // show current stories' genre
            storyData.vGenre = "";
            //storyData.vGenre = ws.config.availableGenres[genreId] ? ws.config.availableGenres[genreId].title : "";
            if (ws.config.availableGenres[genreId]) {
                storyData.vGenre = ws.config.availableGenres[genreId].title ? ws.config.availableGenres[genreId].title : "";
                $(this.el).addClass('source-genre-' + genreId);
            }
            if (storyData.vGenre == "") {
                if (ws.config.availableGenres[storyData.topic]) {
                    //check topics if no genre matches
                    storyData.vGenre = ws.config.availableGenres[storyData.topic] ? ws.config.availableGenres[storyData.topic].title : "";
                    $(this.el).addClass('source-genre-' + storyData.topic);
                }
                if (genreId == "rolling_news") {
                    storyData.vGenre = ws.config.rollingNews.title;
                    $(this.el).addClass('source-genre-rolling_news');
                }
            }
        } else {
            // show contextual category user visited from
            storyData.vGenre = ws.config.availableGenres[ws.genreContext].title;
            $(this.el).addClass('source-genre-' + ws.genreContext);
        }

        $(this.el).html(this.template(storyData));

        // Add class to first paragraph
        $(this.el).find('.bodytext p:first').addClass('ingress');

        // Insert any rich media
        var storyMedia = this.model.get('media');

        // Set font size
        var storyTextSize = ws.prefs('textSize') || 1;
        $(this.el).find('.ws-story-content').attr('class', 'ws-story-content text-size-'+storyTextSize);

        // Force update of connection status - this updates ws.env.network
        ws.connection.check();

        _.each(storyMedia, function(val, i) {
            var html = '';
            switch(val.type) {
                case "image":
                    // Image media
                    var size = val.size.split('x'),
                        src  = val.base64 || val.src;

                    html = '<div class="module image img-w'+size[0]+'">'
                             + '<img src="'+src+'" width="'+size[0]+'" height="'+size[1]+'"/>';
                    if (val.caption) { html += '<p class="caption">'+val.caption+'</p>' }
                    html += '</div>';
                    break;

                case "video":
                    // Video media
                    var playerAttributes = 'controls webkit-playsinline-DISABLED ';
                    var mp4Sources = _.select(val.sources, function(src){ return src.type == "video/mp4"; });
                    mp4Sources = _.sortBy(mp4Sources, function(src){ return parseInt(src.size); });

                    // Switch video quality depending on what type of connection we're on
                    // 3G: Low quality | Wifi: Medium quality
                    var videoSrc = ws.env.network.speed == "fast" ? mp4Sources[1] : mp4Sources[0];

                    // If user tapped on the teaser image, and it's a Media Asset Page, start autoplay
                    if (storyData.type == 'av_instance' && self.clickedOnImage) {
                        playerAttributes += 'autoplay ';
                        if (ws.nativeVideoPlayer == true) {
                            cordova.plugins.LandscapePlayer.playVideo(videoSrc.src);
                        }
                    }


                    if (typeof(videoSrc) != 'undefined') {
                        //html = '<video src="'+videoSrc.src+'" poster="'+val.poster+'" width="304" height="171"></video>'
                        var duration = val.duration,
                            minutes = parseInt(duration/60) % 60,
                            seconds = parseInt(duration % 60, 10),
                            niceDuration = (minutes<10 ? "0" + minutes : minutes) + ":" + (seconds<10 ? "0" + seconds : seconds);


                        html = '<div class="video-container" data-source="'+videoSrc.src+'">'
                             + '<img src="'+val.poster+'" width="304" height="171" />';

                        if (!ws.nativeVideoPlayer) {
                            html += '<video src="'+videoSrc.src+'" poster="'+val.poster+'" width="304" height="171" '+playerAttributes+'></video>';
                        }

                        html += '<span class="cta">'+niceDuration+'</span><span class="warning">'+ws.config.vocab.offlineVideoCta+'</span></div>';

                    }
                    break;


                case "picturegallery":
                    //ws.log('picturegallery');
                    //$(self.el).find('#'+val.id).remove();

                    html = '<div class="picture-gallery-container"><img src="http://ichef.bbci.co.uk/wsimagechef/ic/304x171/'+ val.images[0].src.replace('http://wscdn.bbc.co.uk/','') + '" />';

                    html += '<span class="cta"><span class="icon"></span><span>'+ws.config.vocab.launchGallery+'</span></span><span class="warning">'+ws.config.vocab.galleryOfflineMessage+'</span></div>';

                    self.gallery = val.images;

					break;

				case "audio":
					// Audio media
					var playerAttributes = 'controls webkit-playsinline-DISABLED ';
					var mp3Sources = _.select(val.sources, function(src){ return src.type == "audio/mpeg";});
					mp3Sources = _.sortBy(mp3Sources, function(src){ return parseInt(src.size) });

					if (val.poster == '') {val.poster = 'images/608x342_blank.png';}

					var audioSrc = mp3Sources[0];

					if (typeof(audioSrc) != 'undefined') {
                        var duration = val.duration,
                            minutes = parseInt(duration/60) % 60,
                            seconds = parseInt(duration % 60, 10),
                            niceDuration = (minutes<10 ? "0" + minutes : minutes) + ":" + (seconds<10 ? "0" + seconds : seconds);

                        html = '<div class="audio-container" data-source="'+audioSrc.src+'">'
							 + '<img src="'+val.poster+'" width="144" height="81" />';

						html += '<span class="cta">'+niceDuration+'</span><span class="warning">'+ws.config.vocab.offlineVideoCta+'</span></div>';
					}
					break;

            }

            // Replace media node in body area
            $(self.el).find('#'+val.id).html(html);
        });

        // handle special stories
        if (ws.config.specials) {
            var thisurl = storyData.href;
            _.each(ws.config.specials, function(special,i) {
                if (special.url == thisurl) {
                    $(self.el).find(special.targetEl).html(special.html);
                }
            });
        }

        // check after json loaded if story body has been populated, if not show message
        _.delay(function() {
            // handle empty pages
            if (ws.env.downloading == false && $(self.el).find('.bodytext').children().length < 1) {
                $(self.el).find('.bodytext').append('<p class="ingress">' + ws.config.vocab.storyError + '</p></div>');
                //ws.log('this story is empty');
            }
        },1000);

        // If there's related stories, insert them in
        if (_.size(storyData.related)) {
            var output = '';
            _.forEach(storyData.related, function(val, key) {
                if (typeof(val) == 'object') {
                    // New related items format (post Nov 3 2011)
                    var date = new Date(Date.parse(val.updated)).format(ws.config.dateFormat);

                    output += '<li><a href="#story/url/'+val.href+'">'+val.title+'</a><div class="date">'+date+'</div></li>';
                } else {
                    // Original JSON format
                    output += '<li><a href="#story/url/'+key+'">'+val+'</a><div class="date"></div></li>';
                }
            });
            node.find('.related ul').html(output);
			//catch related link tap event to prevent when app is online
			node.find('.related ul li').bind('tap', function(e) {
				if (ws.env.status == 'offline') { e.preventDefault(); }
			});
        } else {
            node.find('.related').remove();
        }


        // Disable prev/next buttons if at the edge of a collection needs to be reworked
        if (this.collection) {

			//ws.log('ws.currentGenre', ws.currentGenre, ws.homepageList, storyData);

			if (ws.currentGenre == 'homepage' || ws.currentGenre == 'latestnews') {

                var currentList = ws.currentGenre == 'homepage' ? ws.homepageList : ws.latestList;

                //ws.log(currentList);

				this.storyPosition = _.indexOf(currentList, storyData.bbcws_id);

				if (this.storyPosition <= 0) { node.find('.prev').addClass('disabled'); }
				if (this.storyPosition >= currentList.length-1) { node.find('.next').addClass('disabled'); }

				//ws.log("this.storyPosition " + this.storyPosition);

				// this if checks homepage array for stories stored offline and enables/disables next/previous buttons when appropriate
				if (ws.env.status == 'offline' && this.storyPosition > 0 && this.storyPosition < currentList.length) {

					node.find('.next').addClass('disabled');
					node.find('.prev').addClass('disabled');

					// loop through array, if stories from list are stored in array enable next/prev
					$.each(currentList, function(i) {
						// if index > than current story position and story is saved then we can enable next
						if (i > self.storyPosition && ws.store('s'+currentList[i]) && node.find('.next').hasClass('disabled')) {
							node.find('.next').removeClass('disabled');
						}
						// if index < than current story position and story is saved then we can enable prev
						if (i < self.storyPosition && ws.store('s'+currentList[i]) && node.find('.prev').hasClass('disabled')) {
							node.find('.prev').removeClass('disabled');
						}
					});

				}

			} else {

				if (this.storyPosition <= 0) { node.find('.prev').addClass('disabled'); }
				if (this.storyPosition >= this.collection.length-1) { node.find('.next').addClass('disabled'); }

				if (ws.env.status == 'offline' && this.storyPosition > 0 && this.storyPosition < this.collection.length) {
					//ws.log('disable next previous if offline');

					if (!ws.store('s'+this.collection.at(this.storyPosition + 1).id) || this.storyPosition >= ws.config.storiesToOffline-1) {
						//ws.log('the next one doesnt exist!');
						node.find('.next').addClass('disabled');
					}

					if (!ws.store('s'+this.collection.at(this.storyPosition - 1).id)) {
						//ws.log('the previous one doesnt exist!');
						node.find('.prev').addClass('disabled');
					}
				}

			}

        }


		if (this.options.referrer == 'byUrl') { this.options.disableNavigation = true; }

        // Check if any controls need to be disabled
        if (this.options.disableNavigation) { node.find('#story-navigation').remove(); }
        if (this.options.disableSharing)    { node.find('.share').remove(); }

        this.handleTextSize();

        ws.views.scrollers.drawScrollablePage(this.el);
        //ws.views.scrollers.refresh();

        return this;
    }
});

ws.views.PictureGallery = Backbone.View.extend({
    el: "#picture-gallery",
    captionScrolls: [],
    events: {
        "tap .done": "returnToStory",
        "tap .next": "nextImage",
        "tap .previous": "previousImage",
        "tap .captions":  "toggle",
        "swiperight li .captionHandle":  "scrollCaption",
        "swipeleft li .captionHandle":  "scrollCaption"
    },

    initialize: function() {
        //ws.log('[INIT PICTURE GALLERY]');
        _.bindAll(this, "render");
        //this.template = _.template($(this.el).html());
        this.template  = _.template($.ajax({ type:'GET', url:'templates/picture-gallery.html', async:false }).responseText);
    },
    calculateWindowSize: function() {
        this.windowWidth = $(window).width(); //ws.env.width;
        this.windowHeight = $(window).height(); //ws.env.height;

        if (ws.debug == true) {
            this.windowWidth = parseInt($("body").find('.page').css('width').replace('px',''));
        }
        //console.log('width ' + this.windowWidth);
        //console.log('height ' + this.windowHeight);
    },
    setElDimensions: function() {
        // dynamically set the height and width of scrolling elements
        var self = this;

        if (self.orientation == 'responsive') {
            //ws.log('reponsive');
            $(self.el).find('#pg-wrapper').css('width', self.windowWidth);
            $(self.el).find('#pg-scroller').css('width', self.windowWidth*self.galleryLength);
            $(self.el).find('li').css('width', self.windowWidth);
            $(self.el).find('li').css('height', self.windowHeight);
            $(self.el).find('li > .rotateThis').css('width', self.windowWidth).css('margin-left', 0 - self.windowWidth/2);
            $(self.el).find('li > .rotateThis').css('height', self.windowHeight).css('margin-top', 0 - self.windowHeight/2);
            $(self.el).find('li > .rotateThis p').css('width', self.windowWidth - 48);
            //$(self.el).find('.captionHandle').css('width', self.windowWidth);
            $(self.el).find('.captionContainer').css('height', self.windowHeight);
        } else {
            //ws.log('forcedVertical');
            $(self.el).find('#pg-wrapper').css('height', self.windowHeight);
            $(self.el).find('#pg-scroller').css('height', self.windowHeight*self.galleryLength);
            $(self.el).find('li').css('width', self.windowWidth);
            $(self.el).find('li').css('height', self.windowHeight);
            $(self.el).find('li .rotateThis').css('width', self.windowHeight).css('margin-left', 0 - self.windowHeight/2);
            $(self.el).find('li .rotateThis').css('height', self.windowWidth).css('margin-top', 0 - self.windowWidth/2);
            $(self.el).find('li .rotateThis p').css('width', self.windowHeight - 48);
            $(self.el).find('.captionHandle').css('height', self.windowHeight);
            $(self.el).find('.captionContainer').css('width', self.windowHeight);
        }
    },
    setIChefSize: function() {
        var self = this,
            imageSize = '448x252';

        // set which imageChef recipe to use
        if (self.orientation == 'responsive') {
            if (self.windowWidth >= 304 && self.windowWidth < 336) {
                imageSize = '320x180';
            } else if (self.windowWidth >= 336 && self.windowWidth < 384) {
                imageSize = '336x189';
            } else if (self.windowWidth >= 384 && self.windowWidth < 448) {
                imageSize = '384x216';
            } else if (self.windowWidth >= 448 && self.windowWidth < 480) {
                imageSize = '384x216';
            } else if (self.windowWidth >= 480 && self.windowWidth < 512) {
                imageSize = '448x252';
            } else if (self.windowWidth >= 512 && self.windowWidth < 640) {
                imageSize = '512x288';
            } else if (self.windowWidth >= 640) {
                imageSize = '640x360';
            }
        }

        //ws.log(imageSize);

        return imageSize;
    },
    createScroller: function() {
        var self = this;
        self.scroller = new iScroll('pg-wrapper', {
            snap: 'li',
            vScroll: self.vScroll,
            hScroll: self.hScroll,
            momentum: false,
            hScrollbar: false,
            vScrollbar: false,
            desktopCompatibility: true,
            onScrollEnd: function () {
                //ws.log(this.currPageY);
                self.checkNavPosition();
                self.loadImage(self.currImageIndex); //incase you've swipped too fast
            }
        });
        _.delay(function() {
            self.scroller.refresh();
            if (ws.config.dir == 'rtl') {
                if (self.orientation == 'responsive') {
                    self.scroller.scrollToPage(self.currImageIndex, 0, 0);
                } else {
                    self.scroller.scrollToPage(0, self.currImageIndex, 0);
                }
            }
            $(self.el).find('.pg-overlay').addClass('hidden');
            //self.checkNavPosition();
            self.prepareCaptions();
        },1000);
    },
    createCaptionScroller: function(i) {
    	//ws.log('createCaptionScroller');

    	var self = this;

		self.captionScrolls[i] = new iScroll('captionScroll' + i, {
            vScroll: true,
            hScroll: false,
            zoom: false,
            scrollbarClass: 'scrollbar-',
            desktopCompatibility: true
        });

    },
    prepareCaptions: function() {
        //ws.log('prepareCaptions');

        var self = this,
            totalParagraphHeight,
            totalAvailableSpace,
            responsiveHandlePosClosed,
            responsiveHandlePosOpen,
            targetParagraphHeight = parseInt($(self.el).find('p').css('line-height')) * 4 + parseInt($(self.el).find('p').css('padding-top')); // four lines of text

        if (self.orientation == 'responsive') {
            totalAvailableSpace = self.windowHeight - $(self.el).find('.pg-controls').height() - $(self.el).find('.pg-nav').height() - parseInt($(self.el).find('.captionContainer').css('border-bottom-width'));
        } else {
            totalAvailableSpace = self.windowWidth - $(self.el).find('.pg-controls').height() - $(self.el).find('.pg-nav').height() - parseInt($(self.el).find('.captionContainer').css('border-bottom-width'));
        }

        //ws.log($(self.el).find('.pg-controls').height());
        //ws.log($(self.el).find('.pg-nav').height());

        // in each list item, remove classes if they exist and calculate the total height of paragraph element
        _.each($(self.el).find('ul li'), function(item, i) {

            totalParagraphHeight = $(item).find('p').height() + parseInt($(item).find('p').css('padding-top'));
            $(item).find('.captionHandle').removeClass('truncated');
            $(item).find('.captionHandle').removeClass('expanded');

            //ws.log('totalParagraphHeight ' + totalParagraphHeight);
            //ws.log('targetParagraphHeight ' + targetParagraphHeight);

            // if paragraph is greater than 4 lines of text, truncate the parapgrah and display handle to slide show all text
            if (totalParagraphHeight > targetParagraphHeight) {

                $(item).find('.captionHandle').addClass('truncated');

                // when the app is responsive, the handle is built differently
                responsiveHandlePosClosed = targetParagraphHeight + $(self.el).find('.pg-nav').height() + parseInt($(self.el).find('.captionHandle span').css('border-bottom-width')) + parseInt($(self.el).find('.captionContainer').css('border-bottom-width')) - $(self.el).find('.captionHandle').height();

                responsiveHandlePosOpen = totalAvailableSpace + $(self.el).find('.pg-nav').height() + parseInt($(self.el).find('.captionHandle span').css('border-bottom-width')) + parseInt($(self.el).find('.captionContainer').css('border-bottom-width')) - $(self.el).find('.captionHandle').height()

                if (self.orientation == 'responsive') {
                    $(item).find('.captionHandle').css('bottom', responsiveHandlePosClosed);
                } else {
                    $(item).find('.captionHandle').css('width', targetParagraphHeight);
                }

                $(item).find('.captionContainer').css('height', targetParagraphHeight).attr('id', 'captionScroll'+ i);
                $(item).find('.captionWrapper').css('height', totalParagraphHeight);

                // event handler for tap on handle
                $(item).find('.captionHandle span').bind('tap', function(e) {

                    if ($(item).find('.captionContainer').height() == totalAvailableSpace) {
                    	//close caption
                        if (self.orientation == 'responsive') {
                            $(item).find('.captionHandle').css('bottom', responsiveHandlePosClosed);
                        } else {
                            $(item).find('.captionHandle').width(targetParagraphHeight);
                        }
                        $(item).find('.captionHandle').removeClass('expanded');
                        $(item).find('.captionContainer').height(targetParagraphHeight);

                        // destroy any scrollers if it had one
                        if (self.captionScrolls[i]) {
                        	//ws.log('destroy scroll');
                            self.captionScrolls[i].destroy();
                            self.captionScrolls[i] = null;
                        }

                    } else {
                    	//open caption
                        if (self.orientation == 'responsive') {
                            $(item).find('.captionHandle').css('bottom', responsiveHandlePosOpen);
                        } else {
                            $(item).find('.captionHandle').width(totalAvailableSpace);
                        }
                        $(item).find('.captionHandle').addClass('expanded');
                        $(item).find('.captionContainer').height(totalAvailableSpace);

                        _.delay( function() {
                            // if after expanding the caption the paragraph is still not big enough to show all text, create a scroller
                            if ($(item).find('.captionWrapper').height() > $(item).find('.captionContainer').height()) {
                                //ws.log('create scroller');
                                self.createCaptionScroller(i);
                            }
                        }, 750);

                    }
                    return false;
                });

            } else {
            	 $(item).find('.captionContainer').css('height', totalParagraphHeight);
            }
        });

    },
    scrollCaption: function(e) {
        //ws.log('scroll caption');
        var self = this;

        if (e.type == 'swiperight' && self.captionScrolls[self.currImageIndex]) {
            self.captionScrolls[self.currImageIndex].scrollTo(0,self.captionScrolls[self.currImageIndex].y - 100,200);
        }

        if (e.type == 'swipeleft' && self.captionScrolls[self.currImageIndex]) {
            self.captionScrolls[self.currImageIndex].scrollTo(0,self.captionScrolls[self.currImageIndex].y + 100,200);
        }
    },
    returnToStory: function(e) {
        //ws.log('back');
        var self = this;
        $(window).unbind('orientationchange');
        $(window).unbind('offline');
        $(window).unbind('online');
        self.unbind();

        ws.views.switchTo('#story', true);
        _.delay(function() {
            self.scroller.destroy(); // improves performance :)
            self.scroller = null;
            $(self.el).find('.captionHandle span').unbind('tap');
            _.each(self.captionScrolls, function(scroll, i) {
                //ws.log(scroll, i);
                if (scroll != null) {
                    scroll.destroy();
                    scroll = null;
                }
            });
            $(this.el).empty();
            $(this.el).removeClass('showDetails');
        },750);
        e.stopImmediatePropagation();
        return false;
    },
    loadImage: function(i) {
        var self = this,
            thisNode = $(self.el).find('ul li .rotateThis').get(i),
            imgSrc;

        // escape if at end of gallery
        if (i > this.galleryLength - 1 || i < 0) { return; }

        // escape if offline or app disabled
        if (ws.env.status == 'offline' || ws.env.remoteConfig.appEnabled == false) { return; }

        if ($(thisNode).find('img').length == 0) {
            //ws.log('loading new image');

            imgSrc = 'http://ichef.bbci.co.uk/wsimagechef/ic/'+self.icSize+'/'+ self.gallery[i].src.replace('http://wscdn.bbc.co.uk/','');
            $(thisNode).append('<img src="'+imgSrc+'" width="'+self.icSize.substring(0,3)+'" height="'+self.icSize.substring(4)+'" />');
            $(thisNode).find('img').css('margin-left', 0 - parseInt(self.icSize.substring(0,3))/2).css('margin-top', 0 - parseInt(self.icSize.substring(4))/2);

            $(thisNode).addClass('image-ready');
        }
    },
    nextImage: function(e) {
        //ws.log('nextImage');
        if (this.currImageIndex < this.galleryLength - 1) {
            //ws.log('try');
            if (this.orientation == 'responsive') {
                this.scroller.scrollToPage(this.currImageIndex + 1, 0, 750);
            } else {
                this.scroller.scrollToPage(0, this.currImageIndex + 1, 750);
            }
        }
        if (e) { e.stopImmediatePropagation(); }
        return false;
    },
    previousImage: function(e) {
        //ws.log('prevImage');
        if (this.currImageIndex > 0) {
            //ws.log('try');
            if (this.orientation == 'responsive') {
                this.scroller.scrollToPage(this.currImageIndex - 1, 0, 750);
            } else {
                this.scroller.scrollToPage(0, this.currImageIndex - 1, 750);
            }
        }
        if (e) { e.stopImmediatePropagation(); }
        return false;
    },
    checkNavPosition: function() {
        //ws.log('checkNavPosition');

        var self = this,
            newImageIndex = self.orientation == 'responsive' ? self.scroller.currPageX : self.scroller.currPageY;

        //ws.log(newImageIndex);

        // preload next image
        if (newImageIndex > self.currImageIndex) {
            self.loadImage(newImageIndex + 1);
        } else {
            self.loadImage(newImageIndex - 1);
        }

        self.currImageIndex = newImageIndex;

        self.updateNavPosition();

        // handle nav buttons
        if (self.currImageIndex == self.galleryLength - 1) {
            $(this.el).find('.next').addClass('disabled');
        } else if (self.currImageIndex == 0) {
            $(this.el).find('.previous').addClass('disabled');
        } else {
            $(this.el).find('.disabled').removeClass('disabled');
        }
        //ws.log('currImageIndex '+ self.currImageIndex);
    },
    updateNavPosition: function() {
        //ws.log('updateNavPosition');
        var currentPicture = this.currImageIndex + 1,
            totalPictures = this.galleryLength;

        if (ws.config.dir == 'rtl') {
            currentPicture = totalPictures - currentPicture + 1;
        }

        var navTemplate = _.template(ws.config.vocab.galleryNav, {currPic : currentPicture, totalPic: this.galleryLength});
        $(this.el).find('.pg-nav span').html(navTemplate);
    },
    toggle: function(e) {
        // hide controls and captions
        //ws.log('toggle');
        $(this.el).toggleClass('showDetails');
        if ($(this.el).hasClass('showDetails')) {
            $(this.el).find('.captions span').html(ws.config.vocab.galleryHideCaptions);
        } else {
            $(this.el).find('.captions span').html(ws.config.vocab.galleryShowCaptions);
        }
        e.stopImmediatePropagation();
        return false;
    },
    handleRotation: function() {
        ws.log('orientationchange in pic gallery');

        var self = this;

        $(self.el).find('.pg-overlay').removeClass('hidden');

        self.calculateWindowSize();
        self.setElDimensions();

        self.icSize = self.setIChefSize();
        $(self.el).find('ul li div').each(function() {
            $(this).find('img').remove();
        });
        self.loadImage(self.currImageIndex);
        self.loadImage(self.currImageIndex + 1);
        self.loadImage(self.currImageIndex - 1);

        $(self.el).find('.captionHandle span').unbind('tap');
        _.delay(function() {
	        _.each(self.captionScrolls, function(scroll, i) {
	            //ws.log(scroll, i);
	            if (scroll != null) {
	                scroll.destroy();
	                scroll = null;
	            }
	        });

        	self.prepareCaptions();
        }, 500);

        // refresh and rescroll the scroller
        self.scroller.refresh();
        self.scroller.scrollToPage(self.currImageIndex, 0, 0);

        _.delay(function() {
        	$(self.el).find('.pg-overlay').addClass('hidden');
		}, 500);
    },
    closeExpandedCaption: function(e) {
        ws.log('close this caption');

        var self = this;

        $(self.el).find('.pg-overlay').addClass('hidden');

        _.delay(function() {
            self.captionScroller.destroy(); // improves performance :)
            $(self.el).find('.pg-overlay').removeClass('captionOverlay').empty();
        },750);

        e.stopImmediatePropagation();
        return false;
    },
    render: function() {
        //ws.log('[RENDER PICTURE GALLERY]');

        var self = this,
            list = '';

        $(self.el).html(self.template(ws.config));

        // set some env variables
        if (ws.env.allowRotate == false) {
            $(this.el).addClass('rotated');
            self.orientation = 'forcedVertical';
            self.vScroll = true;
            self.hScroll = false;
        } else {
            self.orientation = 'responsive';
            self.vScroll = false;
            self.hScroll = true;
        }

        $(window).bind( 'orientationchange', function(e) {
            //ws.log(e.orientation);
            self.handleRotation();
            return false;
        });
        // couldn't get pubsub unsubscribe to work
        $(window).bind('offline', function(e) {
            //ws.log('checkOfflineStatus');
        });
        $(window).bind('online', function(e) {
            //ws.log('checkOnlineStatus');

            _.delay(function() {
                self.loadImage(self.currImageIndex);
                self.loadImage(self.currImageIndex + 1);
                self.loadImage(self.currImageIndex - 1);
            }, 1000);
        });

        // make copy of object array
        self.gallery = self.options.gallery.slice();

        self.galleryLength = self.gallery.length;
        self.currImageIndex = 0;

        // reverse the object if its right-to-left
        if (ws.config.dir == 'rtl') {
            self.gallery.reverse();
            self.currImageIndex = self.galleryLength - 1;
        }

        self.updateNavPosition();

        self.calculateWindowSize();

        self.icSize = self.setIChefSize();

        // prepare the html
        $(self.el).addClass('showDetails');

        // populate the list
        _.each(self.gallery, function(image,i) {
        	var caption = image.caption,
                captionClass = 'normal';
            list += '<li><div class="captionHandle"><span></span></div><div class="rotateThis"><div class="captionContainer"><div class="captionWrapper"><p>'+ caption +'</p></div></div><div class="offline-message"><span>'+ws.config.vocab.galleryOfflineMessage+'</span></div></div></li>';
        });

        //insert the images
        $(self.el).find('ul').html(list);

        // load the first image
        self.loadImage(self.currImageIndex);
        if (ws.config.dir == 'ltr') {
            // load the second image
            self.loadImage(self.currImageIndex + 1);
            $(this.el).find('.previous').addClass('disabled');
        } else {
            self.loadImage(self.currImageIndex - 1);
            $(this.el).find('.next').addClass('disabled');
        }

        self.setElDimensions();

        self.createScroller();

        return this;

    }
});

ws.views.ZoomerView = Backbone.View.extend({
    el: "#zoomer",
    doPageTransition: true,
    events: {
        "tap .back": "back"
    },
    initialize: function() {
         _.bindAll(this, "render");
        this.template = _.template($(this.el).html());
    },
    back: function() {
        ws.views.switchTo('#story', true);
    },
    render: function() {
        var viewData = {
            vocab:    ws.config.vocab,
            imageSrc: this.imageSrc
        };

        $(this.el).html(this.template(viewData));
        new iScroll('zoomer-wrapper', { zoom: true });
    }
});


/* Subviews of "Main" go here
 * (Home, Popular, Categories, Settings)
************************************** */
ws.views.homeView = Backbone.View.extend({
    id: 'ws-home',
    events: {
        "tap #ws-view-chooser li": "handleViewTab"
    },
    initialize: function() {
		//ws.log('[INIT HOMEVIEW]');
        _.bindAll(this, "handleViewTab", "render"); // to solve the 'this' issue
        this.node      = $(this.el);
        this.template  = _.template($.ajax({ type:'GET', url:'templates/home.html', async:false }).responseText);
        ws.currentGenre = null;
		ws.currentHomeTab = null;
		ws.currentPopularTab = null;

        this.topStoriesView = new ws.views.StoryList({
            id: "ws-top-news-list",
            groupgenre: "top_stories",
            collection: this.collection,
            showFullStoryList: false,
            showTopStories:    true
        });

        this.recentStoriesView = new ws.views.mostRecentList({
            id: "ws-latest-list",
            collection: this.collection
        });
    },
    handleViewTab: function(e) {
        var node   = $(e.currentTarget),
            target = node.find('a').attr('data-tabtarget');

        node.parent().find('li').removeClass('active');
        $(target, this.node).parent('.tab-group').children().hide();
        $(target, this.node).show();

		ws.currentHomeTab=null;
		if (target=='#ws-latest-list') {
			if (ws.config.rollingNews.enabled) {
				ws.currentHomeTab = 'rolling_news';
			} else {
				ws.currentHomeTab = 'latest_news';
			}
		}

        node.addClass('active');
		ws.views.renderLastUpdated();
        //ws.views.scrollers.refresh(this.el);
    },
    render: function() {
		//ws.log('[HOME] rendering view');

        ws.trackView('INDEX', ws.config.serviceRoot);

        this.node.html(this.template(ws.config)); // Render template and mixin vocab

		ws.activeStoryListViewId = this.topStoriesView.cid;

        this.node.find('#ws-top-news-list').replaceWith(this.topStoriesView.render().el);
        this.node.find('#ws-latest-list').replaceWith(this.recentStoriesView.render().el);

        // Set default 'Top Stories/Recent Stories' tab
        if (ws.prefs('defaultHomeTab') == "latestnews") {
            this.node.find('a[data-tabtarget="#ws-latest-list"]').trigger('tap');
        } else {
            this.node.find('a[data-tabtarget="#ws-top-news-list"]').trigger('tap');
        }

        return this;
    }
});


ws.views.popularView = Backbone.View.extend({
    id: 'ws-popular',
    events: {
        "tap #ws-view-chooser li": "handleViewTab"
    },
    initialize: function() {
		//ws.log('[INIT POPULAR VIEW]');
        _.bindAll(this, "handleViewTab", "render"); // to solve the 'this' issue
        this.node      = $(this.el);
        this.template  = _.template($.ajax({ type:'GET', url:'templates/popular.html', async:false }).responseText);

        this.mostReadView    = new ws.views.mostPopularList({ collection: ws.popularStories.read, id: 'ws-most-read', attributes: {'data-groupgenre': 'most_read'} });
        this.mostWatchedView = new ws.views.mostPopularList({ collection: ws.popularStories.watched, id: 'ws-most-watched', attributes: {'data-groupgenre': 'most_watched'} });
    },
    handleViewTab: function(e) {

		if (ws.popularStories.read.length < 1 && ws.env.status == 'offline') { return false; }

        var node   = $(e.currentTarget),
            target = node.find('a').attr('data-tabtarget');

        node.parent().find('li').removeClass('active');
        $(target).parent('.tab-group').children().hide();
        $(target).show();
		if (target == '#ws-most-read') {
			ws.currentPopularTab = 'read';
		} else {
			ws.currentPopularTab = 'watched';
		}

        node.addClass('active');
        ws.views.scrollers.refresh(this.el);
    },
    switchOff: function(feature) {
        if (feature === 'mostRead') { this.$('.most-read-tab').hide(); }
        if (feature === 'mostWatched') { this.$('.most-watched-tab').hide(); }
    },
    render: function() {
		//ws.log('[POPULAR] rendering view');

        this.node.html(this.template(ws.config)); // Render template and mixin vocab

		this.node.find('.ws-status-message').hide();

        // Hide tabs if any of the feeds are switched off
        if (ws.config.livestats.mostRead === false || ws.config.livestats.mostWatched === false) {
            this.switchOff('mostRead');
            this.switchOff('mostWatched');
        }

		// if offline the livestats won't show if it hasn't been looked at already online, so this displays message in that case
        if (ws.popularStories.read.length < 1 && ws.env.status == 'offline') {

			this.node.find('.ws-status-message').show();
			this.node.find('#ws-view-chooser li:first').addClass('active');

		} else if (ws.env.remoteConfig.appEnabled == false) {

			this.node.find('#ws-view-chooser li:first').addClass('active');

		} else {

			if (ws.popularStories.read.length == 0 && ws.config.livestats.mostRead === true) {
				ws.popularStories.read.fetch({
					success: function(data) {
						$.publish('/feeds/downloadcomplete');
						var genreLUKey = 'lastUpdatedread';
						ws.store(genreLUKey, new Date());
					}
				});
			}
			if (ws.popularStories.watched.length == 0 && ws.config.livestats.mostWatched === true) {
				ws.popularStories.watched.fetch({
					success: function(data) {
						$.publish('/feeds/downloadcomplete');
						var genreLUKey = 'lastUpdatedwatched';
						ws.store(genreLUKey, new Date());
					}
				});
			}

			this.node.find('#ws-most-read').replaceWith(this.mostReadView.render().el);
			this.node.find('#ws-most-watched').replaceWith(this.mostWatchedView.render().el);

            this.node.find('li:visible:first a').trigger('tap');

		}

        return this;
    }
});



ws.views.categoryListView = Backbone.View.extend({
    id: 'ws-categories',
    initialize: function() {
		//ws.log('[INIT CATEGORY LIST VIEW]');
        _.bindAll(this, "goToCategory", "render"); // to solve the 'this' issue
    },
    goToCategory: function(genreId) {

		// check if app enabled via config.json
		if (ws.env.remoteConfig.appEnabled == false) { return false; }

		//ws.log('from categorised list');
        var node = $(this.el);
        var renderedList = new ws.views.generalStoryList({
            collection: ws.stories.fetchGenre(genreId),
            id: 'ws-categorised-list'
        }).render().el;

        $('#ws-categorised-list').remove();
        node.after(renderedList);

        _.delay(function() { ws.views._activeMainView.goToCategoryView(); }, 50);
    },
	render: function() {
		//ws.log('[CATEGORY LIST] rendering view');
        var self       = this,
            node       = $(this.el),
            listNode   = $('<ul id="ws-categorylist" class="active"></ul>'),
            output     = "",
            genreOrder = ws.store('genreOrder');

        _.each(genreOrder, function(selected, genreId) {
            var classname = selected ? 'is-offline' : '';

            var genre        = ws.config.availableGenres[genreId],
                tempNode     = $('<li class="'+classname+'">'+genre.title+'</li>');

            tempNode.bind('tap', function() {
				if (!$(this).hasClass('is-offline') && ws.env.status == 'offline') { return false; }
                self.goToCategory(genreId);
				ws.currentGenre = genreId;
				ws.currentHomeTab = null;

            });

            listNode.append(tempNode);
        });

        node.html(listNode);
		//node.html(listNode).prepend(ws.views.templates.pulldown);

        ws.views.scrollers.refresh(node);

        return this;
    }
});


ws.views.settingsView = Backbone.View.extend({
    id: 'ws-settings',
    events: {
        'tap .genre-list li': 'toggleGenre',
        'tap .share-stats'  : 'toggleStats',
		'tap .set-default-tab'  : 'setDefaultTab',
        'tap .set-dialect'  : 'setDialect'
    },
    initialize: function() {
        this.node      = $(this.el);
        this.template  = _.template($.ajax({ type:'GET', url:'templates/settings.html', async:false }).responseText);
    },
    render: function() {
        var node = this.node,
            self = this;

        var viewVars = ws.config;
        viewVars.version = ws.version;

        node.html(this.template(viewVars));

        // Create sortable list
        var genreListHtml = '<li class="topnews selected locked">'+ws.config.vocab.topNews+'<span class="handle" /></li>',
            numListHtml   = '<li>1</li>',
            genreOrder    = ws.store('genreOrder'),
            count         = 2;


        _.each(genreOrder, function(selected, genreId) {
            var genre     = ws.config.availableGenres[genreId],
                className = selected === true ? 'selected' : '';

            numListHtml   += '<li>'+count+'</li>';
            genreListHtml += '<li id="'+genreId+'" class="'+className+' sortable">'+genre.title+'<span class="handle" /></li>';
            count++;
        });


        node.find('.genre-num').html(numListHtml);
        node.find('.genre-list').html(genreListHtml);
        node.find('.genre-list').sortable({
            opacity: 1,
            //delay:  100,
            items:  '.sortable',
            handle: '.handle',
            containment: 'parent',
            axis:   'y',
            create: function(event, ui) {

            },
            start: function(event, ui) {
                this.viewContainerId = $(this).parents('.ws-scrollable').attr('id');
                ws.views.scrollers._list[this.viewContainerId].disable();
            },
            stop: function(event, ui) {
                ws.views.scrollers._list[this.viewContainerId].enable();
            },
            update: function(event, ui) {
                self.saveSortOrder(this);
                self.render();
            }
        });

        if (ws.prefs('shareStats')) { node.find('.share-stats').addClass('active'); }

		if (ws.prefs('defaultHomeTab') == 'topnews') {
			node.find('.set-default-tab#top-news').addClass('active');
		} else {
			node.find('.set-default-tab#latest-news').addClass('active');
		}

        // set the dialect
		if (ws.config.dialects) {
	        _.each(ws.config.dialects, function(i, k) {
	            //ws.log(i, k);
				node.find('#select-default-dialect ul').append('<li id="' + k + '" class="set-dialect">' + i + '</li>')
	        });
			$('#' + ws.prefs('defaultDialect')).addClass('active');
		}


		//catch legal list link tap event to prevent when app is online
		node.find('.legal-list li').bind('tap', function(e) {
			if (ws.env.status == 'offline') { e.preventDefault(); }
		});

        return this;
    },
    toggleStats: function(e) {
        var node = $(e.target);
        node.toggleClass('active');

        node.hasClass('active') ? ws.prefs('shareStats', 1) : ws.prefs('shareStats', 0)
    },
    toggleGenre: function(e) {

		if (ws.env.remoteConfig.appEnabled == false) { return false; }

		var node = $(e.target);
		if (node.hasClass('locked')) return;
		node.toggleClass('selected');

		if (node.hasClass('selected')) {
			// If the user selects a genre which is not in the local story index,
			// go off and fetch the index from the server
			var genreId = node.attr('id');
			if (_.isUndefined(ws.storyIndex[genreId])) { ws.stories.fetchGenre(genreId); }
		}

		this.saveSortOrder();

    },
	setDefaultTab: function(e) {
		// store 'topnews' for top news, 'latestnews' for latest news
		// ws.config.defaultHomeTab set in service.js needs to be overriden if this exists
		var node = $(e.target);
		if (!node.hasClass('active')) {
			$('.set-default-tab').removeClass('active');
			node.addClass('active');
			if ($('.set-default-tab.active').attr('id') == 'top-news') {
				ws.prefs('defaultHomeTab', 'topnews');
			} else {
				ws.prefs('defaultHomeTab', 'latestnews');
			}
		}
	},
    saveSortOrder: function(el) {
        var node = $(this.el);
        var sortedItems = node.find('.genre-list').sortable('toArray'),
            finalList = {};

        _.forEach(sortedItems, function(genreId) {
            finalList[genreId] = node.find('#'+genreId).hasClass('selected')
        });

        ws.genreOrder = finalList;
        ws.store('genreOrder', finalList);
    },
    setDialect: function(e) {

        if (ws.env.remoteConfig.appEnabled == false) { return false; }

		//change the dialect
        var node = $(e.target);

        if (!node.hasClass('active') && ws.env.status == "online") {
            var overlayNode = $('#overlay');
            overlayNode.empty().append('<div></div>').addClass('loading').addClass('visible');
            ws.loadNewService(ws.service.replace(ws.prefs('defaultDialect'), node.attr('id')), 'settings');
        }
    }
});




/* Mini views go here
************************************** */
ws.views.StoryListRow = Backbone.View.extend({
    tagName:  "li",
    events:  {
        "tap":         "open",
        "touchstart":  "makeActive",
        "touchmove":   "makeInactive",
        "tap .genre":  "goToCategory"
    },
    initialize: function() {
        _.bindAll(this, "render", "open");
        this.template = _.template(ws.views.templates.storyRow);
        this.node = $(this.el);
    },
    render: function() {
        //ws.log('[STORY LIST ROW RENDER]');

        // If there's no model associated with this view, abort render
        if (_.isUndefined(this.model)) return;

        var node         = this.node,
            storyData    = this.model.toJSON(),
            ctaIcon      = '';

        //ws.log('coll ', this.model.collection);

        // Filter out any stories not in the defined genre list
        if (!_.isUndefined(storyData.genre) && _.isUndefined(ws.config.availableGenres[storyData.genre])) {
            //ws.log('[WARNING] Vernacular for title "'+storyData.genre+'" not found. Skipping story');
            return this;
        }

        // Prettify the output
        var storyDate       = new Date(Date.parse(storyData.updated)),
            updatedDistance = new Date().getTime() - storyDate.getTime(),
            threshold       = ws.config.timeAgoThreshold * 60 * 60 * 1000; // in hours

        if (updatedDistance < threshold) {
            // Display relative time
            storyData.prettyDate = jQuery.timeago(storyData.updated);
            node.addClass('relative-time');
        } else {
            // Display datetime stamp
            storyData.prettyDate = new Date(Date.parse(storyData.updated)).format(ws.config.dateFormat);
        }

        storyData.vGenre  = _.isUndefined(storyData.genre) ? '' : ws.config.availableGenres[storyData.genre].title;
        storyData.summary = ws.utils.truncate(storyData.summary, ws.config.summarySize);


        // If this story comes from a genre which is not selected in Preferences,
        // hide the genre link as the feeds wouldn't have downloaded
        if (storyData.genre && (ws.genreOrder[storyData.genre] === false || ws.config.availableGenres[storyData.genre].enabled == false)) { storyData.vGenre = "";
        }

        // Check if story has been offlined, and add class if necessary
        if (this.model.isOffline()) {
			//ws.log(this.model.collection); //trying to only show the first 3 even if the rest are stored offline
			var thisIndexOf = this.model.collection.indexOf(this.model);

			if (thisIndexOf < ws.config.storiesToOffline) {
				node.addClass('is-offline');
			}
		}

        html = this.template(storyData);
        node.addClass('ws-entry clearfix ' + storyData.bbcws_id).html(html);

        // Add CTA icons if needed - this really should be in CSS, but this way no <img>
        // tags are used, only CSS background, which has improved performance
        // http://code.flickr.com/blog/2011/07/20/lessons-learned-from-the-flickr-touch-lightbox/
        if (storyData.type == 'av_instance') { ctaIcon = 'images/play_48x48.png'; }

        if (ctaIcon != '') {
            node.find('.image').css('background-image', 'url('+ctaIcon+'),url('+storyData.teaser+')').css('-webkit-background-size', '24px 24px, 104px 59px');
        }

        // Add gallery cta
        if (storyData.contains == 'gallery') {
            node.find('.image').css('background-image', 'url(images/camera_icon_small.png),url('+storyData.teaser+')').css('-webkit-background-size', '24px 20px, 104px 59px');
        }

        // If there is no image, remove the placeholder
        if (storyData.teaser.match('images/placeholder')) {
            node.find('.image, .genre').remove();
            node.addClass('disable-image');
        }


        return this;
    },
    makeActive: function() {
        this.node.addClass('active');
    },
    makeInactive: function() {
        this.node.removeClass('active');
    },
    open: function(e) {
        //ws.log('[OPEN STORY]');

        var target = $(e.target),
            node   = this.node;

		ws.views._activeStoryView.clickedOnImage = false;
		//if app is offline, the model has not been saved and is not in the local storage
        if (ws.env.status == 'offline' && !this.model.has('offline') && !ws.store('s'+this.model.id) || !node.hasClass('is-offline') && ws.env.status == 'offline') { return false; }
        if (target.hasClass('genre')) { return false; }

        node.addClass('active');

        //record what context the user accessed the story from
        if (target.closest('[data-groupgenre]').length > 0) {
            //ws.log(target.closest('[data-groupgenre]').attr('data-groupgenre'));
            ws.genreContext = target.closest('[data-groupgenre]').attr('data-groupgenre');
        }

        if (e.target.className.toLowerCase() == 'image' && ws.env.status == 'online') { ws.views._activeStoryView.clickedOnImage = true; }

		if (ws.currentGenre == null) { //set genre 'homepage' for stories clicked from this list
			if ($('#ws-home').length > 0 && $('#ws-top-news-list').css('display') == 'block') {
				//ws.log('clicked from the homepage');
				ws.currentGenre = 'homepage';
			}
			if ($('#ws-home').length > 0 && $('#ws-latest-list').css('display') == 'block' && ws.config.rollingNews.enabled == false) {
				//ws.log('clicked from the homepage');
				ws.currentGenre = 'latestnews';
			}
			if ($('#ws-home').length > 0 && $('#ws-latest-list').css('display') == 'block' && ws.config.rollingNews.enabled == true) {
				//ws.log('clicked from the homepage');
				ws.currentGenre = 'rollingnews';
			}
		}

        // Pass the collection to the story view so it can be used for Prev/Next
        ws.views._activeStoryView.storyPosition = this.model.collection.indexOf(this.model);
        ws.views._activeStoryView.collection = this.model.collection;

        var storyId = this.model.id || this.model.cid;
        //ws.log('opening story ', this.model.get('title'), this.model.collection);

        // Route to story view
        ws.thisRouter.navigate('story/' + storyId, true);

        setTimeout(function() {
            node.removeClass('active');
        }, 350);
    },
    goToCategory: function(e) {
		//ws.log('from story list row');
        var selectedGenre = this.model.get('genre');//$(e.currentTarget).attr('data-genre');
		ws.currentGenre = selectedGenre;
		ws.currentHomeTab = null;
        new ws.views.generalStoryList({
            collection:  ws.stories.getByGenre(selectedGenre),
            el: '#ws-categorised-list'
        });

        ws.views._activeMainView.goToCategoryView();
    }
});


ws.views.StoryList = Backbone.View.extend({
    tagName:   "ul",
    className: "ws-entries",
    attributes: {
        "data-groupgenre": "top_stories"
    },
    events:   {
        "tap .content-group > h2" : "toggleContentAccordion",
        "tap .more-stories"     : "goToCategory"
    },
    initialize: function() {
		//ws.log('[INIT STORYLIST]');
        _.bindAll(this, "render"); // to solve the 'this' issue

        //this.collection.bind('change', this.render);
        //this.collection.bind('add', this.render);

		$.subscribe('/feeds/downloadcomplete', this.render);

        this.showFullStoryList  = this.options.showFullStoryList;
        this.showTopStories     = this.options.showTopStories;
        this.collapseAccordions = this.options.collapseAccordions;

        this.node = $(this.el);
    },
    render: function() {
		//ws.log('[STORY LIST RENDER]');


		if (this.cid != ws.activeStoryListViewId) {
			//ws.log(this.cid, ws.activeStoryListViewId);
			//ws.log('XXX CATCH OLD VIEW RENDER XXX');
			return false;
		}

        //ws.log('[STORYLIST] rendering view');

        var self        = this,
            node        = this.node,
            genres      = _.keys(ws.config.availableGenres),
            genreOrder  = ws.store('genreOrder'),
            storyIdList = []; // Keep track of stories already listed, so to prevent duplicates

        node.empty();
        // This shows the top X stories, typically from the Homepage
        if (self.showTopStories == true) {

            var homepageStories = ws.stories.getByGenre('homepage');

            // Protect against the homepage feed coming back with less stories than needed
            if (homepageStories.length >= ws.config.numStoriesOnHome) {
                for (var i=0; i<ws.config.numStoriesOnHome; i++) {

                    var story     = homepageStories.at(i),
                        className = 'top-story ';

                    // Add last classname - for some reason li:last-child isn't working in Webkit
                    if (i+1 == ws.config.numStoriesOnHome) { className += 'last '; }

                    var storyRow = new ws.views.StoryListRow({ model: story });
                    node.append($(storyRow.render().el).addClass(className));
                    storyIdList.push(story.id);
                };
            }
        }

        // Do other genres
        _.each(genreOrder, function(selected, genreId) {

			// checks to see if on homepage and genre selected
            if (!selected) return;
			//ws.log("################################## Do other genres");

            var genre        = ws.config.availableGenres[genreId],
                genreStories = self.collection.getByGenre(genreId);

            ws.views.renderLastUpdated(genreId);

			if (genreStories.length) {
                var numStories     = genreStories.length > ws.config.numStoriesOnHome ? ws.config.numStoriesOnHome : genreStories.length,
                    accordionClass = self.collapseAccordions ? '' : 'open';

                if (self.showFullStoryList) { numStories = genreStories.length; }

                var contentNode = $('<div class="content-group '+accordionClass+'"></div>');
                contentNode.append('<h2>'+genre.title+'</h2>');

                var storiesNode = $('<div class="content"></div>');
                // check for android, add this attribute
                if (ws.viewsAndroid) {
                	storiesNode = $('<div class="content" data-groupgenre="'+genreId+'"></div>');
                }

                var storiesShown = 0;
                genreStories.forEach(function(storyModel, i) {
                    if (storiesShown>=numStories) return false;

                    var storyId = genreStories.at(i).id;
                    if (_.include(storyIdList, storyId)) return false; // Skip story if it's already been displayed earlier

                    var storyEntry = new ws.views.StoryListRow({ model: storyModel });
                    storiesNode.append(storyEntry.render().el);
                    storyIdList.push(storyId);
                    storiesShown++;
                });

                if (self.showFullStoryList == false) {
                    // Show 'More stories' only if the full list of stories by genre is not displayed
                    var moreStoriesText = ws.config.vocab.readMoreByGenre.replace('{genre}', genre.title);
                    storiesNode.append('<a href="#" data-genre="'+genreId+'" class="more-stories">'+moreStoriesText+'</a>');
                }

                contentNode.append(storiesNode);

                node.append(contentNode);
            }

        });
        //ws.log('END Rendering storyList');
		ws.homepageList = storyIdList;
		ws.views.scrollers.refresh(this.el);

        return this;
    },

    toggleContentAccordion: function(e) {
        var node = $(e.currentTarget).parent();

        node.toggleClass('open');
        ws.views.scrollers.refresh(this.el);
    },


    goToCategory: function(e) {
		//ws.log('from homepage (story list)');
        var genreId = $(e.currentTarget).attr('data-genre');
		ws.currentGenre = genreId;
		ws.currentHomeTab = null;

        var renderedList = new ws.views.generalStoryList({
            collection: ws.stories.fetchGenre(genreId),
            id: 'ws-categorised-list'
        }).render().el;

        $('#ws-categorised-list').remove();
        ws.views._activeMainView.contentNode.append(renderedList); // This needs to be handled better

        _.delay(function() { ws.views._activeMainView.goToCategoryView(); }, 50);
    },


});


ws.views.generalStoryList = Backbone.View.extend({
    //tagName:   'ul',
    //className: 'ws-entries',
    initialize: function() {
        _.bindAll(this, "render"); // to solve the 'this' issue

		this.collection.bind('reset', this.render);
        //this.collection.bind('change', this.render);
        this.collection.bind('add', this.render);

        $(this.el).empty(); // Clear the element when starting
    },
    render: function() {
		//ws.log('[GENERAL STORY LIST] rendering view');
        var self        = this,
            node        = $(this.el),
            storyIdList = []; // Keep track of stories already listed, so to prevent duplicates


        node.html(ws.views.templates.pulldown + '<ul class="ws-entries" data-groupgenre="'+this.collection.genreId+'"></ul>');
		var listNode = node.find('.ws-entries');

        this.collection.forEach(function(story) {
            // If 'filterGenre' option is specified, only display those items
            if (!_.isUndefined(self.options.filterGenre)) {
                var filterGenre = self.options.filterGenre;
                if (filterGenre != story.get('genre') && filterGenre != story.get('topic')) return;
            }

            var storyEntry = new ws.views.StoryListRow({ model: story });

            // Check if story has been displayed already, otherwise skip
            if (_.indexOf(storyIdList, story.id) == -1) {
                listNode.append(storyEntry.render().el);
                storyIdList.push(story.id);
             }

        });

        ws.views.scrollers.refresh();
        return this;
    }
});





ws.views.mostRecentList = Backbone.View.extend({
    tagName:   "ul",
    className: "ws-entries",
    attributes: {
        "data-groupgenre": "latest_stories"
    },
    initialize: function() {
        _.bindAll(this, "render");
        //this.collection.bind('change', this.render);
        this.collection.bind('add', this.render);

		$.subscribe('/feeds/downloadcomplete', this.render);

    },
    render: function() {
        //ws.log('[MOST RECENT LIST RENDER]');

        var self = this,
            node = $(this.el),
            latestStoryIds = [];

        node.empty();

        if (ws.config.rollingNews.enabled === true) {
            // If Rolling News is enabled, then use that feed for this view
            ws.stories.getByGenre('rolling_news').forEach(function(story) {
                var storyEntry = new ws.views.StoryListRow({ model: story });
                node.append(storyEntry.render().el);
            });

        } else {
            // Otherwise just sort all other stories in date order
            ws.stories.sortByMostRecent().forEach(function(story, i) {

                if (i >= ws.latestNews.numOfItems) {return;}

                var storyEntry = new ws.views.StoryListRow({ model: story });

                //ws.log(story);
                // only add the stories that are output into latest news array
                if (!_.isUndefined(story.attributes.genre) && !_.isUndefined(ws.config.availableGenres[story.attributes.genre])) {
                    //ws.log('[WARNING] Vernacular for title "'+story.attributes.genre+'" found. Skipping other stories');
                    latestStoryIds.push(story.id);
                }
                node.append(storyEntry.render().el);
            });
            ws.latestList = latestStoryIds;
            //ws.log(ws.latestList);
        }

        return this;
    }
});


ws.views.mostPopularList = Backbone.View.extend({
    tagName:   "ul",
    className: "ws-entries",
    initialize: function() {
        _.bindAll(this, "render");
        this.collection.bind('change', this.render);
        //this.collection.bind('add', this.render);
        this.node = $(this.el);

        // Since this is a separate collection from the main story list,
        // it needs to trigger its own update to the Story View when the model loads
        // this.collection.bind('change', ws.views._activeStoryView.render);

        //this.collection.fetch();
    },
    render: function() {
        var node = this.node;

        node.empty();
        _.each(this.collection.models, function(story, i) {
            var storyEntry = new ws.views.StoryListRow({ model: story });
            var fragment = $(storyEntry.render().el);

            fragment.append('<div class="rank">'+(i+1)+'</div>');
            node.append(fragment);
        });
        return this;
    }
});