describe("App config setup", function() {

    beforeEach(function() {

    });

    afterEach(function() {

    });

    it("should attempt an ajax call", function() {
        spyOn($, "ajax");
        ws.checkRemoteConfig();
        expect($.ajax).toHaveBeenCalled();
    });

    it("should make ajax call to the correct url", function() {
        spyOn($, "ajax");
        ws.checkRemoteConfig();
        expect($.ajax.mostRecentCall.args[0]["url"]).toEqual(ws.remoteConfigRoot+'/config.json');
    });

    it("should read status", function() {
        spyOn($, "ajax").andCallFake(function(options) {
            options.success({"appEnabled":true});
        });
        ws.checkRemoteConfig();
        //console.log($.ajax.mostRecentCall.args[0]['success']);
        //$.ajax.mostRecentCall.args[0].success({"appEnabled": true});
        expect(ws.env.remoteConfig).toEqual({"appEnabled":true});
    });

    it("should load and read online service config", function() {
        var callback = jasmine.createSpy();

        $.ajax({
            url: ws.remoteConfigRoot + '/services/' + ws.service + '.js',
            dataType: 'script',
            cache: false,
            crossDomain: true,
            success: function(data) {
                //console.log('success');
                callback();
            },
            error: function() {
                //console.log('error');
            }
        });

        waitsFor(function() {
            return callback.callCount > 0;
        });

        var expectedServiceRoot = ws.service;
        if (expectedServiceRoot == 'brasil') { expectedServiceRoot = 'portuguese'; }

        runs(function() {
            expect(callback).toHaveBeenCalled();
            expect(ws.config.serviceRoot).toEqual('http://wsapps.bbc.co.uk/' + expectedServiceRoot);
        });
    });

    it("should load and read local service config", function() {
        var callback = jasmine.createSpy();

        $.ajax({
            url: 'scripts/services/' + ws.service + '.js',
            dataType: 'script',
            cache: false,
            crossDomain: true,
            success: function(data) {
                //console.log('success');
                callback();
            },
            error: function() {
                //console.log('error');
            }
        });

        waitsFor(function() {
            return callback.callCount > 0;
        });

        var expectedServiceRoot = ws.service;
        if (expectedServiceRoot == 'brasil') { expectedServiceRoot = 'portuguese'; }

        runs(function() {
            expect(callback).toHaveBeenCalled();
            expect(ws.config.serviceRoot).toEqual('http://wsapps.bbc.co.uk/' + expectedServiceRoot);
        });
    });

});

describe("save to and delete from local storage", function() {

    var testText = "abcdefghijklmnopqrstuvwxyz";

    it("should save to local storage", function() {
        //console.log(ws.store('testText', testText));
        expect(ws.store('testText', testText)).toBeTruthy();
    });

    it("should save accurately", function() {
        expect(ws.store('testText')).toEqual(testText);
    });

    it("should remove from local storage", function() {
        localStorage.removeItem("testText");
        expect(ws.store('testText')).toEqual(null);
    });

});

describe("save to ws.prefs", function() {

    var testText = "abcdefghijklmnopqrstuvwxyz";

    it("should save to local storage", function() {
        //console.log(ws.prefs('testText', testText));
        expect(ws.prefs('testText', testText)).toBeTruthy();
    });

    it("should save accurately", function() {
        expect(ws.prefs('testText')).toEqual(testText);
    });

});

/*
describe("load json feed", function() {

    beforeEach(function() {

        var callback = jasmine.createSpy();

        var stories = new ws.models.StorySet({url: "http://www.bbc.co.uk/mundo/index.app.json" });


        stories.fetch({
            success: function(data) {
                console.log('success');
                callback();
            }

        });

        waitsFor(function() {
            //console.log(stories);
            //console.log(callback.callCount);
            return callback.callCount > 0;
        });

        runs(function() {
            expect(callback).toHaveBeenCalled();
        });


    });

    it("callback", function() {
        expect(callback).toHaveBeenCalled();
    });

});
*/
describe("App initialise", function() {

    var finalList = {};
    var outputNode;
    var beforeEachCalled = false;

    beforeEach(function() {

        //ws.config = {}
        //if (!beforeEachCalled) {
            //console.log('beforeEach');
            var callback = jasmine.createSpy();

            jasmine.getFixtures().fixturesPath = 'tests/jasmine/spec/javascripts/fixtures/';
            loadFixtures('index.html');

            //console.log($('#main'));

            $.ajax({
                url: 'scripts/services/' + ws.service + '.js',
                dataType: 'script',
                cache: false,
                crossDomain: true,
                success: function(data) {
                    ws.init();

                    //_.delay(function(){callback();}, 5000);
                },
                error: function() {
                    //console.log('error');
                }
            });

            waitsFor(function() {
                //console.log('waitsfor');
                //return callback.callCount > 0 ;
                return $('#main').find('#ws-latest-list').children().length > 1;
            });

            runs(function() {
                //console.log('runs');
                //console.log($('#main'));
                beforeEachCalled = true;
                //expect(callback).toHaveBeenCalled();
            });

        //}

    });

    afterEach(function() {
        //console.log('after');
        localStorage.clear();
        //Backbone.history.stop()
    });

    it("service config is ready", function() {
        var expectedServiceRoot = ws.service;
        if (expectedServiceRoot == 'brasil') { expectedServiceRoot = 'portuguese'; }

        expect(ws.config.serviceRoot).toEqual('http://wsapps.bbc.co.uk/' + expectedServiceRoot);
    });

    it("body DOM is created with id = service", function() {
        expect($("body").attr('id')).toEqual(ws.service);
    });

    it("genre list is ready", function() {
        expect(ws.store('genreOrder')).toBeDefined();
    });

    describe("- home", function() {

        it("home view changes tabs on tap", function() {
            $('#main').find('a[data-tabtarget="#ws-top-news-list"]').trigger('tap');
            expect($('#main').find('a[data-tabtarget="#ws-top-news-list"]').parent().hasClass('active')).toBeTruthy();

            $('#main').find('a[data-tabtarget="#ws-latest-list"]').trigger('tap');
            expect($('#main').find('a[data-tabtarget="#ws-latest-list"]').parent().hasClass('active')).toBeTruthy();
            //console.log($('#main'));
        });

        it("accordion toggles between open and closed on tap", function() {
            var firstContentGroup = $('#main').find('.content-group:eq(0)');

            // all the accordions are open at first
            expect(firstContentGroup.hasClass('open')).toBeTruthy();
            firstContentGroup.find('h2').trigger('tap');
            expect(firstContentGroup.hasClass('open')).toBeFalsy();
            firstContentGroup.find('h2').trigger('tap');
            expect(firstContentGroup.hasClass('open')).toBeTruthy();
        });

        it("home view lists populated", function() {
            expect($('#main').find('#ws-top-news-list').children().length).toBeGreaterThan(2);
            expect($('#main').find('#ws-latest-list').children().length).toBeGreaterThan(1);
        });

    });

    describe("- most popular (livestats)", function() {

        beforeEach(function() {

            $('#main').find('#ws-tab-bar .ws-popular').trigger('tap');

            waitsFor(function() {
                return $('#main').find('#ws-most-read').children().length > 0;
            });

            runs(function() {

            });

        });

        it("footer tab is active", function() {
            expect($('#main').find('#ws-tab-bar .ws-popular').hasClass('active')).toBeTruthy();
        });

        it("creates most-popular view", function() {
            expect($('#main').find('#ws-popular').length).toEqual(1);
        });

        it("livestats populated", function() {
            expect($('#main').find('#ws-most-read').children().length).toBeGreaterThan(0);
        });

    });

    describe("- categories", function() {

        beforeEach(function() {

            $('#main').find('#ws-tab-bar .ws-categories').trigger('tap');

            waitsFor(function() {
                return $('#main').find('#ws-categorylist').children().length > 0;
            });

            runs(function() {

            });

        });

        it("footer tab is active", function() {
            expect($('#main').find('#ws-tab-bar .ws-categories').hasClass('active')).toBeTruthy();
        });

        it("creates category list view", function() {
            expect($('#main').find('#ws-categories').length).toEqual(1);
        });

        it("list of categories populated", function() {
            expect($('#main').find('#ws-categorylist').children().length).toBeGreaterThan(0);
        });

    });

    describe("- settings", function() {

        beforeEach(function() {

            $('#main').find('#ws-tab-bar .ws-settings').trigger('tap');

            waitsFor(function() {
                return $('#main').find('#ws-settings').children().length > 0;
            });

            runs(function() {

            });

        });

        it("footer tab is active", function() {
            expect($('#main').find('#ws-tab-bar .ws-settings').hasClass('active')).toBeTruthy();
        });

        it("creates settings view", function() {
            expect($('#main').find('#ws-settings').length).toEqual(1);
        });

        it("list of genres populated", function() {
            expect($('#main').find('.genre-list').children().length).toBeGreaterThan(0);
        });

        it("list of selected genres corresponds to local storage", function() {
            _.each(ws.store('genreOrder'), function(selected, genre) {
                if (selected == true) {
                    expect($('#main').find('.genre-list').find('#' + genre).hasClass('selected')).toBeTruthy();
                }
            });
        });

        it("selecting genre's affects UI and local storage", function() {
            var genreItem = $('#main').find('.genre-list li:not(.selected):eq(0)'),
                thisGenre = genreItem.attr('id');

            genreItem.trigger('tap');

            expect(genreItem.hasClass('selected')).toBeTruthy();
            expect(ws.store('genreOrder')[thisGenre]).toBeTruthy();

            genreItem.trigger('tap');

            expect(genreItem.hasClass('selected')).toBeFalsy();
            expect(ws.store('genreOrder')[thisGenre]).toBeFalsy();

        });

        it("toggling share stats affects stored preferences", function() {

            var initialStats = ws.prefs('shareStats');

            $('#ws-settings').find('.share-stats').trigger('tap');

            var resultingStats = ws.prefs('shareStats');

            expect(initialStats).not.toEqual(resultingStats);

        });

        it("toggling default tab updates stored preferences", function() {

            runs(function() {

                var initialTab = ws.prefs('defaultHomeTab');

                $('#ws-settings').find('.set-default-tab:not(.active)').trigger('tap');

                var resultingTab = ws.prefs('defaultHomeTab');

                expect(resultingTab).not.toEqual(initialTab);

                $('#main').find('#ws-tab-bar .ws-home').trigger('tap');

            });

            waits(500);

            runs(function() {

                var openTabIndex = ws.prefs('defaultHomeTab') == 'latestnews' ? 1 : 0;

                expect($('#main').find('#ws-view-chooser li:eq('+openTabIndex+')').hasClass('active')).toBeTruthy();

            });


        });

    });

    describe("- story", function() {

        var thisStoryId,
            thisStoryIndex = null;

        beforeEach(function() {
            //console.log($('#main'));
            //console.log($('#main').find('#ws-top-news-list .ws-entry:eq(0)'));
            $('#main').find('#ws-top-news-list .ws-entry:eq(0)').trigger('tap');



            waitsFor(function() {
                //console.log($('#story').find('.ws-story-content'));
                return $('#story').find('.ws-story-content').children().length > 0;
            })

            runs(function() {
                //console.log('before each');
                //console.log($('.ws-story-content'));

                thisStoryId = ws.views._activeStoryView.model.id;

                if (thisStoryIndex == null) {
                    _.each(ws.homepageList, function(model,i) {
                        if(model.toString() == thisStoryId) {
                            thisStoryIndex = i;
                        }
                    });
                }

            });
        });

        it("story exists", function() {
            expect($('#story').find('.ws-story-content').length).toBeGreaterThan(0);
        });

        it("prev navigation is disabled", function() {
            expect($('#story').find('#story-navigation .prev').hasClass('disabled')).toBeTruthy();
        });

        it("next navigation is enabled", function() {
            expect($('#story').find('#story-navigation .next').hasClass('disabled')).toBeFalsy();
        });

        it("text size can be increased and decreased", function() {

            var re = /\btext-size-(.\d?)/;
            var initialSize = $('.ws-story-content').attr('class').match(re)[1];

            $('#story').find('.ws-text-size .larger a').trigger('tap');

            var secondSize = $('.ws-story-content').attr('class').match(re)[1];

            expect(secondSize).toBeGreaterThan(initialSize);

            $('#story').find('.ws-text-size .smaller a').trigger('tap');

            var thirdSize = $('.ws-story-content').attr('class').match(re)[1];

            expect(secondSize).toBeGreaterThan(thirdSize);

        });

        it("share button opens share panel", function() {

            runs(function() {
                $('#story').find('.share').trigger('tap');
            });

            waits(500);

            runs(function() {
                expect($('.sharetools').children().length).toBeGreaterThan(0);
            });

        });

        it("story back button hides view", function() {

            runs(function() {
                $('#story').find('#ws-back').trigger('tap');
            });

            waits(1000);

            runs(function() {
                expect($('#story').hasClass('active')).toBeFalsy();
            });
        });

        describe("next navigation moves to next model in collection", function() {

            var nextStoryId,
                nextStoryIndex;

            beforeEach(function() {
                $('#story').find('#story-navigation .next').trigger('tap');

                waitsFor(function() {
                    return ws.views._activeStoryView.model.id != thisStoryId;
                });

                runs(function() {
                    nextStoryId = ws.views._activeStoryView.model.id;
                    nextStoryIndex = thisStoryIndex + 1;
                });

            });

            it("next story is next story in collection", function() {
                expect(ws.homepageList[nextStoryIndex]).toEqual(nextStoryId);
            });

            it("prev naviagtion is now enabled", function() {

                waits(500);

                runs(function() {
                    expect($('#story').find('#story-navigation .prev').hasClass('disabled')).toBeFalsy();
                });

            });

            describe("prev navigation returns to prev model in collection", function() {

                beforeEach(function() {

                    $('#story').find('#story-navigation .prev').trigger('tap');

                    waitsFor(function() {
                        return ws.views._activeStoryView.model.id != nextStoryId;
                    });

                    runs(function() {

                    });

                });

                it("prev story is prev story in collection", function() {
                    expect(ws.views._activeStoryView.model.id).toEqual(thisStoryId);
                });

            });


        });


    });



    describe("- story with picture gallery", function() {

        beforeEach(function() {
            ws.thisRouter.storyByUrl("tests/json/120926_galeria_premios_arquitectura_np.shtml");

            waitsFor(function() {
                return $('#story').find('.picture-gallery-container').length > 0;
            });

            runs(function() {

            });

        });

        it("gallery cta exists", function() {
            expect($('#story').find('.picture-gallery-container').length).toBeGreaterThan(0);
        });

        it("gallery object exists with children", function() {
            expect(ws.views._activeStoryView.gallery.length).toBeGreaterThan(0);
        });

        describe("open the gallery", function() {

            beforeEach(function() {
                $('#story').find('.picture-gallery-container').trigger('tap');

                waitsFor(function() {
                    return $('#picture-gallery').find('ul li').length > 1;
                });

                runs(function() {
                    //console.log(ws.views._activeStoryView.pictureGalleryView.scroller.currPageY);
                });
            });

            it("opens a new view", function() {
                expect($('#picture-gallery').hasClass('active')).toBeTruthy();
            });

            it("previous button is disabled", function() {
                expect($('#picture-gallery').find('.previous').hasClass('disabled')).toBeTruthy();
            });

            describe("navigate 'next'", function() {

                beforeEach(function() {
                    ws.views._activeStoryView.pictureGalleryView.nextImage();

                    waitsFor(function() {
                        return ws.views._activeStoryView.pictureGalleryView.currImageIndex > 0;
                    });

                    runs(function() {

                    });
                });

                it("goes to next image in gallery", function() {
                    expect(ws.views._activeStoryView.pictureGalleryView.currImageIndex).toEqual(1);
                });

                it("previous button is enabled", function() {
                    expect($('#picture-gallery').find('.previous').hasClass('disabled')).toBeFalsy();
                });

                describe("navigate 'previous'", function() {

                    beforeEach(function() {
                        ws.views._activeStoryView.pictureGalleryView.previousImage();

                        waitsFor(function() {
                            return ws.views._activeStoryView.pictureGalleryView.currImageIndex < 1;
                        });

                        runs(function() {

                        });
                    });

                    it("goes to previous image in gallery", function() {
                        expect(ws.views._activeStoryView.pictureGalleryView.currImageIndex).toEqual(0);
                    })

                });

            });

            describe("navigate to last image", function() {

                beforeEach(function() {
                    ws.views._activeStoryView.pictureGalleryView.scroller.scrollToPage(0, ws.views._activeStoryView.pictureGalleryView.galleryLength - 1, 750);

                    waitsFor(function() {
                        return ws.views._activeStoryView.pictureGalleryView.currImageIndex == ws.views._activeStoryView.pictureGalleryView.galleryLength - 1;
                    });

                    runs(function() {

                    });
                });

                it("next button is disabled", function() {
                    expect($('#picture-gallery').find('.next').hasClass('disabled')).toBeTruthy();
                });

            });

            it("picture captions are displayed", function() {
                expect($('#picture-gallery').find('p').css('opacity')).toEqual('1');
            });

            describe("hide captions", function() {

                beforeEach(function() {
                    $('#picture-gallery').find('.captions').trigger('tap');

                    waitsFor(function() {
                        return $('#picture-gallery').hasClass('showDetails') == false;
                    });

                    runs(function() {

                    });
                });

                it("picture captions are hidden", function() {
                    expect($('#picture-gallery').find('.captionContainer').css('opacity')).toBeLessThan(1);
                });

            });


        });

    });

    /*
    describe("Views start", function() {

        var stories, mainView, view, outputNode, tabsActive, outputNode2;

        beforeEach(function() {

            var callback2 = jasmine.createSpy();

            stories = new ws.models.StorySet({ url: 'http://www.bbc.co.uk/russian/index.app.json' });

            stories.fetch({
                success: function() {
                    callback2();
                }
            });

            waitsFor(function() {
                return callback2.callCount > 0 ;
            });

            runs(function() {
                view = new ws.views.homeView({ collection: stories });
                outputNode = view.render().el;
            });
        });

        it("stories exists", function() {
            expect(stories).toBeDefined();
        });

        it("home view exists", function() {
            expect(view).toBeDefined();
        });

        it("main view exists", function() {
            //console.log(ws.views.MainView);
        });

        it("home view tabs are rendered", function() {
            tabsActive = $('#ws-view-chooser li.active', outputNode).length;
            expect(tabsActive).toEqual(1);
        });

        it("home view changes tabs on tap", function() {
            //spyOn(view, 'handleViewTab');

            $(view.el).find('a[data-tabtarget="#ws-top-news-list"]').trigger('tap');

            expect($(view.el).find('a[data-tabtarget="#ws-top-news-list"]').parent().hasClass('active')).toBeTruthy();

            $(view.el).find('a[data-tabtarget="#ws-latest-list"]').trigger('tap');

            expect($(view.el).find('a[data-tabtarget="#ws-latest-list"]').parent().hasClass('active')).toBeTruthy();
            //console.log($(view.el));
        });

    });

    */


    /*it("stories object is populated", function() {
        //console.log(ws.stories.length);
        //expect(ws.stories.length).toBeGreaterThan(0);
    });*/



    /*describe("home view", function() {

        var view,
            outputNode,
            tabsActive;

        beforeEach(function() {
            view = new ws.views.homeView({ collection: ws.stories });
            outputNode = view.render().el;
            tabsActive = $('#ws-view-chooser li.active', outputNode).length;
        });

        it("is rendered", function() {
            expect(tabsActive).toEqual(1);
        });

        it("changes tabs", function() {
            //console.log($('#ws-view-chooser li', outputNode)[1]);

            view.handleViewTab($('#ws-view-chooser li', outputNode)[1]);
            console.log(view.el);
            console.log(outputNode);
            expect(tabsActive).toEqual(1);
        });

    });*/

});






// TRYING TO RUN INIT() A LITTLE AT A TIME
// ROUTER DOESN'T WORK


/*
describe("testing views", function() {
    var view, stories;

    beforeEach(function() {
        var callback = jasmine.createSpy();

        jasmine.getFixtures().fixturesPath = 'tests/jasmine/spec/javascripts/fixtures/';
        loadFixtures('index.html');

        $.ajax({
            url: 'scripts/services/' + ws.service + '.js',
            dataType: 'script',
            cache: false,
            crossDomain: true,
            success: function(data) {
                callback();
            },
            error: function() {
                //console.log('error');
            }
        });

        waitsFor(function() {
            return callback.callCount > 0 ;
        });

        runs(function() {
            //recreate init()

            //init main view object
            ws.views.init();

            //online?
            ws.connection.check();

            //appEnabled?
            ws.checkRemoteConfig();

            //genre list
            var finalList = {};
            _.each(ws.config.availableGenres, function(genre, genreId) {
                if (genre.enabled == true) {
                    finalList[genreId] = genre.selected;
                }
            });
            ws.genreOrder = finalList;
            ws.store('genreOrder', finalList);

            // Set up collections for most popular stories
            ws.popularStories = {
                read:    new ws.models.mostPopular(),
                watched: new ws.models.mostWatched()
            }

            //stories model
            ws.stories = new ws.models.StorySet({ url: 'http://www.bbc.co.uk/' + ws.service + '/index.app.json' });

        });


    });

    describe("- main", function() {

        beforeEach(function() {
            var callback2 = jasmine.createSpy();
            //ajax fetch the stories
            ws.stories.fetch({
                success: function(data) {
                    callback2();
                }
            });

            waitsFor(function() {
                return callback2.callCount > 0;
            });

            runs(function() {
                //create mainview
                ws.views._activeMainView = new ws.views.MainView();
                ws.views._activeStoryView  = new ws.views.StoryView();

                //ws.router = new ws.router();
                //Backbone.history.start();
                //ws.router.navigate('', true);

                //fetchAllStories
                ws.stories.fetchAllStories({ skipHomepage: false });
            });
        });


        it("view object exists", function() {
            expect(ws.views._activeMainView).toBeDefined();
        });

        describe("- home", function() {

            beforeEach(function() {
                var callback3 = jasmine.createSpy();

                // wait for fetchAllStories??
                //_.delay(function(){callback3();}, 1000);

                waitsFor(function() {
                    return $('#main').find('#ws-latest-list').children().length > 1;
                });

                runs(function() {

                });
            });


            it("home view changes tabs on tap", function() {
                $('#main').find('a[data-tabtarget="#ws-top-news-list"]').trigger('tap');
                expect($('#main').find('a[data-tabtarget="#ws-top-news-list"]').parent().hasClass('active')).toBeTruthy();

                $('#main').find('a[data-tabtarget="#ws-latest-list"]').trigger('tap');
                expect($('#main').find('a[data-tabtarget="#ws-latest-list"]').parent().hasClass('active')).toBeTruthy();
                //console.log($('#main'));
            });

            it("accordion toggles between open and closed on tap", function() {
                var firstContentGroup = $('#main').find('.content-group:eq(0)');

                // all the accordions are open at first
                expect(firstContentGroup.hasClass('open')).toBeTruthy();
                firstContentGroup.find('h2').trigger('tap');
                expect(firstContentGroup.hasClass('open')).toBeFalsy();
                firstContentGroup.find('h2').trigger('tap');
                expect(firstContentGroup.hasClass('open')).toBeTruthy();
            });

            it("home view lists populated", function() {
                expect($('#main').find('#ws-top-news-list').children().length).toBeGreaterThan(2);
                expect($('#main').find('#ws-latest-list').children().length).toBeGreaterThan(1);
            });

            describe("- story", function() {

                beforeEach(function() {

                    //$('#main').find('#ws-top-news-list .ws-entry:eq(0)').trigger('tap');

                    ws.router.storyById(19958468);


                });

                it("exists", function() {
                    console.log($('#story'));
                });

            });

        });

        describe("- most popular (livestats)", function() {

            beforeEach(function() {

                $('#main').find('#ws-tab-bar .ws-popular').trigger('tap');

                waitsFor(function() {
                    return $('#main').find('#ws-most-read').children().length > 0;
                });

                runs(function() {

                });

            });

            it("footer tab is active", function() {
                expect($('#main').find('#ws-tab-bar .ws-popular').hasClass('active')).toBeTruthy();
            });

            it("creates most-popular view", function() {
                expect($('#main').find('#ws-popular').length).toEqual(1);
            });

            it("livestats populated", function() {
                expect($('#main').find('#ws-most-read').children().length).toBeGreaterThan(0);
            });

        });

        describe("- categories", function() {

            beforeEach(function() {

                $('#main').find('#ws-tab-bar .ws-categories').trigger('tap');

                waitsFor(function() {
                    return $('#main').find('#ws-categorylist').children().length > 0;
                });

                runs(function() {

                });

            });

            it("footer tab is active", function() {
                expect($('#main').find('#ws-tab-bar .ws-categories').hasClass('active')).toBeTruthy();
            });

            it("creates category list view", function() {
                expect($('#main').find('#ws-categories').length).toEqual(1);
            });

            it("list of categories populated", function() {
                expect($('#main').find('#ws-categorylist').children().length).toBeGreaterThan(0);
            });

        });

        describe("- settings", function() {

            beforeEach(function() {

                $('#main').find('#ws-tab-bar .ws-settings').trigger('tap');

                waitsFor(function() {
                    return $('#main').find('#ws-settings').children().length > 0;
                });

                runs(function() {

                });

            });

            it("footer tab is active", function() {
                expect($('#main').find('#ws-tab-bar .ws-settings').hasClass('active')).toBeTruthy();
            });

            it("creates settings view", function() {
                expect($('#main').find('#ws-settings').length).toEqual(1);
            });

            it("list of genres populated", function() {
                expect($('#main').find('.genre-list').children().length).toBeGreaterThan(0);
            });

            it("list of selected genres corresponds to local storage", function() {
                _.each(ws.store('genreOrder'), function(selected, genre) {
                    if (selected == true) {
                        expect($('#main').find('.genre-list').find('#' + genre).hasClass('selected')).toBeTruthy();
                    }
                });
            });

            it("selecting genre's affects UI and local storage", function() {
                var genreItem = $('#main').find('.genre-list li:not(.selected):eq(0)'),
                    thisGenre = genreItem.attr('id');

                genreItem.trigger('tap');

                expect(genreItem.hasClass('selected')).toBeTruthy();
                expect(ws.store('genreOrder')[thisGenre]).toBeTruthy();

                genreItem.trigger('tap');

                expect(genreItem.hasClass('selected')).toBeFalsy();
                expect(ws.store('genreOrder')[thisGenre]).toBeFalsy();

            });

        });


    });


});

describe("testing story view", function() {
    var view;

    beforeEach(function() {
        jasmine.getFixtures().fixturesPath = 'tests/jasmine/spec/javascripts/fixtures/';
        loadFixtures('index.html');

        ws.views.init();

        view = new ws.views.StoryView();
    });

    it("exists without errors", function() {
        //console.log($('#story').find('.ws-header'));
        //console.log(view);
        expect(view).toBeDefined();
    });
});

*/