/* Some boilerplate */
QUnit.config.autorun = false;
QUnit.config.autostart = false;
$(document).ready(function() {
    localStorage.clear();
    $('#app-version').text('v'+ws.version);
    $('#app-service').text(ws.service);
	
	var scroller = new iScroll("wrapper");

	$.getScript('scripts/services/'+ws.service+'.js', function() {  
		ws.debug = false;
		ws.init();
		QUnit.start();
	});
	
	QUnit.done = function() {
		scroller.refresh();
	}
	
	$('#qunit-tests').bind('click', function() {
		scroller.refresh();
	});	
	

	/* Storage Tests */
		module("Storage");
		test("Using localStorage - ws.store", function() {
			var testText = "abcdefghijklmnopqrstuvwxyz";    
			equals(ws.store("testText", testText), true, "Write text to localStorage");      
			equals(ws.store("testText"), testText, "Read text from localStorage");      
			
			
			var testArray = [1,12,123,1234,12345,123456];
			equals(ws.store("testArray", testArray), true, "Write array to localStorage");
			deepEqual(ws.store("testArray"), testArray, "Read array from localStorage");        
			
			var testObject = { key1:"value1", key2:"value2" }
			equals(ws.store("testObject", testObject), true, "Write object to localStorage");
			deepEqual(ws.store("testObject"), testObject, "Read object from localStorage");
			
			localStorage.removeItem("testObject");
			equals(ws.store("testObject"), null, "remove object from localStorage");

		});


	/* Prefs Tests */
		module("Preferences");
		test("Using ws.prefs()", function() {
			var testText = "abcdefghijklmnopqrstuvwxyz";    
			equals(ws.prefs("testText", testText), true, "Write text to prefs");      
			equals(ws.prefs("testText"), testText, "Read text from prefs");      
			
			var testArray = [1,12,123,1234,12345,123456];
			equals(ws.prefs("testArray", testArray), true, "Write array to prefs");
			deepEqual(ws.prefs("testArray"), testArray, "Read array from prefs");        
			
			var testObject = { key1:"value1", key2:"value2" }
			equals(ws.prefs("testObject", testObject), true, "Write object to prefs");
			deepEqual(ws.prefs("testObject"), testObject, "Read object from prefs");        
		});


	/* Remote Feeds */
		module("Remote Feeds");  
		test("Master config.json", function() {
			var feedUrl = ws.remoteConfigRoot+'/config.json';
			stop();
			$.ajax({
				url:      feedUrl,
				dataType: "json",
				useCache: true,
				success:  function(data) {
					ok(true, "Retrieved ok with values:"+JSON.stringify(data))
					start();
				},
				error:    function() {
					ok(false, "Feed failed retrieval: "+feedUrl)
					start();
				}
			});
		});
		test("Remote "+ws.service+".js", function() {
			var feedUrl = ws.remoteConfigRoot+'/services/'+ws.service+'.js';
			stop();
			$.ajax({
				url:      feedUrl,
				dataType: "script",
				useCache: true,
				success:  function(data) {
					ok(true, "Feed exists")
					start();
				},
				error:    function() {
					ok(false, "Feed failed retrieval: "+feedUrl)
					start();
				}
			});
		});



	/* Feed ingestion */
		// Index Level
		module("Index Feeds");
		asyncTest("Index JSON Feed", function() {
			expect(4);
			
			var stories = new ws.models.StorySet({url: "tests/json/homepage.app.json" });

			stories.fetch({
				success: function() {                                
					equal(stories.length, 5, "Expecting 5 stories in feed");        
														
					equal(stories.at(0).get('title'), "This is the title of the first story", "Expecting story title to be 'This is the title of the first story'");            
					
					var mostRecentStory = stories.sortByMostRecent().at(0).get('title');
					equal(mostRecentStory, "This is the title of the third story", "Sorting by most recent, the third story should come first");

					var teaserHasBase64 = stories.at(0).get('teaser').indexOf('data:image/jpeg;base64,/9j/4AAQSkZJ') == 0;
					equal(teaserHasBase64, true, "Story teaser attribute should be populated with Base64");
					
					start();
				}
			});
			//console.log(ws.stories);    
		});

		asyncTest("Index JSON Feed with missing Base64 Images", function() {
			expect(2);
			var stories = new ws.models.StorySet({url: "tests/json/homepage_no_b64.app.json" });
					
			stories.fetch({
				success: function() {                
					equal(stories.length, 5, "Expecting 5 stories in feed");        
					
					var count = 0;
					stories.bind("change", function(model) {
						var teaserHasBase64 = model.get('teaser').indexOf('data:image/jpeg;base64,') == 0;
											
						count++;
						if (count == 5) { 
							equal(teaserHasBase64, true, "5 Stories retrieved Base64 teaser from server ");            
							start(); 
						}
					});                        
									
				}
			});

		});
		
		asyncTest("Index JSON Feed with missing title", function() {
		
			var stories = new ws.models.StorySet({url: "tests/json/russian_index_missing_title.app.json" });
			stories.fetch({
				success: function() {
					equal(stories.length, 8, "Expecting 8 stories in feed (two stories out of 10 should be dropped)");                                                                    
					start();
				}
			});        
			
		});
		


	// Story Level
		module("Story Feeds");
		test("Create story model and load JSON feed", function() {
			var story = new ws.models.Story({url: "tests/json/russian_story.app.json"});
			
			equal(story.get('title'), '', "Expecting title to start off empty");
			equal(story.get('summary'), '', "Expecting summary to start off empty");      
			
			stop();
			story.fetch({
				success: function() {
					start();
					
					var storyId = story.id;
					
					// Check ID
					equal(story.get('bbcws_id'), storyId, "Loaded: Model ID should be set to Topcat ID");
					
					// Check title+summary
					equal(story.get('title'), 'создан самый легкий искусственный материал на свете', "Loaded: Title should be set");                
					equal(story.get('summary'), 'Американские физики и инженеры создали самый легкий на планете искусственный материал.Он состоит из решетки, образованной тончайшими металлическими трубками.', "Loaded: Summary should be set");                

					// Check offline
					story.storeOffline();
					var storedStory = JSON.parse(localStorage.getItem('s'+storyId));
					
					equal(storedStory != null, true, "Offline: Exists in localStorage");
					deepEqual(storedStory, story.toJSON(), "Offline: Stored copy has identical data");
				}
			});

		});



	// Live stats
		module("Livestats");
		asyncTest("Live Feed", function() {
			expect(1);        
			new ws.models.mostPopular().fetch({
				success: function(topStories) {                        
					equal(topStories.length, 5, "Expecting 5 stories in feed");                        
					
					for (var i=0; i<5; i++) {
						
					}
					
					start();
				}
			});
		});

		asyncTest("Mock Feed", function() {
			expect(3);
			ws.config.livestats.mostReadUrl = "tests/json/livestats_top5chart.inc";

			new ws.models.mostPopular().fetch({
				success: function(topStories) {
					
					equal(topStories.length, 5, "Expecting 5 stories in feed");
					
					equal(topStories.at(0).get('title'), "Popular Story 1", "Expecting story title to be 'Popular Story 1'");            
					
					equal(topStories.at(0).get('href'), "http://www.bbc.co.uk/russian/science/2011/11/111117_antarctic_mountains.shtml", "Expecting story 1 href to be to be 'http://www.bbc.co.uk/russian/science/2011/11/111117_antarctic_mountains.shtml'");            

					start();
				}
			});
		});



	/* View Tests
	***********************/

	// Main View
		var stories = new ws.models.StorySet({url: "tests/json/homepage.app.json" });

		module("Views");

		test("Home View", function() {
			var view       = new ws.views.homeView({ collection: stories }),
				outputNode = view.render().el;
			
			//console.log(outputNode.innerHTML);
			
			// Check that the view chooser tabs have one default tab selected
			var tabsActive = $('#ws-view-chooser li.active', outputNode).length;
			equal(tabsActive, 1, "One tab from the Top Stories/Most Recent should be active");

			
		});

	
	
});
	